#include <windows.h>
#include <mmsystem.h>

/* Z80 */
#define FLAG_C      0x01
#define FLAG_N      0x02
#define FLAG_V      0x04
#define FLAG_3      0x08
#define FLAG_H      0x10
#define FLAG_5      0x20
#define FLAG_Z      0x40
#define FLAG_S      0x80

#define FLAGS_SZC   ( FLAG_S | FLAG_Z | FLAG_C )
#define FLAGS_SZV   ( FLAG_S | FLAG_Z | FLAG_V )
#define FLAGS_S53   ( FLAG_S | FLAG_5 | FLAG_3 )
#define FLAGS_SZ    ( FLAG_S | FLAG_Z )
#define FLAGS_53    ( FLAG_5 | FLAG_3 )
#define FLAGS_HC    ( FLAG_H | FLAG_C )

#define RegAF       Cpc.AF.Word
#define RegBC       Cpc.BC.Word
#define RegDE       Cpc.DE.Word
#define RegHL       Cpc.HL.Word

#define Reg_AF      Cpc._AF.Word

#define RegA        Cpc.AF.Byte.High
#define FLAGS       Cpc.AF.Byte.Low
#define RegB        Cpc.BC.Byte.High
#define RegI        Cpc.IR.Byte.High
#define RegR        Cpc.IR.Byte.Low

#define RegPC       Cpc.PC.Word
#define RegSP       Cpc.SP.Word

#define INDIRECT_HL     0x06
#define INDIRECT_HL3    0x30

/* CRTC */
#define TAILLE_Y_CRTC   312
#define TAILLE_VBL      26

/* PSG */
#define SND_RATE        125000
#define SND_BUFSIZE     512
#define SND_NB_BUFFERS  16

/* UPD */
#define STATUS_CB       0x10
#define STATUS_EXE      0x20
#define STATUS_DIO      0x40
#define STATUS_RQM      0x80

#define ST0_NR 0x08
#define ST0_EC 0x10
#define ST0_SE 0x20
#define ST0_IC1 0x40
#define ST0_IC2 0x80

#define ST1_MA 0x01
#define ST1_ND 0x04
#define ST1_OR 0x10
#define ST1_DE 0x20
#define ST1_EN 0x80

#define ST2_MD 0x01
#define ST2_BC 0x02
#define ST2_WC 0x10
#define ST2_CRC 0x20
#define ST2_CM 0x40

#define READ_TRACK 0xFF // Masque spcial pour fonction "ReadTrack"

#define ST3_T0 0x10
#define ST3_RY 0x20

#define TPS_MOTOR_ON    400000
#define TPS_DATA_READ   34
#define TPS_OVER_RUN    68

#define MAX_DSK         4
#define MAX_TRACKS      99
#define MAX_SECTS       29

/* GENERAL */
#define X_ECR_MAX       768
#define Y_ECR_MAX       574

#define TIME_OUT_CRTC   2000000
#define TIME_OUT_TURBO  1100000

#define TITRE "PC-CPC - Version 0.1at-Fast/Light (Build " __DATE__ ")"

#define TEXT_COLOR RGB( 0xFF, 0xFF, 0xFF )
#define BK_COLOR RGB( 0x00, 0x00, 0x00 )

/* Z80 */
typedef union {
	USHORT Word;
	struct {
		BYTE Low, High;
	} Byte;
} Registre;

typedef int ( * pfct )( void );

/* PSG */
typedef struct {
	int Cnt, Step, Out;
} StTone;

/* UPD */
typedef struct {
	BYTE C, H, R, N, ST1, ST2;
	SHORT SectSize;
} CPCEMUSect;

typedef struct {
	char ID[ 0x10 ];
	BYTE Track, Head;
	SHORT Unused;
	BYTE SectSize, NbSect, Gap3, OctRemp;
	CPCEMUSect Sect[ MAX_SECTS ];
} CPCEMUTrack;

typedef int ( * pFctExec )( int, int, int );

enum { SENS_NONE = 0, SENS_RD, SENS_WR };

/* Z80 */
extern pfct TabInstr[ 256 ], TabInstrCB[ 256 ], TabInstrED[ 256 ];

static BYTE * Regs[ 16 ], * RegsDD[ 16 ], * RegsFD[ 16 ], ** PtrRegs;
static int LastInstr;
static int SupIrqWaitState;
static int memPtr;

static BYTE TabSZYX[ 256 ] = {
	0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28,
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28,
	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28, 0x28,
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88,
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88,
	0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8,
	0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8,
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88,
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88,
	0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8,
	0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA0, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8, 0xA8
};

static BYTE TabSZYXP[ 256 ] = {
	0x44, 0x00, 0x00, 0x04, 0x00, 0x04, 0x04, 0x00, 0x08, 0x0C, 0x0C, 0x08, 0x0C, 0x08, 0x08, 0x0C,
	0x00, 0x04, 0x04, 0x00, 0x04, 0x00, 0x00, 0x04, 0x0C, 0x08, 0x08, 0x0C, 0x08, 0x0C, 0x0C, 0x08,
	0x20, 0x24, 0x24, 0x20, 0x24, 0x20, 0x20, 0x24, 0x2C, 0x28, 0x28, 0x2C, 0x28, 0x2C, 0x2C, 0x28,
	0x24, 0x20, 0x20, 0x24, 0x20, 0x24, 0x24, 0x20, 0x28, 0x2C, 0x2C, 0x28, 0x2C, 0x28, 0x28, 0x2C,
	0x00, 0x04, 0x04, 0x00, 0x04, 0x00, 0x00, 0x04, 0x0C, 0x08, 0x08, 0x0C, 0x08, 0x0C, 0x0C, 0x08,
	0x04, 0x00, 0x00, 0x04, 0x00, 0x04, 0x04, 0x00, 0x08, 0x0C, 0x0C, 0x08, 0x0C, 0x08, 0x08, 0x0C,
	0x24, 0x20, 0x20, 0x24, 0x20, 0x24, 0x24, 0x20, 0x28, 0x2C, 0x2C, 0x28, 0x2C, 0x28, 0x28, 0x2C,
	0x20, 0x24, 0x24, 0x20, 0x24, 0x20, 0x20, 0x24, 0x2C, 0x28, 0x28, 0x2C, 0x28, 0x2C, 0x2C, 0x28,
	0x80, 0x84, 0x84, 0x80, 0x84, 0x80, 0x80, 0x84, 0x8C, 0x88, 0x88, 0x8C, 0x88, 0x8C, 0x8C, 0x88,
	0x84, 0x80, 0x80, 0x84, 0x80, 0x84, 0x84, 0x80, 0x88, 0x8C, 0x8C, 0x88, 0x8C, 0x88, 0x88, 0x8C,
	0xA4, 0xA0, 0xA0, 0xA4, 0xA0, 0xA4, 0xA4, 0xA0, 0xA8, 0xAC, 0xAC, 0xA8, 0xAC, 0xA8, 0xA8, 0xAC,
	0xA0, 0xA4, 0xA4, 0xA0, 0xA4, 0xA0, 0xA0, 0xA4, 0xAC, 0xA8, 0xA8, 0xAC, 0xA8, 0xAC, 0xAC, 0xA8,
	0x84, 0x80, 0x80, 0x84, 0x80, 0x84, 0x84, 0x80, 0x88, 0x8C, 0x8C, 0x88, 0x8C, 0x88, 0x88, 0x8C,
	0x80, 0x84, 0x84, 0x80, 0x84, 0x80, 0x80, 0x84, 0x8C, 0x88, 0x88, 0x8C, 0x88, 0x8C, 0x8C, 0x88,
	0xA0, 0xA4, 0xA4, 0xA0, 0xA4, 0xA0, 0xA0, 0xA4, 0xAC, 0xA8, 0xA8, 0xAC, 0xA8, 0xAC, 0xAC, 0xA8,
	0xA4, 0xA0, 0xA0, 0xA4, 0xA0, 0xA4, 0xA4, 0xA0, 0xA8, 0xAC, 0xAC, 0xA8, 0xAC, 0xA8, 0xA8, 0xAC
};

static int TabV[ 4 ] = { 0, FLAG_V, FLAG_V, 0, };

/* CRTC */
static int xEcr, yEcr;
static int LigneCRTC, MaCRTC, MaHi, HDelay, VDelay, VswCount, HDT, VDT, HS, MR, VT, VSync, OldVSync, LastMode;
static int VCharCount, VHCount;

/* PPI */
static int clav[ 16 ], MaskPortC;
static int modePSG;

/* PSG */
static StTone TabTone[ 3 ];
static int stepE, stepN, CntEnv, EnvStep, CntNoise, Rng, OutNoise;
static int Hold, Alternate, Attack, Holding;
static int IsSound, Exit;

/* VGA */
static BYTE MemCPC[ 0x20000 ], * TabPOKE[ 4 ], * TabPEEK[ 4 ];
static int TabCoul[ 32 ];
static int MemoMode, MemoColor;

/* VGA - ROM */
static int ROMINF[ 0x1000 ] = {
	0xED7F8901, 0x0591C349, 0xC3B98AC3, 0xC9C5B984, 0xC3BA1DC3, 0xC9D5BA17, 0xC3B9C7C3, 0x00E9B9B9, 0xC3BAC6C3, 0x0000B9C1, 0x00BA35C3, 0xFBD949ED, 0x2B21D9F3, 0x08187100, 0xC9B941C3, 0x00000000,
	0xE818D1CB, 0x2D004021, 0xFB20777E, 0x3032C73E, 0x03A62100, 0x01B90011, 0xB0ED01E4, 0xB8D93AF3, 0xB8D75BED, 0x2D21CD06, 0x230036B8, 0x0E47FB10, 0x47C0A9FF, 0x7CC9575F, 0x042079B5, 0xC006217D,
	0x32B8D632, 0xD722B8D9, 0xABFF21B8, 0x01004011, 0x0031B0FF, 0xB8D7DFC0, 0x5BEDF3C7, 0xB42AB8B6, 0xF3C9FBB8, 0xB8B832AF, 0xB8B653ED, 0xFBB8B422, 0xB8B421C9, 0xFC282334, 0x78EDF506, 0x2A08301F,
	0xB77CB8B9, 0x2A0153C4, 0xB77CB8BB, 0xCD0153C4, 0xBF2120D7, 0x36C035B8, 0xBDF4CD06, 0x7CB8BD2A, 0x3121C8B7, 0xC9C6CBB8, 0x2B00362B, 0xB7B82E3A, 0x2D220C20, 0xB82F22B8, 0xCBB83121, 0x5BEDC9F6,
	0x2F22B82F, 0x2373EBB8, 0x73EDC972, 0xB431B832, 0xC5D5E5B8, 0xCBB83121, 0xCB1E2876, 0xB82D2AFE, 0x0E28B77C, 0xED56235E, 0x23B82D53, 0xF30209CD, 0x3121EB18, 0x2846CBB8, 0x37003610, 0x0189CD08,
	0x312108B7, 0x20B77EB8, 0xC10036D2, 0x7BEDE1D1, 0x5EC9B832, 0xB7237E23, 0x5701E2CA, 0x01E2CDD5, 0xE5F018E1, 0xD2CD2323, 0xB911E101, 0x0379C3B8, 0xC3B8B911, 0x23E50388, 0x01D2CD23, 0xB8BB11E1,
	0x110379C3, 0x88C3B8BB, 0xB8BD2A03, 0x5EC8B77C, 0x4E235623, 0xB1784623, 0x780B1628, 0xD50E20B1, 0x23E52323, 0xE101E2CD, 0x2B4E2B46, 0x712B70D1, 0xE5D918EB, 0x73F32323, 0x71237223, 0x11E17023,
	0x79C3B8BD, 0xB8BD1103, 0xD00388CD, 0x235E23EB, 0x23F3C956, 0x23003623, 0x23732370, 0x23712372, 0x2323C9FB, 0xFA347EF3, 0x20B70201, 0x2B7E2315, 0x022EF2B7, 0x08113008, 0x00E8F287, 0x18232335,
	0x38083521, 0xC908FB01, 0x3D7EFB08, 0x1BCDE5F8, 0xC835E102, 0x34020DF2, 0x232323C9, 0xD21F237E, 0x235EB9C1, 0x21E9EB56, 0xC1220000, 0x47E5C9B8, 0xEBB8C311, 0x2B562B2B, 0x28B77A5E, 0x13131307,
	0xEF30B81A, 0x7E231BD1, 0x2B721B12, 0x0873127E, 0x08FB0138, 0xC02AF3C9, 0x28B77CB8, 0x235EE517, 0x3A232356, 0x30BEB8C2, 0x327EF50A, 0x53EDB8C2, 0xE1F1B8C0, 0xC232C9FB, 0x352323B8, 0x2EF2F3C8,
	0xC9FB3402, 0x11028DCD, 0x88C3B8C0, 0x36232303, 0xC92B2BC0, 0xCBB8C221, 0xC221C9EE, 0xC9AECBB8, 0xD35BEDE5, 0xB8D322B8, 0x23722373, 0xE1702371, 0xB8C311C9, 0xCD001001, 0x2BEBBAA1, 0xD32AFECB,
	0x10187DB8, 0x4E2323E5, 0xF1CD4623, 0xEBD8D102, 0x6F66237E, 0x0EED20B4, 0x7ECD0CFF, 0x03E6F5BA, 0x02F1CC47, 0xF1061CDC, 0x79EE3087, 0xE93810FE, 0xC00421C9, 0x0428B778, 0xFF0E6960, 0xC5BA79CD,
	0x2356235E, 0x011718EB, 0xBE0AB8C3, 0x03230820, 0xEBF73087, 0x237E0C18, 0x13FB3087, 0xB77E1313, 0xC3C1E520, 0x0F0EBA87, 0x0D0330CD, 0xC90328F2, 0xB9B8D93A, 0x10FE79C8, 0xBA79CDD0, 0xE6C0003A,
	0x22203D03, 0x06CD37C5, 0xD51A30C0, 0xDA21EB23, 0xD64BEDB8, 0x090006B8, 0x72237309, 0x19FFFC21, 0x2B02A0CD, 0x87C3C1D1, 0x23BB7EBA, 0x03202B7E, 0xB7C837BA, 0x18676EC8, 0xCDF3EBF0, 0x06380369,
	0x13722373, 0xC9FB12AF, 0x69CDF3EB, 0x1A063003, 0x1A231377, 0xF3C9FB77, 0xB8D521D9, 0xC0F67756, 0xD97A79ED, 0x5FC3C9FB, 0xBA66C3BA, 0xC3BA51C3, 0x70C3BA58, 0xBA79C3BA, 0xC3BA9DC3, 0x87C3BA7E,
	0xBAA1C3BA, 0x3ABAA7C3, 0xC8B7B8C1, 0x0618F3E5, 0x36B8BF21, 0xC02AC901, 0x28B77CB8, 0x23232307, 0xBEB8C23A, 0xF3C9FBE1, 0xD9333808, 0x08FB3779, 0x91CBF5F3, 0xB1CD49ED, 0x4F08B700, 0x313A7F06,
	0x1428B7B8, 0x79B972FA, 0xCBF50CE6, 0x0ACDD991, 0x79E1D901, 0x4FB4F3E6, 0xF1D949ED, 0xE108C9FB, 0xEDD1CBF5, 0x003BCD49, 0xE5F3CF18, 0x0618D1D9, 0x5EE1D9F3, 0x7A085623, 0xB2CBBACB, 0x07070707,
	0xA90CE6A9, 0xB9B0CDC5, 0x7908D9F3, 0xCB03E6C1, 0xB181CB89, 0x4FD50118, 0x08B749ED, 0xF3C9FBD9, 0xD9E57908, 0xF31518D1, 0x18E1D9E5, 0xE1D9F309, 0x2356235E, 0x235EEBE5, 0x7E082356, 0xBE30FCFE,
	0x79EDDF06, 0x46B8D621, 0xE5FDC577, 0x0F3010FE, 0x6FDAC687, 0x6795B8CE, 0x6F66237E, 0x06E1FDE5, 0xD7CB797F, 0xB0CD9FCB, 0xF3E1FDB9, 0xC15908D9, 0xEDDF0678, 0xB8D63279, 0x187B7F06, 0xD9E5F390,
	0xF30818D1, 0x235EE1D9, 0x08E52356, 0xCBFACB7A, 0x07C0E6F2, 0xB8D92107, 0xF3A51886, 0x235EE1D9, 0xED91CB56, 0x4653ED49, 0xCDFBD9BA, 0xD9F3BA45, 0x49EDD1CB, 0xF3C9FBD9, 0x91CB79D9, 0xD9F31318,
	0x18D1CB79, 0x79D9F30C, 0x051899CB, 0xCB79D9F3, 0xD949EDD9, 0xD9F3C9FB, 0xA90CE6A9, 0xCDF2184F, 0x0F18BA5F, 0x3ABA79CD, 0x012AC000, 0xCD78F5C0, 0xE5F1BA70, 0xEDDF06F3, 0xB8D62149, 0x47487146,
	0x3AC9E1FB, 0xCDC9B8D6, 0xB0EDBAAD, 0xBAADCDC9, 0xF3C9B8ED, 0xCBC5E1D9, 0xEDD9CBD1, 0xBAC2CD49, 0xEDC1D9F3, 0xC9FBD949, 0xC9FBD9E5, 0xCB59D9F3, 0xEDDBCBD3, 0xD97ED959, 0xFBD949ED, 0xF679D9C9,
	0xDD79ED0C, 0x49ED007E, 0xC744C9D9, 0xC7C7C7C7, 0x8201F3C7, 0x0149EDF7, 0x49EDF400, 0xEDF60001, 0xEF7F0149, 0xF50649ED, 0x10E678ED, 0x2005D521, 0x05E52103, 0xEDBC0F01, 0x047E2B49, 0x0D0579ED,
	0x1805B7F2, 0x2E283F20, 0x1900268E, 0x0007001E, 0xC0003000, 0x2E283F00, 0x19061F8E, 0x0007001B, 0xC0003000, 0x06771100, 0x18000021, 0xC0003132, 0x1FE9CDE5, 0xF8FF01F3, 0x5CCD49ED, 0xC5D5E100,
	0x1B98CDE5, 0xCD1084CD, 0x5FCD0AD0, 0x1ECDE1BA, 0x38D1C100, 0x1148EB07, 0x031806F9, 0xF3073711, 0x01D956ED, 0x49EDDF00, 0xEDF8FF01, 0x7FC00149, 0x7E0149ED, 0x79EDAFFA, 0x11B10021, 0xF901B101,
	0xB0ED7707, 0xED7F8901, 0x08AFD949, 0xE5C00031, 0x44CDD5C5, 0x08BDCD00, 0xCD1B5CCD, 0xBFCD1FE9, 0x1074CD0A, 0xCD15A8CD, 0xE0CD24BC, 0xCDE1FB07, 0xE1C1001E, 0x210077C3, 0x70CD0202, 0x0723CD11,
	0x2106FCCD, 0x74180688, 0x38323120, 0x694D204B, 0x636F7263, 0x75706D6F, 0x20726574, 0x33662820, 0x04021F29, 0x79706F43, 0x68676972, 0x04021F74, 0x383931A4, 0x6D412035, 0x61727473, 0x6F432064,
	0x6D75736E, 0x45207265, 0x7463656C, 0x696E6F72, 0x70207363, 0x0C1F636C, 0x646E6105, 0x636F4C20, 0x746F6D6F, 0x20657669, 0x74666F53, 0x65726177, 0x64744C20, 0x07011F2E, 0x07052100, 0xC8B7237E,
	0x1813FECD, 0x2A2A2AF7, 0x4F525020, 0x4D415247, 0x414F4C20, 0x41462044, 0x44454C49, 0x2A2A2A20, 0x06000A0D, 0x2F78EDF5, 0x210F0EE6, 0x473C0738, 0x20B7237E, 0xC9F910FB, 0x6F6E7241, 0x4100646C,
	0x7274736D, 0x4F006461, 0x6E6F6972, 0x68635300, 0x6469656E, 0x41007265, 0x53006177, 0x76616C6F, 0x5300786F, 0x68736961, 0x7254006F, 0x706D7569, 0x73490068, 0x03FE0070, 0xCBD9F3D0, 0xB181CB89,
	0xFB49ED4F, 0x21E5C9D9, 0x04180000, 0x000121E5, 0x01EBC5D5, 0xAACD7F10, 0x000E2307, 0x1907AACD, 0x10FE790C, 0xD1C1F620, 0x49EDC9E1, 0xF61FE67E, 0xC979ED40, 0xF506C5F5, 0x301F78ED, 0xC9F1C1FB,
	0xE60F0FC5, 0x1F7C4F30, 0x01B103E6, 0x49EDBC0C, 0x0579ED04, 0x0449ED0C, 0x1F7D1F7C, 0xC9C179ED, 0x1107F721, 0x1501B804, 0x21B0ED00, 0xB4C307F1, 0xBDF1030A, 0x0A0835C3, 0x5CA15EA0, 0x23A37BA2,
	0x7CAB40A6, 0x7EAD7DAC, 0x5BAF5DAE, 0x4F3C87E7, 0x04110006, 0xDC2AFEB8, 0xC5C9BAA1, 0xB80421E5, 0x28050446, 0x23BE230A, 0xFE7EF820, 0xCD0328FF, 0xC1E1BDF1, 0x003201C9, 0x300858CD, 0x0DF91007,
	0xC9B7F620, 0xE6EF06C5, 0xF679ED7F, 0x79EDF380, 0xEDFB7FE6, 0xC937C179, 0xF5064FC5, 0x171778ED, 0xF3C9C179, 0x79EDF406, 0x78EDF606, 0x79EDC0F6, 0x79ED3FE6, 0x49EDF406, 0xF64FF606, 0xED79ED80,
	0x01C9FB49, 0x49EDF40E, 0x78EDF606, 0xF64F30E6, 0xED79EDC0, 0x923E0449, 0xCBC579ED, 0xEDF606F1, 0xEDF40649, 0xA0774678, 0x1323122F, 0x0FE6790C, 0xE9200AFE, 0xED823EC1, 0x49ED0579, 0x08DE21C9,
	0x01BB0011, 0xCCCDCBCF, 0x20EF0108, 0xED131279, 0x072F03A0, 0xB680E607, 0x10231312, 0x1B5CC9EF, 0x1BBF1B98, 0x1BFA1BC5, 0x1CB31C46, 0x1CDB1C04, 0x1E451CE1, 0x1DE51D38, 0x1EC41ED8, 0x1EC91EDD,
	0x1ECE1EE2, 0x1E2F1E34, 0x1DF21DF6, 0x1E0B1DFA, 0x10741E19, 0x14591084, 0x13FE1452, 0x13AC1335, 0x120813A8, 0x154F1252, 0x1165115A, 0x117C1170, 0x12971286, 0x127E1276, 0x126511CA, 0x12A61265,
	0x12AB12BA, 0x12C612C0, 0x1388137B, 0x12F212D4, 0x132B12FE, 0x10E414D4, 0x15A81103, 0x15FE15D7, 0x160615FB, 0x161C160E, 0x16EA16A5, 0x172D1717, 0x17671736, 0x176E1775, 0x1783177A, 0x17971780,
	0x17A91794, 0x194017A6, 0x0AD00ABF, 0x0B3C0B37, 0x0AE90B56, 0x0B170B0C, 0x0B6A0B5D, 0x0C050BAF, 0x0C1F0C11, 0x0C8E0C39, 0x0CF20CA7, 0x0CF70D1A, 0x0CEA0D1F, 0x0DB90CEE, 0x0DE50DBD, 0x0E440E00,
	0x0F2A0EF9, 0x0C740C55, 0x0F9B0F93, 0x24CE24BC, 0x2BBB24E1, 0x2BC12BBF, 0x255024E5, 0x25A02557, 0x26072618, 0x24FE2603, 0x2599257F, 0x265325C6, 0x29AF2692, 0x29C129A6, 0x21141FE9, 0x21EB21CE,
	0x205021AC, 0x2495206B, 0x24A6249A, 0x005C24AB, 0x03300326, 0x02B102A0, 0x016A0163, 0x01760170, 0x0183017D, 0x01C501B3, 0x01E201D2, 0x02840227, 0x02190255, 0x02940276, 0x028D029A, 0x00A30099,
	0x061C05ED, 0x077607B4, 0x078607C0, 0x07E0078C, 0x0858081B, 0x08630844, 0x1D3C08BD, 0x14601BFE, 0x19D515EC, 0x17AC17B0, 0x19D9162A, 0x080C0B45, 0x2C020397, 0x2F9F2F91, 0x2FD92FC8, 0x30143001,
	0x305F3055, 0x34A230C6, 0x349E3159, 0x36043577, 0x36DF3188, 0x37273731, 0x2F733345, 0x32AF32AC, 0x31B131B6, 0x3353322F, 0x33C83349, 0x2FD133D8, 0x31433136, 0x2300064E, 0x2356235E, 0x11C9B0ED,
	0x86CD1052, 0x32C03E07, 0xD0CDB7C6, 0x0B12C30A, 0x0C55CDAF, 0xCD0ADD21, 0xD8C30AB4, 0xBDE5090C, 0xC30C8AC3, 0x17C30C71, 0xFE03E60B, 0xCDF5D003, 0xCDD10D55, 0xCDF510B3, 0x7AE515CE, 0xCD0B31CD,
	0xCDE1BDEB, 0xCDF115AE, 0x221810D1, 0xFEB7C33A, 0x013EC901, 0xCD0B31CD, 0x00210D55, 0x0B37CD00, 0x2EB7C52A, 0x011E5400, 0x753FFF01, 0x42C3B0ED, 0xB7C3320D, 0x3A0776C3, 0x0318B7C6, 0xCDB7C42A,
	0xC0C30B45, 0x32C0E607, 0x7CF5B7C6, 0xCB6707E6, 0xB7C42285, 0xC42AC9F1, 0xB7C63AB7, 0x0B0CCDC9, 0xD8131801, 0x06C82706, 0xCDD5C94F, 0x04060B0C, 0x02060538, 0xC5050128, 0x6200165C, 0x295D54D5,
	0x29291929, 0x19D12929, 0x5BEDFD10, 0x7C19B7C4, 0x3A6707E6, 0x6784B7C6, 0x7BC9D1C1, 0x87873C95, 0x947A5F87, 0x6ACD573C, 0x1082AF0B, 0xD5C957FD, 0x00C721EB, 0x7D52EDB7, 0x878707E6, 0xE67D4F87,
	0x5D546FF8, 0x29192929, 0xF6CDC5D1, 0x28A3780B, 0x3D09CB05, 0x61E3FB20, 0x0F78E34D, 0x1BCB3ACB, 0x19F9380F, 0xB7C45BED, 0x07E67C19, 0xB7C63A67, 0xD1678184, 0x0CCDC94A, 0x01AA010B, 0x038801D8,
	0x078001C8, 0x24C02CC9, 0xC007E67C, 0x6708D67C, 0xB72D7DC9, 0xE6257CC0, 0xC67CC007, 0x7CC96708, 0xE66708C6, 0xD67CC038, 0xC67D6740, 0x24D06F50, 0xC007E67C, 0x6708D67C, 0x08D67CC9, 0xFE38E667,
	0xC67CC038, 0xD67D6740, 0x7CD06F50, 0xC007E625, 0x6708C67C, 0x2103E6C9, 0x0C280C74, 0x7A2E02FE, 0x7F2E0638, 0x852E0228, 0xC732C33E, 0xB7C822B7, 0xB7C7C3C9, 0xAEA1AE78, 0xA178C977, 0x79C977AE,
	0x77A6B02F, 0xB6A178C9, 0xC37EC977, 0xD5C50CB2, 0x5F0CC8CD, 0x060BF6CD, 0x170BCB08, 0x023809CB, 0xF51003CB, 0xC5C9C1D1, 0x0BF6CDF5, 0x0CB2CDF1, 0x11D5C9C1, 0xCB0F0008, 0x3809CB12, 0x1D1ACB02,
	0xCD7AF420, 0xC9D10CC8, 0x0B0CCD57, 0x0F0FD07A, 0x9F0F00CE, 0xC9AA06E6, 0x11105221, 0x2201B7D4, 0xAFB0ED00, 0x21B7F632, 0xD2220A0A, 0xD22AC9B7, 0x0FE6C9B7, 0xAF01183C, 0x10CD785F, 0xCD79460D,
	0x7B4E0D10, 0x710D35CD, 0xFF3E70EB, 0xC9B7F732, 0x99C61FE6, 0x950DCE6F, 0x0FE6C967, 0xAF01183C, 0x1A0D35CD, 0x0D2ACD5E, 0x000E7B41, 0xBE0D9921, 0x180C23C8, 0x00165FFA, 0x19B7E521, 0xFFEF21EB,
	0xF921C919, 0x70CDE5B7, 0x0D73CD01, 0x060D6111, 0x63C3E181, 0xB7F92101, 0xCD0170CD, 0x86C30D87, 0xB7F82107, 0x2B0C2835, 0xCDC8B77E, 0x8CCD0D87, 0xCD0F1807, 0xF8320D87, 0x078CCDB7, 0x7EB7F621,
	0x32AF772F, 0x11C9B7F7, 0xF63AB7E5, 0xD33AB7B7, 0xD411C8B7, 0xB7D23AB7, 0x150414C9, 0x0C1D181C, 0x06160D05, 0x1F001E17, 0x120F070E, 0x191A1302, 0x0B030A1B, 0x10090801, 0x9BCD4F11, 0xCD7AE50B,
	0x09300EEE, 0x05CD7142, 0x18FA100C, 0x71D5C510, 0x4A082815, 0x5D540006, 0xD1B0ED13, 0x1FCDE1C1, 0xD9201D0C, 0x4FA978C9, 0x160B6ACD, 0x7EC5E508, 0x05CD77A9, 0xC1F8100C, 0x0C1FCDE1, 0xC9EE2015,
	0xD011C54F, 0xCD3006FF, 0xCDC10E2A, 0xB77807B4, 0xB0110D20, 0x0E3DCDFF, 0x06000011, 0x110B1820, 0x3DCD0050, 0xFFB0110E, 0xC42A2006, 0xE67C19B7, 0xC63A6707, 0x506784B7, 0xBDC3081E, 0xB7C42A0D,
	0x0B37C319, 0x28B778F5, 0x9BCDE530, 0xCD2CE30B, 0x7B4A0B6A, 0x284708D6, 0xB4CDD117, 0xD5E5C507, 0xE10EAACD, 0xEB0C1FCD, 0x0C1FCDE1, 0xD5EE10C1, 0x081E51E1, 0xBDC34FF1, 0xCDD5E50D, 0x7B4A0B9B,
	0xD14708D6, 0xC5E928E3, 0xCD1C546B, 0xCDEB0B6A, 0xCDC10B6A, 0x39CD07B4, 0xCDEBE50C, 0xC5E50C39, 0xC10EAACD, 0xEE10E1D1, 0x0006C618, 0x380EECCD, 0x0EECCD16, 0xAFC52530, 0xB0ED4F95, 0x813C2FC1,
	0x08D67C4F, 0xCD141867, 0x12380EEC, 0x4F93AFC5, 0x2FC1B0ED, 0x7A4F813C, 0xED5708D6, 0x7E41C9B0, 0x0C05CD12, 0x0C05CDEB, 0xC9F410EB, 0x853DEB79, 0x07E67CD0, 0x37C007EE, 0x0B0CCDC9, 0x06280D38,
	0xED000801, 0x8801C9B0, 0x01031802, 0x083E04AA, 0x606EE5F5, 0x3005CBAF, 0x09CBB101, 0x1312F730, 0xE144F210, 0x203DF123, 0xCD4FC9E7, 0x0CCD0B6A, 0x3808060B, 0x7E0B2836, 0x13122FA9, 0x100C1FCD,
	0xE5C5C9F6, 0x0F5ACDD5, 0xCD0C05CD, 0xD17B0F5A, 0xCDE11312, 0x10C10C1F, 0x8816C9E9, 0xA97E0406, 0x370120A2, 0x0ACB13CB, 0xC5C9F410, 0x0406D5E5, 0xAAE6A97E, 0xCB370120, 0xE6A97E13, 0x37012055,
	0x05CD13CB, 0x7BE9100C, 0xE11312D1, 0xC10C1FCD, 0xCDC9D910, 0xC2CD0FAD, 0xCD06180F, 0x16CD0FAD, 0xB8022A10, 0xB6A3327D, 0xB6B3327C, 0xA32AE5C9, 0xB6A332B6, 0x67B6B33A, 0xB332FF3E, 0xB80222B6,
	0xCD37C9E1, 0x00CB103B, 0x1D133079, 0x28150320, 0x3809CB2C, 0x2878CB28, 0x00CBB124, 0x201DED18, 0x0D281503, 0x093809CB, 0x052078CB, 0x1800CBB1, 0x3A4FC5ED, 0x3A47B6A4, 0x18B7B6B4, 0x3A4FC507,
	0xAF47B6A3, 0xC1BDE8CC, 0x05C479CB, 0x20B37A0C, 0xB33278B5, 0xCDB7C9B6, 0x00CB103B, 0x38B6A33A, 0xB6B43A09, 0x3A0920B7, 0x47C5B6A4, 0xC1BDE8CD, 0x1D0C39CD, 0x2015E420, 0xE5D618E1, 0x6B620230,
	0xCD42EDB7, 0x2C241939, 0x0BAFCDE3, 0x47B6B33A, 0x0404C9D1, 0x0B0C130A, 0x060D1514, 0x12071F1E, 0x04170419, 0x0C130A04, 0x0D15140B, 0x071F1E06, 0x070A1912, 0xAF1084CD, 0x21B73532, 0x39CD0001,
	0x109FC311, 0xCD108D21, 0x64C30AB4, 0xBDCD0F14, 0xC3125FC3, 0x4BC3125F, 0x13BEC313, 0x3E140AC3, 0xB6B61108, 0x01B72621, 0xB0ED000E, 0x32F5203D, 0x3AC9B6B5, 0x064FB6B5, 0xCD3D7808, 0xD0CD10E4,
	0x12C0CDBD, 0xCDB73032, 0x2F3212BA, 0x79EA10B7, 0x08064FC9, 0xE4CD3D78, 0x2F2AC510, 0x1139CDB7, 0x79F110C1, 0xB52107E6, 0xC5C8BEB6, 0x47774ED5, 0x1126CD79, 0x78111ECD, 0xEB1126CD, 0x79111ECD,
	0x3AC9C1D1, 0x79F5B6B5, 0x7810E4CD, 0xCDB6B532, 0x79D51126, 0xE11126CD, 0xF1111ECD, 0x01C5C618, 0xB0ED000E, 0x07E6C9C1, 0x8783875F, 0xB6C68783, 0x93B6CE5F, 0xB7262157, 0x833EEBC9, 0x7AB72E32,
	0x7B12ABCD, 0xAF12A6CD, 0xCD13A8CD, 0x0021137B, 0x7F7F1100, 0xC31208CD, 0x213D1459, 0x2A86B72A, 0x1867B726, 0x29213D0E, 0x262A86B7, 0x03186FB7, 0xCD1186CD, 0x2622BDD0, 0xBDCDC3B7, 0xCDB7262A,
	0x2D3A1193, 0x293AC9B7, 0x6F853DB7, 0x3DB72A3A, 0x3AC96784, 0x2F95B729, 0x3A6F3C3C, 0x2F94B72A, 0xC9673C3C, 0x2ABDD0CD, 0xD6CDB726, 0xB7262211, 0x2D21E5D8, 0x3C8778B7, 0x52CD7786, 0xB7303A12,
	0x0E44DCF5, 0x0E00D4F1, 0x86CDC9E1, 0x11D6CD11, 0x1193CDF5, 0x2C3AC9F1, 0xE2F2BCB7, 0xB72A3A11, 0x2A3A2C67, 0xFABC3DB7, 0x2C3A11EF, 0x3A2D67B7, 0xBD3DB729, 0x3A1202F2, 0x37BDB72B, 0xFF066FF0,
	0x6F3CC9B7, 0xC9B70006, 0x7C0B5DCD, 0x671240CD, 0x1240CD7A, 0x0230BC57, 0xCD7D6754, 0x7B6F1249, 0x5F1249CD, 0x5D0230BD, 0xB729226F, 0xB72B53ED, 0x0620B57C, 0x0220A87A, 0x2832A97B, 0x1173C3B7,
	0x1245F2B7, 0x78D8B8AF, 0x4EF2B7C9, 0xD8B9AF12, 0x292AC979, 0x2B5BEDB7, 0xB7283AB7, 0x3AC9FFC6, 0x03E6B72E, 0xE5D5C5C0, 0xED11A7CD, 0xCDB72F4B, 0xD1E10DE5, 0x3EF5C9C1, 0x1288CDFD, 0x3EF5C9F1,
	0x1299CD02, 0xFE3EC9F1, 0xBDD0CDF5, 0x2E21E5F1, 0xE177A6B7, 0x3EBDCDC3, 0xD0CDF501, 0x21E5F1BD, 0x77B6B72E, 0x2F21C9E1, 0x210318B7, 0xCDF5B730, 0xCDF1BDD0, 0xC3770C8E, 0x2F3ABDCD, 0x0CA7C3B7,
	0xC3B7303A, 0xD0CD0CA7, 0xB72F2ABD, 0x226F657C, 0xE318B72F, 0x2BCD5FD5, 0x57093013, 0x303F927B, 0x03185F03, 0xF5380021, 0x29EB0016, 0xF1192929, 0xCDEBC9D1, 0xEBD012D4, 0xED000801, 0x7AE5C9B0,
	0x200016B7, 0x4BD51519, 0xD4CD79EB, 0x20AA7C12, 0x28AB7D04, 0xF8CDC508, 0x200CC112, 0x2BCDD1EC, 0x3453ED13, 0x53EDD1B7, 0x2AC9B736, 0x0F7CB734, 0xB7362A7D, 0x2E3A47C9, 0xC5D807B7, 0x2411A4CD,
	0x25B72622, 0xBDD3CDF1, 0xE5BDCDC3, 0x1112D4CD, 0xCDD5B738, 0xE1D10EF9, 0x0E0B6ACD, 0xC5E5C508, 0xCD4EEBD5, 0x05CD1377, 0xC113D10C, 0xCDE1F110, 0x0DC10C1F, 0x2AC9E720, 0x21E9B731, 0x28B71392,
	0x13A02103, 0xC9B73122, 0x11B7312A, 0x7C19EC6E, 0x2F2AC9B5, 0xA42F79B7, 0xB0A57947, 0x0318FF0E, 0x47B72F3A, 0x0C74C3EB, 0xC9B73332, 0xCDC5D5E5, 0xD6CD11A4, 0xCDCDF5BD, 0xD1C1F1BD, 0x303AC9E1,
	0xB73811B7, 0x2ACDD5E5, 0x06D5D10F, 0x122F1A08, 0xCDFA1013, 0xE1D113E1, 0x3AC00130, 0x2ACDB72F, 0x79000E0F, 0x1112D4CD, 0x0806B738, 0x0920BE1A, 0xF8101323, 0x378FFE79, 0xE7200CC9, 0xC5F5C9AF,
	0xD9CDE5D5, 0xC1D1E1BD, 0x3A4FC9F1, 0x79B7B733, 0x211940C2, 0x7846B758, 0x31300AFE, 0x790620B7, 0x35D220FE, 0x58700413, 0x71190016, 0x5FB7593A, 0x19B76321, 0xE67E1919, 0x3AD0B80F, 0x07A6B72E,
	0x5E230B38, 0x59215623, 0x16CD79B7, 0x5832AF00, 0x813EC9B7, 0x181299CD, 0xCD7E3EF4, 0xED181288, 0xC9B72E3A, 0xB75832AF, 0x11147421, 0x6001B763, 0xC9B0ED00, 0x81151380, 0x97801335, 0x12868012,
	0x810AE981, 0x59001940, 0x14E18014, 0x80151980, 0x2380151E, 0x15288015, 0x80154F80, 0xAB81153F, 0x12A68112, 0x80155E80, 0x8F801599, 0x15788015, 0x80156580, 0xEC811452, 0x0C558114, 0x8912C680,
	0x0184150D, 0x14EB0015, 0x8214F183, 0x398014FA, 0x15478215, 0xC9B76321, 0x5A000087, 0x140B0000, 0x21E5DD00, 0x14CD14D8, 0xC9E1DD21, 0x7BC39F0F, 0x237E2313, 0xC34E2346, 0x46230CF2, 0xF7C34E23,
	0x2356230C, 0x235E237E, 0x08C3676E, 0x237E2312, 0xCD12F2C3, 0xCDC311A4, 0xFF0011BD, 0x00110D18, 0x11081801, 0x03180001, 0xD500FF11, 0xD111A4CD, 0x7C6F837D, 0x76C36782, 0xB7292A11, 0xCD1173C3,
	0x2A3A11A4, 0x23EE18B7, 0xEB5E2356, 0xCD1170C3, 0x292ABDD0, 0xB72622B7, 0xB72B5BED, 0xA4CD4418, 0x185D5411, 0x158FCD3D, 0xEDB7292A, 0x3AB72B5B, 0x2C6FB726, 0x1118D0BB, 0x2A1599CD, 0x2C3AB729,
	0x263A57B7, 0xBD5F3DB7, 0xB7303AD8, 0xCD0DB9C3, 0x3A5D11A4, 0x1857B72C, 0x11A4CD09, 0x2A3A6BEB, 0x89CD67B7, 0xBDCDC315, 0x2115D7CD, 0xCD7C0001, 0xCD7D176E, 0x00211767, 0xCD5D5400, 0x0011160E,
	0x7FFF2180, 0xA5CDD5E5, 0xC3D1E116, 0x7ACD16EA, 0x75CD6717, 0xCDC96F17, 0xE02115F0, 0x0AB4C315, 0xC3BDDC09, 0x9AC31786, 0x17B4C317, 0x0C55CDAF, 0x19D5CDAF, 0x17B0CD2F, 0xCD17ACC3, 0x53ED165D,
	0x9922B697, 0x5BEDC9B6, 0x992AB697, 0x53EDC9B6, 0x9522B693, 0x000011B6, 0xE2186B62, 0xB6935BED, 0xC9B6952A, 0xCD1606CD, 0xCDE515FE, 0x44ED0B0C, 0x0026FDDE, 0x287ACB6F, 0xEB19EB03, 0x7D5FA32F,
	0x19B6932A, 0x16E5DC0F, 0x16E5DC0F, 0x077AE5D1, 0xCB130130, 0xB6952A83, 0xE5C3D119, 0x972AE516, 0xE5D119B6, 0x19B6992A, 0x9B2AC9D1, 0x52ED37B6, 0x2A167EF2, 0xEDB7B69D, 0xF6F03752, 0xC9AFC9FF,
	0xB7B69F2A, 0x7BFA52ED, 0xB6A12A16, 0xF252ED37, 0xC937167E, 0xE51627CD, 0xE1166ACD, 0xCDEBD5D0, 0xD1EB1680, 0xD1CDE5C9, 0xCDE5D116, 0x7BD116D1, 0x389C7A95, 0xE67BEB01, 0xF67D5FF8, 0x0CCD6F07,
	0xE1FC3D0B, 0xE1FC3D16, 0x9B53ED16, 0xB69D22B6, 0x21B77AC9, 0x21F80000, 0x957B027F, 0xEBD09C7A, 0xCB2ACBC9, 0xCB2CCB1B, 0xCDE5C91D, 0xE5D11703, 0xD11703CD, 0x9A7C937D, 0xEDEB0138, 0x22B69F53,
	0x7AC9B6A1, 0x000021B7, 0xCB3ACBF8, 0x00C7211B, 0x9C7A957B, 0xEDC9EBD0, 0x2AB69B5B, 0x0CCDB69D, 0x27FC3D0B, 0x29F03D17, 0xEB29EB23, 0x9F5BEDC9, 0xB6A12AB6, 0x17CDF118, 0x52EDB717, 0x16E5CD23,
	0xCB16E5CD, 0x5BED453D, 0x9F2AB6A1, 0xEDB7E5B6, 0xED4D2352, 0xE1B69B5B, 0x0BAFCDC5, 0xB6A43AD1, 0x0DBDCD4F, 0xCD1615C3, 0xA3320C8E, 0x8ECDC9B6, 0xB6A4320C, 0xB6A33AC9, 0xA43A0318, 0x0CA7C3B6,
	0xC3165DCD, 0x94CDBDDC, 0xAFCDD016, 0xB6A33A0B, 0xBDE8C347, 0xC3165DCD, 0x94CDBDDF, 0x177AD216, 0xC30BAFCD, 0x5DCDBDE5, 0xBDE2C316, 0xC9B6B332, 0xC9B6B232, 0x188BCDE5, 0x1627CDE1, 0xB6A52AE5,
	0x7C52EDB7, 0xFCB6AD32, 0xE5D11939, 0xB7B6A72A, 0x327C52ED, 0x39FCB6AE, 0xEDB7D119, 0x329F1952, 0xAE3AB6AF, 0xEB0428B6, 0xF5B6AD3A, 0xB6AB53ED, 0xB23A4D44, 0x0128B7B6, 0xB043ED03, 0x1939CDB6,
	0xA92219E5, 0x2CCBE1B6, 0x07F11DCB, 0xCDE51238, 0xAD2A188B, 0x672F7CB6, 0x226F2F7D, 0x1218B6AD, 0xB7B6B23A, 0xE5190D20, 0x07B6AF3A, 0xD418DADC, 0x7AE11928, 0x1898CAB3, 0x0001E5DD, 0xE1DDC500,
	0xB7D1E5DD, 0x5BED5AED, 0x53F2B6AB, 0x19DD0318, 0xAFFA3019, 0x929F5F93, 0x05301957, 0x180B19DD, 0xA95BEDF8, 0xE5C519B6, 0xB7B6B02A, 0x063042ED, 0x214D4409, 0xB0220000, 0x1898CDB6, 0x0830C1E1,
	0xB6B05BED, 0xB820B37A, 0xD5C9E1DD, 0xED1624CD, 0x22B6A553, 0xC9D1B6A7, 0x07B6AF3A, 0xB1784D38, 0xA72A3828, 0x442B09B6, 0x80CDEB4D, 0xB6A72A16, 0xA72223EB, 0x280638B6, 0x9F4BED21, 0x1680CDB6,
	0xEDC00538, 0xD5B6A15B, 0xB6A55BED, 0xE1166ACD, 0xAD210538, 0xDCF0AEB6, 0xA52A1016, 0xB6AD3AB6, 0x02382307, 0xA5222B2B, 0x78C937B6, 0x2A3928B1, 0x2B09B6A5, 0xCDEB4D44, 0xA52A166A, 0x2223EBB6,
	0x0638B6A5, 0x4BED2228, 0x6ACDB69D, 0xC0053816, 0xB69B5BED, 0xA75BEDD5, 0x1680CDB6, 0x210538E1, 0xF0AEB6AE, 0x0FC2DCEB, 0x3AB6A72A, 0x2307B6AE, 0x2B2B0238, 0x37B6A722, 0x6F95AFC9, 0xC967949F,
	0xD4CDE5DD, 0xE1DDE512, 0xCD1624CD, 0x4B301697, 0x0701D5E5, 0xEB09EB00, 0xCD42EDB7, 0xE1D11697, 0xAFCD3930, 0xE508160B, 0x37005EDD, 0xC4CD13CB, 0xDC09CB19, 0x23CB0C05, 0xCDE1F420, 0x23DD0C1F,
	0xDDE42015, 0x1606CDE1, 0x0B0CCDEB, 0x28000801, 0x09033004, 0xEB090909, 0x0615FEC3, 0xDDD5C508, 0x8F37007E, 0xCDF5D5E5, 0x08301697, 0xF10BAFCD, 0x19C4CDF5, 0x13E1D1F1, 0xD1E92087, 0xC123DD2B,
	0xBF18DB10, 0x38B6A33A, 0xB6B43A08, 0xA43AC0B7, 0xE8C347B6, 0xB6B432BD, 0xB6A522C9, 0xED1B0136, 0xCDB6A753, 0xAA320C8E, 0x1624CDB6, 0xDC1697CD, 0xE5D01B42, 0xE31AE7CD, 0xC11B15CD, 0xA932FF3E,
	0xC5D5E5B6, 0xC11A0BCD, 0x32AFE1D1, 0xDECDB6AB, 0x1697CD1A, 0x381A50DC, 0xB6A52AF5, 0x2801FEE7, 0xB6AB322A, 0xB6A72AEB, 0x09000701, 0xEBB6A722, 0x2B47E72B, 0xE72B4FE7, 0x5FE72B57, 0x57E72BD5,
	0x2B5FE72B, 0xEBB6A522, 0x3AC618D1, 0xC90FB6A9, 0xB6AC43ED, 0x381B42CD, 0x1AF1CD09, 0xB6AE22D0, 0xCDE51118, 0xAE221B15, 0x917DC1B6, 0xCBDC987C, 0xCD69601A, 0xB0221AE7, 0xAC4BEDB6, 0x42EDB7B6,
	0x30112809, 0x1AF1CD08, 0x181A9DDC, 0x6960E507, 0x1ACBCDC1, 0xEDB6AE2A, 0x37B6B04B, 0x2AE5D5C9, 0xF911B6A7, 0x30D119FF, 0xB6A7221C, 0x23B6A52A, 0x23722373, 0x722373D1, 0x70237123, 0xB6AB3A23,
	0xB6A52277, 0xA932AFC9, 0xCDC9D1B6, 0x42CD1AD7, 0x1AF1D41B, 0x3A1A9DDC, 0x322FB6AB, 0x3A1BB6AB, 0xC8B7B6AB, 0xAFC91313, 0xB69F4BED, 0x2B1AF3CD, 0xC5FF3EC9, 0xCDF5E5D5, 0x47F11B4F, 0x041B34CD,
	0x47300410, 0x433877AE, 0xEDE323E3, 0x193C2852, 0x180C39CD, 0xE5D5C5E7, 0xB6A14BED, 0xB71B4FCD, 0x272852ED, 0x0C1FCD19, 0x281B34CD, 0xE377AE1E, 0xEB18E32B, 0xAEB6A33A, 0xAA3AC8A1, 0xC8A1AEB6,
	0xD5C5C937, 0x0BAFCDE5, 0xE11B34CD, 0xC5C9C1D1, 0x0BAFCDD5, 0xAFCDE3D1, 0xC9E1EB0B, 0xCD1E0221, 0x32AF1DF6, 0x6F67B655, 0x01B63122, 0xD611FFB0, 0xB69221B5, 0x09EB043E, 0x732B72EB, 0xF6203D2B,
	0x011EEF21, 0xB0ED00FA, 0x35110A06, 0xB63F21B6, 0x361312AF, 0xF91023FF, 0xCD1E75CD, 0x90111BF8, 0x009821B5, 0x211C0ACD, 0xB4CD1BB3, 0x0AB4CD0A, 0x031E0BC3, 0xB8C3BDEE, 0xBDF4031D, 0xCD1D40C3,
	0xFB301BC5, 0x2A21E5C9, 0xFF367EB6, 0x2A2738BE, 0xB77CB628, 0xE1CD1120, 0xFE1B301C, 0xFE173880, 0x12383FA0, 0xD5002E67, 0x381CB3CD, 0x2C002602, 0xD1B62822, 0xC9E1E030, 0x2A32FF3E, 0xC5CDC9B6,
	0xC9FB381B, 0x3F1C0ACD, 0x7DF3C9FB, 0xDE7C31D6, 0x2219D800, 0x22EBB62D, 0x3001B62B, 0x2301360A, 0x100C2371, 0x3C21EBF8, 0xED0A0E1C, 0x1306EBB0, 0x102377AF, 0xB62F22FC, 0xC9B62932, 0x0D012E01,
	0x4E555205, 0xCD780D22, 0xC5D01CC3, 0x6ACDE5D5, 0xD1E13F1C, 0x791BD0C1, 0xE713120C, 0xF9200D23, 0x78B62921, 0x770120AE, 0x0006C937, 0x95796F60, 0x7D0F30C8, 0xEB194F69, 0x1CA7CD09, 0xB0ED2328,
	0x194F1F18, 0xB62F2AE5, 0x2D2AEB09, 0x7C937DB6, 0xCDD8E19A, 0x2F2A1CA7, 0xD50628B6, 0xB8ED2B1B, 0x2F53EDD1, 0x3AC9B7B6, 0x4F95B62F, 0x9CB6303A, 0xCDC9B147, 0xBDD01CC3, 0xE5D03FC8, 0x7E190026,
	0xE6C937E1, 0xD020FE7F, 0xB62B2AE5, 0x3C000011, 0x3D235E19, 0xEB7BFA20, 0xCDC937E1, 0xFB301CE1, 0xCDC5E5C9, 0x3A301E9D, 0x28EFFE79, 0x870FE634, 0x3C3D8787, 0xFB3008CB, 0x211D25CD, 0x7ECBB632,
	0x61FE0A28, 0x7BFE0638, 0xE0C60230, 0xD328FFFE, 0x3121FEFE, 0xFE0528B6, 0x052023FD, 0x18772F7E, 0xE1C137C2, 0xDA11CBC9, 0x3A471ECE, 0xE6B1B631, 0xC9C27840, 0x1EC4C31E, 0xC9B6312A, 0xC9B63122,
	0x21B64911, 0x83CDB63F, 0xB64B3A08, 0x214FA0E6, 0x77B6B637, 0x11B64921, 0x0006B635, 0xC4A6AE1A, 0x127E1DD1, 0x790C1323, 0x0AFE0FE6, 0xE679EE20, 0x4F71CBA0, 0x78BDEEC4, 0x5321C0B7, 0x2AC035B6,
	0x42EBB654, 0x35210016, 0x2A7E19B6, 0xA619B691, 0x5321C8A0, 0x8A3A34B6, 0x79C0B7B6, 0x333A4FB3, 0xB65332B6, 0x791E86CD, 0x606F0FE6, 0xFEB65422, 0x60CBC008, 0xC9F1CBC0, 0xCBB63D21, 0xEE79C856,
	0xC55620A0, 0x8E0A0623, 0xC1FC102B, 0x4920A4FE, 0x5FD5E5C7, 0x47A33C2F, 0xCDB6343A, 0xAB781DA1, 0xE1D1F120, 0xB63B3AC9, 0x3A6F7FE6, 0x7FE6B63E, 0x332AC967, 0x3322C9B6, 0x0BCDC9B6, 0xB657211E,
	0xD2CD4006, 0x32FF3E01, 0xC5C9B656, 0xB65621D5, 0xCD230036, 0xC1D10284, 0xB65621C9, 0xBE00367E, 0x23D5C5C8, 0x0E01E2CD, 0x1E86CDEF, 0x2AC9C1D1, 0x1C18B691, 0x2AD050FE, 0x55CDB691, 0x7E4F2F1E,
	0x77A8A1A8, 0x373AF5C9, 0x4FA0E6B6, 0xB63521F1, 0xA61E55CD, 0xE6F5D5C9, 0x0F0F0FF8, 0x1900165F, 0x6D21E5F1, 0x5F07E61E, 0xD1E17E19, 0x040201C9, 0x40201008, 0x8621F380, 0x231536B6, 0x362377AF,
	0x23772301, 0x8621C977, 0x2835B7B6, 0x1EB4CD0E, 0x21702371, 0x2134B68A, 0x3437B688, 0xB68821C9, 0x0E2835B7, 0x4E1EB4CD, 0x8A214623, 0x862135B6, 0xC93437B6, 0xFE7E3423, 0xAF022014, 0x5EC68777,
	0xC9B6266F, 0x18B68B2A, 0xB68D2A08, 0x8F2A0318, 0x8C6F85B6, 0xC97E6795, 0x18B68B2A, 0xB68D2A08, 0x8F2A0318, 0xD050FEB6, 0x958C6F85, 0xF0C97067, 0x8689F1F3, 0xF28A8B83, 0x858887E0, 0x10808281,
	0x84230D2A, 0x2DFF24FF, 0x7C705E29, 0x403A3D6D, 0x6C696F5C, 0x213B2C6B, 0x6879757D, 0x5D206E6A, 0x67747228, 0x27766266, 0x737A6522, 0x26786364, 0x0961FC7B, 0x0B77FD71, 0x5809080A, 0xF47FFF5A,
	0x8689F5F7, 0xF68A8B83, 0x858887E0, 0x10808281, 0x843E0D3C, 0x5FFF40FF, 0x25507C5B, 0x302F2B4D, 0x4C494F39, 0x382E3F4B, 0x48595537, 0x36204E4A, 0x47545235, 0x34564246, 0x535A4533, 0x31584344,
	0x0941FC32, 0x0B57FD51, 0x5809080A, 0xF87FFF5A, 0x8689F9FB, 0xFA8A8C83, 0x858887E0, 0x10808281, 0x84FF0DFF, 0x1FFF5CFF, 0xFF101EA2, 0x00FFFF0D, 0x0C090F1C, 0xFFFFFF0B, 0x081915FF, 0xA6FF0E0A,
	0x071412FF, 0xFF160206, 0x131A051D, 0x1B180304, 0xE101FC7E, 0xFF17FE11, 0xFFFFFFFF, 0x077FFFFF, 0xFFFF4B03, 0xABFFFFFF, 0xB1ED218F, 0x00360406, 0x11FB1023, 0x8106208B, 0x3E01D2CD, 0xB2B5323F,
	0x01B1F821, 0x0811003D, 0x2377AF01, 0x09732372, 0xEB29EB3C, 0xF22003FE, 0xE5DD070E, 0xB1F021E5, 0x21DDE534, 0xCD79B1B9, 0xC5F52209, 0xCD2286CD, 0xE5DD23E7, 0x131313D1, 0x0113626B, 0x0036003B,
	0x36DDB0ED, 0xF1C1041C, 0x35E1DD20, 0xC9E1DDE1, 0xF3B1EE21, 0xFB00367E, 0x772BC8B7, 0x000E032E, 0xCD85073E, 0x202D0863, 0x11C937F7, 0xB71AB1ED, 0x21DDD5C8, 0x09CDB1B9, 0x7EDDF522, 0x23DEDC0F,
	0xE3F320F1, 0x2300367E, 0xDDC9E177, 0xB1EE3AE5, 0xF53D28B7, 0xB1B921DD, 0xDD003F01, 0x303FCB09, 0x7EDDF5FA, 0x1FDC1F04, 0x077EDD24, 0x231FDC1F, 0xF12213DC, 0x3AC1E220, 0xA02FB1EE, 0x21DD1228,
	0x3F11B1B9, 0xCB19DD00, 0xE7DCF53F, 0xF520F123, 0xB1F032AF, 0x21C9E1DD, 0xB77EB1EE, 0xC03523C8, 0x7E233447, 0x362BC0B7, 0xB1BE2103, 0xAF003F11, 0x3038CB19, 0x052035FB, 0x8A06CB2B, 0x20352323,
	0x06CB2305, 0x042B2B8A, 0xC8B7E610, 0x77B1F021, 0x01E2C323, 0x7E206BCD, 0xC83707E6, 0x1AFCB64F, 0x21DD4120, 0x3F11B1B9, 0x19DDAF00, 0xFA3038CB, 0xDD1E72DD, 0x9F3F1CBE, 0xB7EF1004, 0x1F7E41C0,
	0xE6B01F1F, 0x21E54F0F, 0xE334B1F0, 0xB921DD23, 0x003F11B1, 0x38CB19DD, 0xC5E5FA30, 0xDD1B7EDD, 0x35DD1B34, 0x9CCDEB1C, 0xDDEBE521, 0xA12F017E, 0x237E1312, 0x87878787, 0xE6237E47, 0x1312B00F,
	0xED000601, 0x7EDDE1B0, 0x1A34DD1A, 0xCC03B6DD, 0xE1C1221F, 0xE3BA1004, 0xC937E135, 0x878703E6, 0xDD1FC687, 0x6F85E1E5, 0xC967958C, 0x206BCD6F, 0xC807E67D, 0x34B1F021, 0xB921DDE5, 0x2209CDB1,
	0x03CBDDF5, 0x2219C45E, 0xE1F220F1, 0x07E6C935, 0xB1BC21C8, 0x19003F11, 0xF3FC301F, 0x8787877E, 0x19001911, 0x362323B6, 0xE6C9FB00, 0x21EBC807, 0x3F01B1D5, 0x301F0900, 0xBEF3AFFC, 0x237A0120,
	0xFB772373, 0xE2C3EBC8, 0x003F1101, 0x3FCB19DD, 0xDDF918D8, 0x28B71A7E, 0x197EDD6D, 0x7E219CCD, 0xCB0D28B7, 0xE559205F, 0x90CD0036, 0x5630E122, 0x100336DD, 0xF0E67E23, 0x235FAEF5, 0x2356234E,
	0x0828B1B2, 0x2408CDE5, 0xE10156DD, 0x235E234E, 0x6F66237E, 0x22DECDF1, 0xDDB1EE21, 0xB07E0146, 0x0320A877, 0xDD033623, 0x35DD1934, 0x1C34DD1A, 0xDD1E7EDD, 0xB7001E36, 0x6EDD67C8, 0x01E2C31D,
	0x36DD9ECB, 0xEE210803, 0x017EDDB1, 0xC977A62F, 0xDD47E5DD, 0x21DD014E, 0x47CBB1F8, 0x21DD0C20, 0x4FCBB237, 0x21DD0420, 0x7EDDB276, 0x2728A103, 0x01BEDD78, 0xE5DD1928, 0xB27621DD, 0x042057CB,
	0xB23721DD, 0xA1037EDD, 0x19CD0C28, 0xCDE1DD22, 0xE1DD2219, 0xDDE1C937, 0x0370DDE1, 0xFBCBC9B7, 0x5F0F73DD, 0x0120B47D, 0x0875DD2B, 0x790974DD, 0x3E0828B7, 0x0863CD06, 0xB2027EDD, 0x7B23E8CD,
	0x210A28B7, 0x0016B2A6, 0x20B77E19, 0x231B2103, 0xDD0A75DD, 0xCDCD0B74, 0x010D1823, 0xDDC80001, 0x66DD0D6E, 0x105EDD0E, 0x28FFFE7B, 0x237E8775, 0x0C284938, 0x0F4EDD1D, 0xCB0420B7, 0x81062879,
	0xDBCD0FE6, 0x7EDD4E23, 0x38874709, 0xDD91AF1B, 0x0C380886, 0x235DF205, 0xAF084EDD, 0x0970DD47, 0xB00877DD, 0xFF1E0220, 0xAECCB77B, 0x1073DD23, 0x0671DDF3, 0x800736DD, 0x57C9B7FB, 0xCD0D3E4B,
	0x3E4A0863, 0x0863CD0B, 0xCD0C3E4E, 0x103E0863, 0xCD23DBCD, 0x3C7B23AE, 0x1B218E20, 0x23CDCD23, 0xDDAF8618, 0x77DD0377, 0x0477DD07, 0x35DDC937, 0xDD1E200C, 0x2187097E, 0x1130231B, 0x200834DD,
	0x0934DD06, 0xDDC8FF1E, 0x66DD0A6E, 0x77DD7E0B, 0x235E230C, 0xDD0D75DD, 0xDDC90E74, 0xDD4F0F77, 0x08C6007E, 0xAF0863C3, 0x017EDD47, 0x2102B6DD, 0xB6F3B2B5, 0xFB77BEA8, 0xB7780320, 0xDECDAFC0,
	0x3E4EF323, 0x0863C307, 0x7B2481CD, 0xD024ABCD, 0xC87FE67E, 0xDD1175DD, 0x70CD1274, 0xDD091824, 0x66DD146E, 0x185EDD15, 0xD67B234E, 0x1E0438F0, 0x1D0E1800, 0x579F8779, 0x81167EDD, 0x177EDD4F,
	0x81CD578A, 0xB77B4E24, 0x7EDD1920, 0x10203D13, 0xDD116EDD, 0xC67E1266, 0xDD053880, 0xC9000436, 0xDD2470CD, 0xDDF31873, 0x36DD0571, 0xC9FB8004, 0x231377DD, 0x75DD235E, 0x1574DD14, 0x1CC0B77B,
	0x007EDDC9, 0x71DDF587, 0x0863CD16, 0xDD4A3CF1, 0x63C31771, 0xB2A61108, 0x96110318, 0xAECDEBB3, 0xEDD0EB24, 0xA621C9B0, 0x210318B2, 0xC8B7B396, 0x01D010FE, 0x3D090010, 0xC937FC20, 0xCD2557CD,
	0xCDAF2599, 0xBFCD24E1, 0x014D212B, 0x2929193E, 0x29292929, 0x3FE60F0F, 0xB1E9226F, 0x37B1E73A, 0xB11832C9, 0x1A21DDC9, 0x2502CDB1, 0x26ACDCE5, 0x5BEDD0E1, 0x4BEDB134, 0x313AB137, 0x21DDC9B1,
	0x7EDDB15F, 0x0E3EB700, 0xE3E5DDC0, 0x23732334, 0x23732372, 0xE1EB2372, 0xAF400ED5, 0x200D1312, 0x78D5D1FB, 0x023810FE, 0x48041006, 0x23E70718, 0x122926CD, 0x0DF71013, 0x1A1B0928, 0x032020EE,
	0xE1F41812, 0xDD1534DD, 0xDD161736, 0xC9371C35, 0xB7B11A3A, 0x21C80E3E, 0x0106B11A, 0xC500367E, 0xF1256DCD, 0xAEB1E421, 0x9F77C037, 0xD804FEC9, 0x56235E23, 0x3613626B, 0x07FF0100, 0x3ABAA1C3,
	0x03FEB15F, 0xFFC61328, 0x21D00E3E, 0x2335B175, 0xB6237E23, 0x2786C437, 0xB15F21D0, 0xBC180206, 0x06C5D5E5, 0x25F6CD05, 0x322A1A20, 0x37B57CB1, 0x3026ACCC, 0xB1322A0F, 0xB132222B, 0xE7B11D2A,
	0xB11D2223, 0xD5E52C18, 0x5F214FC5, 0xCD0506B1, 0x1E2025F9, 0x11B1772A, 0x52ED0800, 0x2786D4C5, 0x2A0F30C1, 0x2223B177, 0x622AB177, 0x222371B1, 0xD1C1B162, 0x1A21C9E1, 0xC8B87EB1, 0x0E3E01EE,
	0xCDC970C0, 0xE5D025A0, 0x23B1322A, 0x2AB13222, 0x222BB11D, 0xC9E1B11D, 0xCD0206EB, 0xEDC025F6, 0xCDB13453, 0x342A263C, 0x325BEDB1, 0x342219B1, 0x26ACCDB1, 0x2AC8F038, 0xC937B1BE, 0xEDB11B2A,
	0x7BB1324B, 0xDA9C7A95, 0x2B09BAA1, 0xEB2B09EB, 0xE5BAA7C3, 0x5F214FC5, 0xCD0206B1, 0x2D2025F9, 0x32E1C179, 0x53EDB176, 0x43EDB17C, 0x6022B17E, 0x7753EDB1, 0xF7FF21B1, 0x21D83F19, 0x77220800,
	0x52EDEBB1, 0xB1602AE5, 0x86CDE519, 0xD0D1E127, 0x1A21DC18, 0x3EB77EB1, 0x0436C00E, 0xB11B53ED, 0x24E1CDAF, 0x3826B3CD, 0x2557C3FB, 0xB7B1303A, 0x01C00F3E, 0xE5CD8301, 0x215F3027, 0x4011B1A4,
	0xCD2C3E00, 0x523029A6, 0x2FCD8B06, 0xCD072829, 0x53202737, 0x04CD8906, 0xB75BED28, 0xB1342AB1, 0xFEB11A3A, 0x210E2802, 0x3E19F7FF, 0x2A2B3804, 0x1D22B11B, 0xCD163EB1, 0x1E3029A6, 0x34B12F21,
	0x23B1B53A, 0x3632AF77, 0xB1B72AB1, 0xCDB13222, 0x8C3E292F, 0x37287ECC, 0x21B76518, 0x5828B11A, 0x85CD8506, 0xF5941828, 0x04CD8806, 0x8B30F128, 0x83CD8706, 0x3A841828, 0x28B7B136, 0xB1BB3A1B,
	0x3AC0B72F, 0xC4B7B11F, 0x21C02760, 0x1F11B1A4, 0x004001B1, 0xC9AFB0ED, 0xC02760CD, 0xC9BE1AEB, 0x11B11F21, 0x1006B1A4, 0x2926CD1A, 0x26CD7E4F, 0x23C0A929, 0xC9F11013, 0xCD03367E, 0x9FB7256D,
	0x2BBFCDF5, 0x0201C9F1, 0x27E5CD84, 0x8A064A30, 0xCDB16411, 0x7B212807, 0x27FACDB1, 0x602A3A30, 0xB16222B1, 0xE5B17922, 0x11B16421, 0x2C3E0040, 0xE129AFCD, 0x5BED2230, 0x163EB177, 0x2129AFCD,
	0xFADCB175, 0x21113027, 0x77220000, 0xB17421B1, 0x7B32AF34, 0xA71837B1, 0xB15F21B7, 0x86069A28, 0x182885CD, 0xB1E421B9, 0x3771BE79, 0xD2C4C5E5, 0x9FE1C128, 0x2BBBCDD0, 0xB77EC99F, 0x2C01C837,
	0x2BE2C301, 0x3AB1A411, 0xC0B7B118, 0xCDB11932, 0x98CD28F3, 0x20B71A28, 0xCD8E3E0A, 0x10012899, 0xCD2E1800, 0x0001292F, 0x6B0D2810, 0x28B77E62, 0x10230C04, 0x4F4178F8, 0x1A28FDCD, 0xB72926CD,
	0x203E0220, 0x35CDD5C5, 0x13C1D113, 0xCECDED10, 0xEB09EB28, 0x99CD8D3E, 0xCD020628, 0xCD1A28FD, 0xCECD2914, 0x2FCD1328, 0x130B2029, 0xC60FE61A, 0x28F0CD24, 0x211A5818, 0xC8B6B119, 0x99CD6D18,
	0x3E681828, 0x91CDF5FF, 0x60C6F128, 0x1828F0D4, 0x117CCD5A, 0x28EBC425, 0x7FE6E578, 0x29352147, 0x237E0728, 0x10FB20B7, 0x28B77EF9, 0x28B5CD05, 0x23E1F718, 0x2899FAC9, 0x040006E5, 0x3007237E,
	0x28FDCDFA, 0xE6237EE1, 0x28F0CD7F, 0x203EF710, 0x183A1E18, 0xC037B7B1, 0xCD2891CD, 0x76CD1BFE, 0x1CDBCD12, 0xFE127ECD, 0xCD37C8FC, 0x0A3E28F3, 0xF513FEC3, 0xCD013EE5, 0xF1E1115A, 0x52CDD5C9,
	0x7CCD5C12, 0x833D7C11, 0xD1BA3D80, 0x32FF3ED8, 0xD718B119, 0xD604FF06, 0xC6FB300A, 0xB778F53A, 0xF12914C4, 0x61FECA18, 0xD07BFED8, 0x3AC9E0C6, 0x04FEB11A, 0x657250C9, 0x5000F373, 0x74D9414C,
	0x61EE6568, 0x656BF96E, 0x6500BA79, 0xF26F7272, 0x00818000, 0xC3455280, 0x81E46E61, 0x61655200, 0x570082E4, 0xE5746972, 0x65520082, 0xE46E6977, 0xE5706174, 0x756F4600, 0xA020646E, 0x616F4C00,
	0xE76E6964, 0x76615300, 0x00E76E69, 0x00EB4F00, 0x636F6C62, 0x6E5500EB, 0x656D616E, 0x6C6966E4, 0x20202065, 0xE3CD00A0, 0x2821F529, 0xCD19182A, 0xCDF529E3, 0x67212AD4, 0x2A0DDC2A, 0x182AE9DC,
	0x29E3CD0F, 0x2A3721F5, 0x2A89CDE5, 0x2A0DDCE1, 0x8201F5D1, 0x0149EDF7, 0x49EDF610, 0xC1CD7AFB, 0x32C9F12B, 0x1C1BB1E5, 0xE9CDD5E5, 0xE1DDD11F, 0xF32BBBCD, 0xEDF40E01, 0xF6D00149, 0x100E49ED,
	0x920149ED, 0x0149EDF7, 0x49EDF658, 0x28B77AC9, 0x1ED5E50D, 0x2A1ECD00, 0x15D0E1D1, 0xFF01F320, 0xEB43EDFF, 0xE90116B1, 0xD02B20CD, 0xDD0077DD, 0x201D1523, 0xCD1218F3, 0x47D02B20, 0xA8BAD7CD,
	0xDDC0033E, 0x201D1523, 0x062815EE, 0xD02B20CD, 0x16CDF718, 0x2B20CD2B, 0x0720AAD0, 0xD02B20CD, 0x3EC837AB, 0xCDC9B702, 0x68CDBAD7, 0x23DDD02B, 0xF3201D15, 0xAF072815, 0xD02B68CD, 0x16CDF618,
	0x2B68CD2B, 0x68C37BD0, 0x93CDD52B, 0xB7D8D12A, 0x2EF618C8, 0x2B3DCD55, 0x000011D0, 0x2B3DCD62, 0x0006EBD0, 0x2025EB09, 0x927961F4, 0xEB479F4F, 0x3DCDEB09, 0xCB7AD02B, 0x8A3FCB3F, 0x91EA3894,
	0x1F7AE738, 0xE622678A, 0x2B20CDB1, 0xB1E521D0, 0xC937C0AE, 0x212BF9CD, 0xECCD0801, 0xCDB7D02A, 0x3AD02B78, 0x68C3B1E5, 0x0021212B, 0x78EDF406, 0xE5C804E6, 0x2B78CD37, 0xB57C2BE1, 0xC937EE20,
	0xACB1EB2A, 0x7C2B10F2, 0x7D6708EE, 0x376F10EE, 0xEB226AED, 0xEB2AC9B1, 0x5F2F7DB1, 0xC9572F7C, 0x2A081ED5, 0x44CDB1E6, 0x2B4DDC2B, 0x917C0D30, 0xCD12CB9F, 0x201D2B00, 0xD1377AEA, 0xEDF406C9,
	0xC804E678, 0x03C65FED, 0x1FE60F0F, 0x79F5064F, 0x384F02C6, 0xAD78ED0E, 0xF32080E6, 0xCB4FEDAF, 0xAFC9370D, 0xC93C4FED, 0x57081ED5, 0x78CD02CB, 0x1D03302B, 0xC9D1F620, 0xB1E84BED, 0x9FB1EA2A,
	0x7D072867, 0x796F8087, 0x327D4F90, 0x0A2EB1E8, 0x382BA7CD, 0x0C309106, 0x7C4F3C2F, 0x2E2B00CD, 0x2BA7CD0B, 0xEDC9013E, 0x913FCB5F, 0x203C0330, 0xEDF706FD, 0xEDAFF569, 0x3EC9F14F, 0x3E021810,
	0xF606C5EF, 0xE60448ED, 0x28083E10, 0x79ED3C01, 0x790C2837, 0x01C510E6, 0xCC3700C8, 0x79C12BE2, 0xE5C5C9C1, 0x3E2BF9CD, 0x1E45CD42, 0x0720C1E1, 0x20B1780B, 0xAFC937ED, 0x068201C9, 0x20B1780B,
	0xD5C5C9FB, 0x2DF2CDE5, 0x7E00FF01, 0x073830FE, 0x42DC3AFE, 0x78F4382C, 0x42C47EB7, 0x7E0CE52C, 0xFA20B723, 0xE1B11532, 0xC52EE4CD, 0x2F56CDE5, 0x48CDC1E1, 0xF5F4302C, 0xF12E4FCD, 0xFEC1D1E1,
	0x040CC9FC, 0x2F25C323, 0x2C7221E5, 0x7BB1785F, 0xF0FE0B20, 0xF4FE0738, 0xAE210330, 0xE523562C, 0x23BE2323, 0x20150428, 0x7EF1E3F7, 0x7B6F6623, 0x8A13C9E3, 0x2CD0FC2D, 0x0D2CCEEF, 0x3CF02CF2,
	0x2D0AF12D, 0xF32D34F2, 0x4FF82D02, 0x2D1DF92D, 0xFB2D45FA, 0x21F42D14, 0x2E26F52E, 0xF72E1CF6, 0x65E02E17, 0x2DC37F2E, 0xE12DCD10, 0xFE042D81, 0x2CBDF02C, 0xF22CC1F1, 0xC5F32CC9, 0x180B3E2C,
	0x180A3E0A, 0x18093E06, 0xCD083E02, 0xC9B713FE, 0xF52CF2CD, 0xCD2CEA21, 0x7CCD2CF2, 0x08282511, 0xFECD0D3E, 0x2CC1CD13, 0x422AC9F1, 0x6B616572, 0x7EF5002A, 0x25C4B723, 0xF1F8202F, 0x073EC937,
	0x0116C918, 0x282D1ECD, 0x73CDC9F5, 0xBA90792D, 0x0A18EC38, 0x7A2D73CD, 0x1857C893, 0xB9785101, 0xCDCDD5C8, 0x25D47E2E, 0xD423042F, 0x15D12EE4, 0x3C18EC20, 0x50CD0116, 0xC9C3282D, 0x782D73CD,
	0x18BB38BA, 0x2D73CD0B, 0xC801D67B, 0x51011857, 0xCDC8B778, 0x07302EC7, 0x20152B05, 0x781118F3, 0x050A28B7, 0xA2CDD52B, 0x2015D12E, 0x2EE4CDF2, 0xE5C9FFF6, 0x7A1252CD, 0xCD573C94, 0xE15C117C,
	0xB1153AC9, 0xB115322F, 0xC8B7C9B7, 0xB1153A5F, 0x0B2879B7, 0x730828B8, 0x7BB70423, 0xFE2F25C3, 0x2CFECAFF, 0xB11432AF, 0x0C2D9BCD, 0x5F737EE5, 0xF920B723, 0x2304E177, 0x3A2EE4CD, 0xC4B7B114,
	0x78C92EA2, 0x2EC7C4B7, 0x052CFED2, 0xCAB9782B, 0x23E52CFE, 0x23772B7E, 0x2BF820B7, 0x14322036, 0xE4CDE3B1, 0x0036E32E, 0x143A0DE1, 0xA6C4B7B1, 0x32AFC92E, 0x1732B116, 0x5BEDC9B1, 0xAA7CB116,
	0xC0AB7DC0, 0xCD4FC937, 0x7DC82EC1, 0xCACD6F81, 0x22DF3011, 0x11C9B116, 0x0D180100, 0x18FF0011, 0x00FF1108, 0x01110318, 0xCDE5C500, 0x7CCC2EC1, 0x67827C11, 0xCD6F837D, 0x0B3011CA, 0x2E4FCDE5,
	0xB11622E1, 0xE12E4ACD, 0x6511C9C1, 0x11031812, 0xC1CD1265, 0xCDE5C82E, 0xCDE3117C, 0x16CD1170, 0x70C3E100, 0xCDE5C511, 0xCDEB117C, 0x0C202EC1, 0x2620B178, 0x22117CCD, 0x0618B116, 0xCD1170CD,
	0xACCD1265, 0xCDEBF513, 0x162A1170, 0xCACD24B1, 0x22033011, 0x4ACDB116, 0xC1E1F12E, 0xC32D8ADA, 0x01162CFE, 0xFF160218, 0xCDD5E5C5, 0xCDD12E4F, 0x09282EC1, 0xCD67827C, 0x4ACD2E0E, 0xB7C1E12E,
	0xB1162AC9, 0xD5C9B57C, 0x18FF0811, 0x0911D504, 0xCDE5C501, 0x847A117C, 0x11CACD67, 0x13FEDC7B, 0xC9D1C1E1, 0xCDEBE5C5, 0xEB4F117C, 0xC4B7237E, 0xF8202F02, 0x91117CCD, 0xCD6F85EB, 0xC1E11170,
	0xC5F5C9B7, 0xCD47E5D5, 0x8391117C, 0xCACD485F, 0x78053811, 0x5F833C87, 0x11CACDEB, 0x2F25DC79, 0xF1C1D1E1, 0xD5C5F5C9, 0x7CCD47E5, 0xCDC54F11, 0xDCC111CA, 0xDCF52DFA, 0xC5782E4F, 0xC11335CD,
	0x91117CCD, 0xF12E06C4, 0x329F0730, 0x4ACDB114, 0xC1D1E12E, 0x7CCDC9F1, 0xCACD4F11, 0x2DFACD11, 0xCD1BBFDA, 0x7CCD1276, 0x06C49111, 0x1BBFCD2E, 0x11127EC3, 0x19182F78, 0x490FDAA2, 0x2F821182,
	0x00000F18, 0xEB810000, 0x18B10E21, 0xB1041104, 0xC5D5E5EB, 0x000501EB, 0xD1C1B0ED, 0xD5C937E1, 0x477FF6C5, 0x121312AF, 0xB4900E13, 0xB54F0D20, 0x0E6C0D28, 0x0D041888, 0xF28F25CB, 0xEBA02FB7,
	0x23772373, 0xC9E1C171, 0xA00001C5, 0xC12FD3CD, 0xD5A806C9, 0xD1379CCD, 0xE1DDE5C9, 0x0496DDAF, 0x90C61B28, 0xC6C5D5D0, 0x373DCD10, 0x5AED21CB, 0x7EDD0828, 0xC13FB703, 0x189FC9D1, 0x37676FF9,
	0x3014CDC9, 0x79E5F0D0, 0x23062034, 0x34F9203D, 0xC937E10C, 0xDDE5D5E5, 0x96DDAFE1, 0x060A2004, 0x10237704, 0x18010EFC, 0x30A0C628, 0x3DCDE525, 0x8FB8AF37, 0xE1444DB1, 0x23702371, 0x7E5F2373,
	0x4780E672, 0xB6AF040E, 0x0D2B0520, 0x7B0CF920, 0xE1D137B7, 0x3014CDC9, 0x78CBC8D0, 0xCDA718C8, 0x28473727, 0x3734FC52, 0x047EDDE5, 0x9F5F80D6, 0x29626B57, 0x29192929, 0x19292919, 0x4F09D67C,
	0xC8C4C5E1, 0x30BC1130, 0x3036E2CD, 0x30F5110B, 0xD13577CD, 0xED18D51D, 0xCD30C111, 0x0B3836E2, 0xCD30F511, 0x1CD13604, 0xCDED18D5, 0xD1793001, 0x6F853D42, 0x5FC924D0, 0xC9010E77, 0x3EBC1FF0,
	0x6B27FE96, 0x3C2F9E6E, 0x4FC837B7, 0x2F30D1F2, 0x3131113C, 0x15280DD6, 0xF5C50938, 0xF130EDCD, 0x47EE18C1, 0x83808787, 0x8AFF3E5F, 0xF2B77957, 0x77C33604, 0x00000035, 0x00008420, 0x00874800,
	0x8A7A0000, 0x1C400000, 0x5000008E, 0x00009143, 0x00947424, 0x98189680, 0x3EBC2000, 0x6B28009B, 0xF9009E6E, 0x40A21502, 0xA53A43B7, 0x68D4A510, 0x84E72AA8, 0x6521AC11, 0xB1022289, 0x226C0721,
	0xEBC9B100, 0xEB3136CD, 0xC83727CD, 0x06B10011, 0x12AE1A04, 0xF9102313, 0x022AE5C9, 0x6C0701B1, 0xE5319CCD, 0x01B1002A, 0x9CCD8965, 0x2AE5D531, 0x9CCDB102, 0x2209E331, 0x01E1B100, 0x4AED6C07,
	0x09C109C1, 0xE1B10222, 0x2AE1DDE5, 0x5BEDB100, 0x0001B102, 0x0436DD00, 0x37ACC380, 0x000021EB, 0xC83D113E, 0xCB13CB29, 0x09F73012, 0x1813F430, 0x322A11F1, 0x25110318, 0x3727CD32, 0xD001FE3D,
	0x36D3CDD5, 0x0436DDF5, 0x32201180, 0x3036DFCD, 0x0434DD06, 0xCDF53DF1, 0x11D52F87, 0xCDD52F82, 0xE3D134A2, 0xD1349ACD, 0xCD3604CD, 0x4C043440, 0x7F5E574B, 0x139B080D, 0x38932380, 0x3B208076,
	0xD58238AA, 0xD13577CD, 0xF2B77CE3, 0x3C2F3210, 0x00267C6F, 0xEB2F9FCD, 0x34A2CDE1, 0x3577C3D1, 0x3504F334, 0x7217F880, 0x9A858031, 0x067F1A20, 0x3492CDE1, 0x112F7DD2, 0xDFCD32A2, 0x37E8F236,
	0xCD32A711, 0xE2FA36DF, 0x329D1137, 0x7B3469CD, 0xED3255F2, 0x70CDF544, 0x2F8DCD35, 0x3443CDD5, 0xEB32F403, 0xB808730F, 0x007B52D5, 0x80000000, 0x3443CDE3, 0xDE600902, 0x17F87801, 0xCD7E3172,
	0xE5D13577, 0x349ACDEB, 0x04CDE1EB, 0x326B1136, 0xF134A2CD, 0x048EDD37, 0x370477DD, 0xAA3B29C9, 0x33C78138, 0xF887300F, 0x87B17217, 0xEB326B11, 0xEB3727CD, 0xF52F7DCA, 0x283727CD, 0x34FC4725,
	0x24CDE537, 0x2538E133, 0xEAFAE1E3, 0xCDD5C532, 0xDCD131B6, 0x2FDC3577, 0x78D0C132, 0x3731FCB7, 0x37F1C937, 0x37E8CDF0, 0x3CAFC9AF, 0xC5F14FC9, 0x8F3779F5, 0xCD47FD30, 0x78EB2F8D, 0xF5152887,
	0x303570CD, 0xF430F116, 0xB10411F5, 0x303577CD, 0xE818F10A, 0xFBFC37F1, 0xF1BE1835, 0xE2FAC1F1, 0x37EAC337, 0x2F88CDC5, 0x793014CD, 0x280230C1, 0xC9B77803, 0x9F1F7E4F, 0xFE7947A0, 0x7ED09F02,
	0xAFD827FE, 0xB11332C9, 0x3727CDC9, 0xF63734FC, 0xAF011801, 0x33B411F5, 0x133AF006, 0x0528B7B1, 0x0633B911, 0x3492CDF6, 0xCDF13A30, 0x7BD0346A, 0x3734DC1F, 0x92CDE806, 0x37E2D234, 0xCD0434DD,
	0x1B063440, 0x6EE61A2D, 0x2807FBF8, 0x68890174, 0xDFE17999, 0x287D2335, 0x80A55DE7, 0x490FDAA2, 0x3577C381, 0x2F7DC2F1, 0xFEB1133A, 0xBE11D801, 0x3577C333, 0x22F9836E, 0x0B60B67F, 0x35137936,
	0xD37B0EFA, 0x86652EE0, 0xD52F8DCD, 0xE33349CD, 0xD13353DC, 0xC93604DA, 0xF53727CD, 0x063734FC, 0x3492CDF0, 0xF53D4A30, 0xCD35FBF4, 0xFF0B3440, 0x770F03C1, 0xEBE8FC83, 0x78CA6F79, 0x3ED57B36,
	0xB07CB5B0, 0x7D098BC1, 0xB432E8AF, 0x656C747D, 0xF5D17D62, 0x7A7E9237, 0x7E4CCBC3, 0xAAAAA783, 0xFFFFFE7F, 0x77CD807F, 0x9C11F135, 0x349EF433, 0xB7B1133A, 0xC433C311, 0xFCF13577, 0xC9373734,
	0xCD3570CD, 0x46E12F87, 0x2F90CD23, 0x13131313, 0x0911D513, 0xC5C805B1, 0xCDB10E11, 0xD1C13577, 0xA2CDC5D5, 0x18D1C134, 0xCDF5AFE3, 0x11F13577, 0xA2C4326B, 0xD9CDE534, 0xD113302F, 0x11D5F5E5,
	0x9FCDB109, 0xCDE1EB2F, 0xD1F1349A, 0xAFE1C937, 0xD3CDC93C, 0xC8B8F036, 0x013EC93F, 0x803E0518, 0xE5AF0118, 0xFDD5E1DD, 0x0346DDE1, 0xB7034EFD, 0xBAFA0A28, 0x4FA90F34, 0x47A80218, 0xFD047EDD,
	0x143004BE, 0xB74A4150, 0x047EFD57, 0x280477DD, 0x21FE9254, 0x11184F30, 0x0496FDAF, 0x86DD5928, 0x3021FE04, 0xE1FDE552, 0xA9785FEB, 0xCD7BC5F5, 0xC1793743, 0x3CFAF14F, 0x007EFD35, 0x7EFD6F85,
	0xFD678C01, 0x5F8B027E, 0xCB037EFD, 0xD2578AFF, 0x1ACB37B7, 0x1CCB1BCB, 0x19CB1DCB, 0xC20434DD, 0xEAC337B7, 0x027EFD37, 0xFD0277DD, 0x77DD017E, 0x007EFD01, 0xDD0077DD, 0xC9370370, 0xFD4F91AF,
	0x6F9D007E, 0x9C017EFD, 0x027EFD67, 0x7EFD5F9B, 0x9AFFCB03, 0x78163057, 0x91AF472F, 0x9D003E4F, 0x9C003E6F, 0x9B003E67, 0x9A003E5F, 0x37ACC357, 0xCDB10911, 0xD5EB2F90, 0xDDE5E1FD, 0x047EFDE1,
	0x3D2A28B7, 0x2836AFCD, 0xF51F3024, 0x35B0CDC5, 0xF14FC179, 0x0B207ACB, 0xCB12283D, 0xCB6AED21, 0xDD12CB13, 0xC2B70477, 0xEAC337B7, 0x37E2C337, 0x5D000021, 0x007EFD54, 0xFD35F3CD, 0xF3CD017E,
	0x027EFD35, 0xFD35F3CD, 0x80F6037E, 0x4F1F0806, 0xDD7D1430, 0x7C6F0086, 0x67018EDD, 0x028EDD7B, 0x8EDD7A5F, 0x1ACB5703, 0x1CCB1BCB, 0x19CB1DCB, 0xB7C9DE10, 0x636CD620, 0xCDC9575A, 0xD5EB2F87,
	0xD12F7DCD, 0xE5E1FDD5, 0xFDAFE1DD, 0x5A280496, 0xCA36AFCD, 0x4F3037E2, 0x235E4FC5, 0x237E2356, 0xFDEB6F66, 0xF8CB0346, 0x38369DCD, 0x20B77906, 0x0D331808, 0xCB13CB29, 0x0471DD12, 0xDD3672CD,
	0x72CD0371, 0x0271DD36, 0xDD3672CD, 0x72CD0171, 0x9DDC3F36, 0x699F3F36, 0xDD0166DD, 0x56DD025E, 0xC34FC103, 0xC3C137B7, 0x46DD37EA, 0x37EACD03, 0x010EC9AF, 0xB87A0838, 0x3F36A0CC, 0xFD7D1330,
	0x7C6F0096, 0x67019EFD, 0x029EFD7B, 0x57987A5F, 0x9F11CB37, 0xCB13CB29, 0xD8203C12, 0xC0B87AC9, 0x02BEFD7B, 0xBEFD7CC0, 0xFD7DC001, 0x4FC900BE, 0xFD037EDD, 0xDD4703AE, 0xC8B7047E, 0xA91F4F81,
	0x36CFF279, 0xFE03CBDD, 0xC0377FD6, 0xB7C901FE, 0xE5C9AFF8, 0x7EDDE1DD, 0xD6C8B704, 0xE5C93780, 0xFDD5E1DD, 0x047EDDE1, 0x3804BEFD, 0xB733202C, 0x037EDDC8, 0xFA03AEFD, 0x7EDD3722, 0x0396FD03,
	0x7EDD1720, 0x0296FD02, 0x7EDD0F20, 0x0196FD01, 0x7EDD0720, 0x0096FD00, 0xAEFD9FC8, 0xD89F8703, 0x7EDDC93C, 0xE5F61803, 0x7EDDE1DD, 0x18C8B704, 0xE1DDE5F1, 0xEE037EDD, 0x0377DD80, 0x3821FEC9,
	0x5E213E02, 0x4E235623, 0xEB696623, 0x0001FACB, 0x4F0B1800, 0x7947B578, 0x5A636C4D, 0x08D60016, 0x08C6F130, 0xCB3ACBC8, 0xCB1CCB1B, 0x3D19CB1D, 0x20C9F320, 0xB47B5717, 0x7AC8B1B5, 0x1A3808D6,
	0x655C53C8, 0x14000E69, 0xF8F12815, 0x21CBC83D, 0x13CB6AED, 0x8CF212CB, 0xC9AFC937, 0xDDE1DDE5, 0x5E470470, 0x7E235623, 0xEB6F6623, 0x15047EDD, 0x3773F414, 0xCB0477DD, 0x2C123021, 0x20240F20,
	0xB37A130C, 0x34DD0720, 0x161F2804, 0x7FF67880, 0x0377DDA2, 0xDD0273DD, 0x75DD0174, 0xE1E5DD00, 0xDDAFC937, 0xF5180477, 0xE5DD0006, 0x7FF678E1, 0xF60377DD, 0x0477DDFF, 0x0177DD77, 0xC90277DD,
	0xC3C3C3FF, 0xFFC3C3C3, 0xC0C0C0FF, 0xC0C0C0C0, 0x18181818, 0xFF181818, 0x03030303, 0xFF030303, 0x7E30180C, 0x0030180C, 0xDBE7C3FF, 0xFFC3E7DB, 0x06030100, 0x003078CC, 0xC3C3663C, 0x00E724FF,
	0x60300000, 0x003060FF, 0x060C0000, 0x000C06FF, 0x18181818, 0x183C7EDB, 0xDB7E3C18, 0x18181818, 0x993C5A18, 0x183C7EDB, 0x63330300, 0x003060FE, 0xDBFF663C, 0x3C66FFDB, 0xDBC3663C, 0x3C66C3DB,
	0xFFC3C3FF, 0xFFC3C3C3, 0xDBDB7E3C, 0x3C66C3DF, 0xDFC3663C, 0x3C7EDBDB, 0xFBC3663C, 0x3C7EDBDB, 0xDBDB7E3C, 0x3C66C3FB, 0x1E330100, 0x00317BCE, 0x6666667E, 0xE7666666, 0xFF030303, 0x00030303,
	0x183C66FF, 0xFF663C18, 0x3C3C1818, 0x18183C3C, 0x3066663C, 0x00180018, 0xFFC3663C, 0x3C66C3C3, 0xDBDBDBFF, 0xFFC3C3FB, 0xFBC3C3FF, 0xFFDBDBDB, 0xDFC3C3FF, 0xFFDBDBDB, 0xDBDBDBFF, 0xFFC3C3DF,
	0x00000000, 0x00000000, 0x18181818, 0x00180018, 0x006C6C6C, 0x00000000, 0x6CFE6C6C, 0x006C6CFE, 0x3C583E18, 0x00187C1A, 0x18CCC600, 0x00C66630, 0x76386C38, 0x0076CCDC, 0x00301818, 0x00000000,
	0x3030180C, 0x000C1830, 0x0C0C1830, 0x0030180C, 0xFF3C6600, 0x0000663C, 0x7E181800, 0x00001818, 0x00000000, 0x30181800, 0x7E000000, 0x00000000, 0x00000000, 0x00181800, 0x30180C06, 0x0080C060,
	0xD6CEC67C, 0x007CC6E6, 0x18183818, 0x007E1818, 0x3C06663C, 0x007E6660, 0x1C06663C, 0x003C6606, 0xCC6C3C1C, 0x001E0CFE, 0x7C60627E, 0x003C6606, 0x7C60663C, 0x003C6666, 0x0C06667E, 0x00181818,
	0x3C66663C, 0x003C6666, 0x3E66663C, 0x003C6606, 0x18180000, 0x00181800, 0x18180000, 0x30181800, 0x6030180C, 0x000C1830, 0x007E0000, 0x00007E00, 0x0C183060, 0x00603018, 0x0C66663C, 0x00180018,
	0x0C781060, 0x0076CC7C, 0x66663C18, 0x0066667E, 0x7C6666FC, 0x00FC6666, 0xC0C0663C, 0x003C66C0, 0x66666CF8, 0x00F86C66, 0x786862FE, 0x00FE6268, 0x786862FE, 0x00F06068, 0xC0C0663C, 0x003E66CE,
	0x7E666666, 0x00666666, 0x1818187E, 0x007E1818, 0x0C0C0C1E, 0x0078CCCC, 0x786C66E6, 0x00E6666C, 0x606060F0, 0x00FE6662, 0xFEFEEEC6, 0x00C6C6D6, 0xDEF6E6C6, 0x00C6C6CE, 0xC6C66C38, 0x00386CC6,
	0x7C6666FC, 0x00F06060, 0xC6C66C38, 0x0076CCDA, 0x7C6666FC, 0x00E6666C, 0x3C60663C, 0x003C6606, 0x18185A7E, 0x003C1818, 0x66666666, 0x003C6666, 0x66666666, 0x00183C66, 0xD6C6C6C6, 0x00C6EEFE,
	0x38386CC6, 0x00C6C66C, 0x3C666666, 0x003C1818, 0x188CC6FE, 0x00FE6632, 0x3030303C, 0x003C3030, 0x663C0000, 0x18083E60, 0x0C0C0C3C, 0x003C0C0C, 0x00002418, 0x00000000, 0x00000000, 0xFF000000,
	0x000C1830, 0x00000000, 0x0C780000, 0x0076CC7C, 0x667C60E0, 0x00DC6666, 0x663C0000, 0x003C6660, 0xCC7C0C1C, 0x0076CCCC, 0x663C0000, 0x003C607E, 0x7830361C, 0x00783030, 0x663E0000, 0x7C063E66,
	0x766C60E0, 0x00E66666, 0x18380018, 0x003C1818, 0x060E0006, 0x3C666606, 0x6C6660E0, 0x00E66C78, 0x18181838, 0x003C1818, 0xFE6C0000, 0x00C6D6D6, 0x66DC0000, 0x00666666, 0x663C0000, 0x003C6666,
	0x66DC0000, 0xF0607C66, 0xCC760000, 0x1E0C7CCC, 0x76DC0000, 0x00F06060, 0x603C0000, 0x007C063C, 0x307C3030, 0x001C3630, 0x66660000, 0x003E6666, 0x66660000, 0x00183C66, 0xD6C60000, 0x006CFED6,
	0x6CC60000, 0x00C66C38, 0x66660000, 0x7C063E66, 0x4C7E0000, 0x007E3218, 0x663C0806, 0x003C607E, 0x66660830, 0x003E6666, 0x663C1060, 0x003C607E, 0x0000DC76, 0x00000000, 0x33CC33CC, 0x33CC33CC,
	0x00000000, 0x00000000, 0xF0F0F0F0, 0x00000000, 0x0F0F0F0F, 0x00000000, 0xFFFFFFFF, 0x00000000, 0x00000000, 0xF0F0F0F0, 0xF0F0F0F0, 0xF0F0F0F0, 0x0F0F0F0F, 0xF0F0F0F0, 0xFFFFFFFF, 0xF0F0F0F0,
	0x00000000, 0x0F0F0F0F, 0xF0F0F0F0, 0x0F0F0F0F, 0x0F0F0F0F, 0x0F0F0F0F, 0xFFFFFFFF, 0x0F0F0F0F, 0x00000000, 0xFFFFFFFF, 0xF0F0F0F0, 0xFFFFFFFF, 0x0F0F0F0F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
	0x18000000, 0x00000018, 0x18181818, 0x00000018, 0x1F000000, 0x0000001F, 0x1F181818, 0x0000000F, 0x18000000, 0x18181818, 0x18181818, 0x18181818, 0x0F000000, 0x1818181F, 0x1F181818, 0x1818181F,
	0xF8000000, 0x000000F8, 0xF8181818, 0x000000F0, 0xFF000000, 0x000000FF, 0xFF181818, 0x000000FF, 0xF0000000, 0x181818F8, 0xF8181818, 0x181818F8, 0xFF000000, 0x181818FF, 0xFF181818, 0x181818FF,
	0xC66C3810, 0x00000000, 0x0030180C, 0x00000000, 0x00384438, 0x00000000, 0xF860663C, 0x00FE6660, 0xA2BA4438, 0x003844BA, 0x74F4F47E, 0x00343434, 0x6C38301E, 0x00F01838, 0x000C1818, 0x00000000,
	0x4C44C040, 0x00041E54, 0x524CC040, 0x001E0844, 0x166210E0, 0x00020FEA, 0x7E181800, 0x007E1818, 0x7E001818, 0x00181800, 0x7E000000, 0x00000606, 0x30180018, 0x003C6666, 0x18180018, 0x00181818,
	0xDE730000, 0x0073DECC, 0xFCC6C67C, 0xC0F8C6C6, 0x3C666600, 0x003C6666, 0x3C60603C, 0x003C6666, 0x301E0000, 0x001E307C, 0xFEC66C38, 0x00386CC6, 0x3060C000, 0x00C66C38, 0x66660000, 0x60607C66,
	0xFE000000, 0x006C6C6C, 0x7E000000, 0x0070D8D8, 0x3C0C0603, 0xC0603C66, 0x660C0603, 0xC0603C66, 0x183CE600, 0x00C76C38, 0xC3660000, 0x007EDBDB, 0x3060C6FE, 0x00FEC660, 0xC6C67C00, 0x00EE6CC6,
	0xC0603018, 0x00000080, 0x03060C18, 0x00000001, 0x01000000, 0x180C0603, 0x80000000, 0x183060C0, 0xC3663C18, 0x00000081, 0x03060C18, 0x180C0603, 0x81000000, 0x183C66C3, 0xC0603018, 0x183060C0,
	0xC1603018, 0x180C0683, 0x83060C18, 0x183060C1, 0xC3663C18, 0x183C66C3, 0x3C7EE7C3, 0xC3E77E3C, 0x1C0E0703, 0xC0E07038, 0x3870E0C0, 0x03070E1C, 0x3333CCCC, 0x3333CCCC, 0x55AA55AA, 0x55AA55AA,
	0x0000FFFF, 0x00000000, 0x03030303, 0x03030303, 0x00000000, 0xFFFF0000, 0xC0C0C0C0, 0xC0C0C0C0, 0xF8FCFEFF, 0x80C0E0F0, 0x1F3F7FFF, 0x0103070F, 0x0F070301, 0xFF7F3F1F, 0xF0E0C080, 0xFFFEFCF8,
	0x55AA55AA, 0x00000000, 0x050A050A, 0x050A050A, 0x00000000, 0x55AA55AA, 0x50A050A0, 0x50A050A0, 0x50A854AA, 0x008040A0, 0x152A55AA, 0x0102050A, 0x0A050201, 0xAA552A15, 0xA0408000, 0xAA54A850,
	0xFF99FF7E, 0x7EFFC3BD, 0xFF99FF7E, 0x7EFFBDC3, 0xFEFE3838, 0x003810FE, 0xFE7C3810, 0x0010387C, 0xFEFEFE6C, 0x0010387C, 0xFE7C3810, 0x003810FE, 0xC3663C00, 0x003C66C3, 0xFF7E3C00, 0x003C7EFF,
	0x66667E00, 0x007E6666, 0x7E7E7E00, 0x007E7E7E, 0x780D070F, 0x78CCCCCC, 0x6666663C, 0x187E183C, 0x0C0C0C0C, 0x387C3C0C, 0x1B1E1C18, 0x70F87818, 0xC3245A99, 0x995A24C3, 0x38383810, 0xD67C3838,
	0xFF7E3C18, 0x18181818, 0x18181818, 0x183C7EFF, 0xFF703010, 0x103070FF, 0xFF0E0C08, 0x080C0EFF, 0x3C180000, 0x00FFFF7E, 0xFFFF0000, 0x00183C7E, 0xFEF8E080, 0x0080E0F8, 0xFE3E0E02, 0x00020E3E,
	0x7C923838, 0x28282810, 0xFE103838, 0x82442810, 0x7C123838, 0x22242890, 0x7C903838, 0x88482812, 0x3C183C00, 0x00183C3C, 0x18FFFF3C, 0x1830180C, 0x187E3C18, 0x183C7E18, 0xFF662400, 0x00002466
};

static int ROMSUP[ 0x1000 ] = {
	0x03020180, 0x0031C040, 0xBCCBCDC0, 0xDAF53FCD, 0x32AF0000, 0x3321AC00, 0xC37DCDC0, 0xCDDEAACD, 0xBBCDCB37, 0xC0DECDBD, 0x11C145CD, 0xE9CD00F0, 0x202518F7, 0x49534142, 0x2E312043, 0x000A0A31,
	0x49534142, 0x48CD00C3, 0x0031C0CF, 0xE85CCDC0, 0xCDE254CD, 0x5F38CB01, 0xCDC00031, 0xB5CDC166, 0xBCB6DCDE, 0xCDC4D0CD, 0x2C3AC3D0, 0x45C4B7AE, 0xAD903AC1, 0x092002D6, 0xCDAD9032, 0x38EBCBAA,
	0xC0D721C9, 0xCDC38BCD, 0x013ADEAA, 0x1F28B7AC, 0x30C10DCD, 0xDE4DCDC3, 0x30EECFCD, 0xDE4DCD0A, 0x64CC37B7, 0xD4E330E8, 0x8A21C0DE, 0xCD0818AC, 0xFB30CAF9, 0xCDC398CD, 0x28B7DE4D, 0xEECFCDCA,
	0x4DCD0B30, 0xE7A5CDFB, 0x18C18FCD, 0xDFA4CDBA, 0x2BC4D3CD, 0x52DE60C3, 0x79646165, 0x18AF000A, 0xAC022205, 0x0132FF3E, 0x0A11C9AC, 0xFE022800, 0xCF48C42C, 0x000A11D5, 0xDCDE41CD, 0x37CDCF48,
	0x0422EBDE, 0xE1CDE1AC, 0x87C3C1C0, 0xAC022AC0, 0x38CDD5EB, 0xC0DECDE2, 0xD1CB01CD, 0x042AE5D0, 0xE1D419AC, 0xC937E1C0, 0xC145CDC0, 0xFEC058C3, 0xE50C28A3, 0xCDC178CD, 0x8FCDC15F, 0xCDC9E1C1,
	0x3DC3DE2C, 0xAE622ABD, 0xAE5E2AEB, 0x62FFE4CD, 0x77AF136B, 0x2C32B0ED, 0xD5EACDAE, 0xCDC16FCD, 0xCDAFD300, 0xCCCDBD97, 0xDA20CDFB, 0xCDC1A1C3, 0xDECDDEC5, 0xC189CDC0, 0x8CCDE5C5, 0xD5EACDF6,
	0xCDD638CD, 0xC1E1EA4D, 0xF299CDC9, 0xCDE761CD, 0x7ECDCCAC, 0xC9A3CDCC, 0xCDF64FCD, 0xD4C3D60E, 0xB3CDAFDC, 0xF5E5AFC1, 0xB4DC08FE, 0x0621F1BB, 0xE50418AC, 0xD5AC0721, 0xD1737E5F, 0x063AC9E1,
	0xC908FEAC, 0xFEAC073A, 0xFBCDC909, 0xCDD718C1, 0x1818C1FB, 0xCDC1FBCD, 0xF5C1C1B3, 0xCDC1C4CD, 0x18F1C1ED, 0xC1FBCDCE, 0x313008FE, 0xC1A6CDC1, 0x2CFE7EF5, 0xF1FFFCCD, 0x7EC1A6C3, 0x003E23FE,
	0xC20DCDC0, 0xDE41CDF5, 0xF1DE37D4, 0xDE25CDC9, 0xC50A3E23, 0xB8CD47D5, 0xC1D1B8CE, 0xCB4DC3D8, 0xEF18023E, 0x01C1E5CD, 0x3FC4BB90, 0xDE41CDC2, 0xC220CDD0, 0x18BB9F01, 0xC1E5CD09, 0xCDBB9601,
	0xCDE5C271, 0xC9E1FFFC, 0xE5C262CD, 0xE1BC38CD, 0xC271CDC9, 0xDE15CDF5, 0xF1C262CD, 0xBC32CDE5, 0x6ACDC9E1, 0x41CD41C2, 0x203ED0DE, 0x4FC213CD, 0x18103EC9, 0xCD033E9E, 0xCDE5C213, 0xC9E1BC0E,
	0xE5C1E5CD, 0xE1BB6CCD, 0xC20DCDC9, 0x8D3008FE, 0xDE1DCDF5, 0xC1ECC3F1, 0xCDC289CD, 0x78C3BB60, 0xC289CDFA, 0xC2C7CDE5, 0x0DCD0A18, 0xC290CDC2, 0xC2B9CDE5, 0xE1FF32CD, 0xC1BECDC9, 0xC8AC083A,
	0xD0AC0A3A, 0xCDC3ECC3, 0x87CDBB78, 0xCDC97DBB, 0x0D28C1BE, 0xCDE5D5D0, 0x947ABB69, 0x37D1E13C, 0xAC093AC9, 0xE5C9FFFE, 0xC2CFCD67, 0x6F0E383F, 0x3DC2B9CD, 0x84062837, 0x3D02303F, 0xCDC9E1BD,
	0x51CDC1E5, 0x24EBE5C3, 0xBB75CD2C, 0xE7FEC9E1, 0xE5CD1628, 0xC351CDC1, 0xDE15CDD5, 0xE3C351CD, 0xCD6F557A, 0xC9E1BB66, 0xCDDE2CCD, 0xCD4FC33E, 0x003EDE41, 0x47C33EDC, 0xBBB7CDE5, 0x083EC9E1,
	0xCDC213C3, 0xFF3EC1E5, 0xE5CD0418, 0x63C3AFC1, 0xC358CDBB, 0xDE15CD53, 0xCEC3CDD5, 0xC91D5FD1, 0x28C1E5CD, 0xC220CD0A, 0xBB84CCB7, 0xCDBB81C4, 0xCDD0DE41, 0xCAB7C220, 0x7BC3BB7E, 0x0121E5BB,
	0xAC082284, 0xCDC469CD, 0xF5E1C1A1, 0xB7237EE5, 0x20C3A0C4, 0xC9F1E1F8, 0xCD0A3EF5, 0xC9F1C3A0, 0xA8CDC5F5, 0xC9F1C1C3, 0x0C200AFE, 0xCAC1BECD, 0x31D2C3F5, 0xC3E2C3C4, 0xCD4FC5F5, 0xF1C1C3C1,
	0xAC063AC9, 0xFCCA08FE, 0xC438D2C3, 0xC3E9C379, 0xBB63CDAF, 0x9FCDE5AF, 0x54CDE1BB, 0xC3ECCDBB, 0x0D3EC83D, 0x3EC3E9CD, 0xBB5AC30A, 0xC7CDE5C5, 0xC1E17CC2, 0xCD0D0EC9, 0x0A0EC3FC, 0xAC082AE5,
	0x32C411CD, 0x79E1AC08, 0xD8BD2BCD, 0x18C472CD, 0x0DEE79F6, 0xFE791028, 0x24D87D20, 0x20BC0828, 0xC398CD05, 0xC03C013E, 0xC3CDC93D, 0xAC0932CE, 0xCD0D0EC9, 0x0A0EC438, 0xAC0A2AE5, 0x11CDFF26,
	0xAC0A32C4, 0x95CD79E1, 0x1920D8BC, 0xE5CC37C3, 0x28BC89CD, 0xCD9F3FF7, 0xC9E1FF2D, 0xD8BC80CD, 0x0EEEEA28, 0xCB45CDC0, 0x32013E1F, 0xC3C9AC0A, 0x09CDBB09, 0xFCFED0BB, 0xC4A1CDC0, 0xE5CC3EDA,
	0x9211D5C5, 0x3AFD0EC4, 0xC4B7AC0B, 0xC1D1BB45, 0x09CDC9E1, 0xFE0430BB, 0xCDF720EF, 0xF2C3C4A1, 0xE5D5C5C8, 0xF5BCB6CD, 0x47BD40CD, 0xCDBB81CD, 0xEFFEBB06, 0x48CBF928, 0xFEBB84C4, 0x0B2837FC,
	0x0CC420FE, 0xDCF5F1BB, 0xE1B7BCB9, 0xC9C1D1E1, 0x3E0218AF, 0xAC0B32FF, 0xBB48CDE5, 0x8CCDA218, 0xCDD5C5C5, 0x1730DE41, 0xC5C58CCD, 0xDE15CDD5, 0xC5C58CCD, 0xBBD2CDE3, 0xCDE3D1E1, 0xD1E1BBCF,
	0xBBC9CDE3, 0x3DCDC9E1, 0xC5B4D4DE, 0xBBDBCDE5, 0x71CDC9E1, 0xCDF5E5C2, 0xFCCDFC64, 0x001D01F6, 0x3EFFDECD, 0xCB55DA07, 0x52CDF1EB, 0x01C9E1BD, 0x1718BBC0, 0x18BBC301, 0xBBF60112, 0xF9010D18,
	0x010818BB, 0x0318BBEA, 0xC5BBED01, 0xCDC58CCD, 0x0530DE41, 0xBAC42CFE, 0xDE41CDC5, 0x043E0A30, 0xE5C213CD, 0xE1BC59CD, 0xC1E3C5E3, 0xE1FFFCCD, 0xBBF001C9, 0xF3010318, 0x8CCDC5BB, 0xDE1DCDC5,
	0xC1E3C5E3, 0xCDFFFCCD, 0xC9E1FF32, 0xD5CED8CD, 0xCDDE15CD, 0x4B42CED8, 0xBAFEC9D1, 0x25CD1328, 0x2CFEBBDE, 0xCDC5BAC4, 0xCDD0DE41, 0x46C3C220, 0xDE2CCDBD, 0xC3C271CD, 0x71CDBBE4, 0xBBDEC3C2,
	0x06282CFE, 0xCDCEB8CD, 0x41CDBD4C, 0x20CDD0DE, 0xBD49C3C2, 0xE5D6ECCD, 0x76CDD5C5, 0xAC1222CA, 0xCDEBE5D5, 0x5DCCC6D9, 0x3DCDE1F6, 0x000011DE, 0x44D6BFD4, 0x7AE3E14D, 0xFFD8C4B3, 0xEBC69EC2,
	0xE3DEB1CD, 0xE1DEADCD, 0xC5D5E3F1, 0x160501E5, 0x010928B9, 0x3EB91002, 0xCB55C20D, 0xF672CD78, 0x23722373, 0xDE21CDE3, 0x79CF62CD, 0xE5FEFFCD, 0xCDAC0D21, 0xCDE1FF83, 0xCDECDE25, 0x79E3CF62,
	0xCDFEFFCD, 0xE3EBFF83, 0x000121EB, 0xEBFF35CD, 0x20E6FE7E, 0xDE2CCD06, 0x79CF62CD, 0xE3FEFFCD, 0xCDFF83CD, 0x77EBFDC4, 0xCDE1EB23, 0x73EBDE37, 0xEB237223, 0xEBDEB1CD, 0x23722373, 0x722373D1,
	0x125BED23, 0x722373AC, 0x21D17023, 0x87CDAC0D, 0x0C32AFFF, 0xADCDE1AC, 0xAC122ADE, 0x45CD0918, 0xFF3E01CB, 0xEBAC0C32, 0x20C6D9CD, 0x5DCDEBF1, 0xCDE5EBF6, 0x0F28C702, 0x235E23F1, 0x237E2356,
	0xADCD6F66, 0x01C9EBDE, 0x5E090005, 0xCDE15623, 0xCDEBF65D, 0xCF38DE41, 0xB06F2AC9, 0x23462BE5, 0x9F6F907D, 0x78E36784, 0x0F3807FE, 0x2BE50020, 0x6E2B7E2B, 0xFFD8CD67, 0xEB0420E1, 0xE1C978E1,
	0x235EDA18, 0xFEE52356, 0x012C2810, 0xEB790005, 0xE1FF6CCD, 0xB7AC0C3A, 0x09E51028, 0xE1FD0CCD, 0x2B562BE5, 0x83CDEB5E, 0x0EE5E1FF, 0xFD49CD05, 0x000A01E1, 0xEBC99609, 0x3A56235E, 0x28B7AC0C,
	0x23E5E316, 0x66237E23, 0xDD4ACD6F, 0x55D2063E, 0xE3E1EBCB, 0xE1732B72, 0x66E5237E, 0x02CDEB6F, 0x2323E1DE, 0xCDC99623, 0xA0FECF62, 0x25CD0428, 0xC4CDEBDE, 0xE95BCCFD, 0xDE3DCDC8, 0x281EFED8,
	0xC21DFE05, 0x27CDDE8F, 0xC9EBC0E8, 0xC0E827CD, 0xE5000EEB, 0x72CD063E, 0x732371F6, 0xEB237223, 0xEBDEB1CD, 0x23722373, 0xCD230636, 0xC9E1F65D, 0xC7CFCDC0, 0x4EF65DCD, 0x56235E23, 0x66237E23,
	0xDEADCD6F, 0x01FE79EB, 0xC951CAD8, 0x2AC961C3, 0x7E2BB06F, 0x6F967DF5, 0x2367849F, 0xC806FEF1, 0xCDEF20B7, 0xE503CB45, 0xE5CAC9CD, 0xAC1422EB, 0xCCC85DCD, 0x073EF65D, 0xEBF672CD, 0xEBDEB1CD,
	0x23722373, 0x722373D1, 0xEBE3EB23, 0x23722373, 0xCD230736, 0xD1EBF65D, 0xEBC02A18, 0x3EC85DCD, 0xCB55C21E, 0x000711E5, 0xF65DCD19, 0x22DEB1CD, 0x5EE1AC14, 0xEB235623, 0xEBDEADCD, 0x2356235E,
	0x6F66237E, 0xCF62CDD5, 0xD1FDC4CD, 0xAC142AC0, 0x3EDEADCD, 0xF662CD07, 0xEBF65DCD, 0xB06F2AC9, 0x967DE52B, 0x67849F6F, 0xFE7EE323, 0x200E3807, 0x2B2B2B0E, 0x676E2B7E, 0x20FFD8CD, 0xE1C9E102,
	0x9CFEDE18, 0xCDCCB8CA, 0x7E4FCEB8, 0x28F5A0FE, 0xDE25CD05, 0x2CCD2B9F, 0x0A280DDE, 0xCDCF48CD, 0xF538DE41, 0x27CDC9F1, 0xE9A3C4E8, 0xC790C2F1, 0x32AFC9EB, 0xFBCDAC16, 0x471D30BC, 0xE6AC163A,
	0xAC16327F, 0xFECDE5C5, 0x3AC1E1BC, 0xF517AC16, 0xBD01D478, 0xDE3017F1, 0xE6AC163A, 0xC47FC404, 0x3AAE1B2A, 0x03E6AC16, 0x3EDA1FC8, 0xC3F123CC, 0x1C22DE77, 0x30043EAC, 0xAC1A2A52, 0xB5C4B57C,
	0x30413EDE, 0xB9CDC546, 0x1711C1BC, 0x18020EAC, 0x25CDD522, 0x27CD9FDE, 0xD14B42E8, 0x000A21E5, 0x70237119, 0x2323C9E1, 0xB5CDEB23, 0x30403EDE, 0xD5010E1A, 0x2AC793CD, 0xE1EBAE1B, 0x23732370,
	0x235E2372, 0x1B22EB56, 0x21C23EAE, 0x77B6AC16, 0x5E237EC9, 0x01D55623, 0xCD09FFF7, 0x18E1BD01, 0x1C2A7E11, 0xFFFC01AC, 0xBD01CD09, 0x2AC47FCD, 0xC3F1AC18, 0x7CCDDE60, 0xDE2CC3C9, 0xD0CA8BFE,
	0x11CEFEC4, 0x08280000, 0x9FDE25CD, 0x2BE827CD, 0xAC1A53ED, 0xE5C4D3C3, 0xE1BD04CD, 0x07CDE5C9, 0xCDC9E1BD, 0x4221BCA7, 0xE50406AC, 0xE1BCECCD, 0x19001211, 0x48CDF510, 0xBCF5CDBB, 0x22000021,
	0x7FCDAC1A, 0xAC1E21C4, 0x01030511, 0xDACD0800, 0xAC4821C9, 0x01040B11, 0xD5C50201, 0x2611FD0E, 0xBCEFCDC9, 0x0016D5D1, 0x79C1D119, 0xCB0228B7, 0xE6201500, 0xDE19CDC9, 0xF5CEB8CD, 0xB7CA10CD,
	0x1DCD1D20, 0xC911CDDE, 0xCDEBE5F1, 0xC9E1BCB0, 0xAC1E111F, 0x2A111FD8, 0x111FD8AC, 0xC3D8AC36, 0xCECDCB4D, 0x000001CE, 0xCECD0518, 0xD54B42CE, 0xDE41CDC5, 0xDC000011, 0xCDEBCED8, 0x01E5CA62,
	0xEB090006, 0xD1C911CD, 0xCDEBE3C1, 0xC9E1BCE9, 0xCDFEB6CD, 0xECCDCA62, 0x110338BC, 0xC3EB0000, 0xB77CFF35, 0xFE7DB920, 0x87B43004, 0x87858787, 0xAC42016F, 0xCDEBC909, 0x2BEBDEB1, 0x1A0E0106,
	0xE5E9DDCD, 0xFEDE2CCD, 0xE10828B0, 0xEE209EFE, 0xF1EB1804, 0xB1CDE5EB, 0xADCDE3DE, 0x2805EBDE, 0xDE2CCD24, 0xD5C50E28, 0xD1D6BFCD, 0xDE41CDC1, 0xF2100230, 0x28B7782B, 0xB1CDEB0C, 0xADCDE3DE,
	0x18EBE1DE, 0x2CC3D1B9, 0xCDEB2BDE, 0x06EBDEB1, 0x1D0E0400, 0xE5E9DDCD, 0xE1DE2CCD, 0xF128D6FE, 0xEE20D5FE, 0x2CCDEC10, 0xDE2CC3DE, 0xD8CAF9CD, 0x31C1A1CD, 0x5DC3C000, 0xAC8A21DE, 0x5EC30036,
	0xAC8A21BD, 0xC3BD5ECD, 0x21C5C398, 0x06E5AC8A, 0x36F50E00, 0xC45CCD00, 0x0DFE1A30, 0x044F1028, 0x0AFE0410, 0x2377EB28, 0x45CDE910, 0xFE7917CB, 0x37DE280A, 0xAFC9C1E1, 0x32AD9132, 0xB1CDAD90,
	0xAD8C22DE, 0x187EE3C9, 0x18023E0C, 0x18053E08, 0xCEC3CD04, 0xCB3BCDC0, 0x22AE1B2A, 0x83CDAD8E, 0xF63CCDCC, 0x2AC00031, 0x6ECDAE19, 0xFBCCCDF6, 0xCDDA20CD, 0x962ACBAA, 0x9821EBAD, 0x7A0C30AD,
	0xA60828B3, 0xEB350520, 0x36DE77C3, 0xAD903A00, 0x2ACE8CCD, 0xADCDAD8C, 0xAD903ADE, 0x042020EE, 0x17AD913A, 0xC3CC04D4, 0x8C2AC058, 0xDEB8CDAD, 0x000021D8, 0x21E5D5C9, 0x0B1ECDF6, 0xE5D50718,
	0x1ECDBF21, 0x2AE5F506, 0xB57CAD96, 0x55C27BE1, 0xA6CDAFCB, 0xCDEBF5C1, 0xCDEBCE79, 0xCDF1C398, 0xE1F1C1A6, 0xD0CDC9D1, 0xCBF121C3, 0x18CC15CD, 0x646E551C, 0x6E696665, 0x6C206465, 0x20656E69,
	0xCC1C1100, 0xCDC1A1CD, 0x79CDC3D0, 0xDEB5CDCE, 0x2121EBD0, 0xC38BCDCC, 0xEF44C3EB, 0x61657242, 0x6E6920EB, 0xE5C00020, 0xE1CC01CD, 0x18CC66CD, 0x66CDC032, 0x322318CC, 0x45CDAD91, 0x01CD20CB,
	0xAE1B2ACC, 0x18CC83CD, 0xDEB5CD1A, 0x7ECD1230, 0xAD983ACC, 0xC2133EB7, 0xEDCDCB55, 0xD2F5CDD2, 0xC3DEAACD, 0xCDEBC058, 0xD0EBDEB5, 0x2801FE7E, 0x237E230B, 0x230728B6, 0x23DEADCD, 0x00210518,
	0xEB0C1800, 0xD0DEB5CD, 0x22DEB1CD, 0x22EBAD94, 0xC0C9AD92, 0x7CAD922A, 0xCA113EB5, 0x2AE5CB55, 0xADCDAD94, 0xBCB9CDDE, 0xDE60C3E1, 0xAD9832AF, 0xED000011, 0xC9AD9653, 0xCDDE2CCD, 0xCDA0DE25,
	0xCDE5CF48, 0xE1EBE85C, 0xB0CDE918, 0xAD983ACC, 0x64C3C8B7, 0xFE1128CB, 0xCD1428B0, 0xCDC0E827, 0x23EBCCFA, 0xDE77C3F1, 0xF1CCFACD, 0xCDDE60C3, 0xCDC0DE2C, 0xC323CCFA, 0x983AE9A3, 0x143EB7AD,
	0xAFCB55CA, 0x32AD9032, 0x8C2AAD98, 0xDEADCDAD, 0xC9AD8E2A, 0xEE69E570, 0xF865F265, 0x20EE6F69, 0x6F63F566, 0xEE6172ED, 0x6DEF2074, 0x6574EE65, 0x4E4557E4, 0xEE006FC4, 0x656C6946, 0x746F6EA0,
	0x6F6F74A0, 0x696D20A0, 0x65014CF3, 0x58454EA0, 0xA00473D4, 0x20EC6C05, 0xF26F7202, 0x55534552, 0x7453C54D, 0xA0670172, 0x616D0620, 0x7310E46E, 0x6E55E701, 0x776F6E6B, 0x6F6C0FEE, 0x6C61E76E,
	0x64616572, 0x6E55A079, 0x63657003, 0x7269A00A, 0x98746365, 0x921D951A, 0x746E7953, 0x1D957861, 0x55544552, 0x4144CE52, 0x03204154, 0x73756168, 0x706D498A, 0x72006F72, 0x67726120, 0x4FF40975,
	0x6C660276, 0x654DF76F, 0x79726F6D, 0x6F641194, 0x0E207365, 0xF4736903, 0x73627553, 0x70697263, 0x66087508, 0xE5670720, 0x61727241, 0x641C2079, 0x04730969, 0x6944E465, 0x62136976, 0x027A2079,
	0x766E49EF, 0x64696C61, 0x549E6420, 0x6D100079, 0xE8637461, 0x61707317, 0x17946563, 0x7003179B, 0x13736572, 0x6C70060F, 0x6E614383, 0x4E4F430E, 0xE5750154, 0x7375201A, 0x636E0502, 0x99168474,
	0x1E44961D, 0x756F6620, 0x004FE46E, 0x11996407, 0x464F459B, 0xF4656D20, 0x0079740D, 0x0D991295, 0x981A8C1C, 0x8B1D990B, 0x428C0E0D, 0x656B6F72, 0x1181206E, 0x92CDCD14, 0xE61AD5CE, 0xD420FE7F,
	0x73DCC3A0, 0x131AD1CE, 0xC9EE3017, 0xFECD9411, 0xC8B7D021, 0x131A47C5, 0x10FB3017, 0xD0C9C1F9, 0xC7C7C7C7, 0xC7C7C7C7, 0xC7C7C7C7, 0xC7C7C7C7, 0xC7C7C7C7, 0xC7C7C7C7, 0xF5CED8CD, 0x0B20B77A,
	0xCDC97BF1, 0x7AF5CED8, 0xC3F120B3, 0xD8CDCB4D, 0x177AF5CE, 0xC9F1F538, 0xF5CF62CD, 0xFEB6CDEB, 0xCDC9F1EB, 0x66CDCF62, 0xE50D20FF, 0xCDB0A02A, 0xE1EBFB58, 0xCF62CDC9, 0xCDE5C5F5, 0xE1EBFEEB,
	0xCDC9F1C1, 0xF5C3CF62, 0xCF62CDFB, 0x01FF5EC3, 0xFF110001, 0xDE41CDFF, 0xD8DE3DD4, 0xFEC823FE, 0xCD0A28F5, 0x4B42CF48, 0xDE41CDC8, 0xDE25CDD8, 0xFFFF11F5, 0xDE41CDC8, 0xCF48CDD8, 0xEBDE41C4,
	0xDAFFDECD, 0xC9EBCB4D, 0x235E237E, 0x281EFE56, 0xC21DFE0E, 0xEBE5CB49, 0x5E232323, 0xC3E15623, 0x06C5DE2C, 0xCF6DCD00, 0x2CC32BC1, 0xCDC52BDE, 0xE1E5D033, 0xEEFE7EC1, 0xD0FEFED8, 0x4538F4FE,
	0x20FF66CC, 0x2AE5C512, 0xCDE3B0A0, 0x5ECDD033, 0x1DCDE3FF, 0x7EDC18F9, 0x875FF4D6, 0x5FEDC683, 0x5793CFCE, 0xEBBE78EB, 0x74CDC5D0, 0x1AC5D5FF, 0xCF6ECD47, 0x7E23E3C1, 0xEB6F6623, 0xF662CD79,
	0x18FFFECD, 0x0AFE78AE, 0xD67EC5D0, 0x66CD47ED, 0xD00B11FF, 0x2AE5D420, 0xC5E3B0A0, 0x6ECD0A06, 0xC5E3C1CF, 0xC1F93FCD, 0x18D013CD, 0xFD0C0C86, 0x12FD210C, 0x5212FD35, 0xD53616FD, 0x06FD6710,
	0x790EFD87, 0xFD9204FD, 0x0AFD9C02, 0xCDC5D00E, 0xC6C1FD49, 0xC6A08F01, 0x2DC39FFF, 0xCD1406FF, 0xCDE5CF6D, 0xC9E1FDB4, 0x6DCD0806, 0xA6CDE5CF, 0xCDC9E1FD, 0x1D28DE2C, 0x38380EFE, 0x523820FE,
	0x79CA22FE, 0xCAFFFEF8, 0x21E5D0DA, 0xB4CDD059, 0x2CC3E3FF, 0xCB45CDDE, 0xD0D70816, 0xF4D01DF5, 0xD128D036, 0xD028FED0, 0xE4D142E3, 0xE2ACD18A, 0xD14E40F9, 0x30D6C9CD, 0x2803FE0B, 0xCDEBE50F,
	0xC9E1FF6C, 0x1BC203FE, 0xD09111FF, 0xB0A053ED, 0x0ED600C9, 0xFE00165F, 0x231B380A, 0x280BFE5E, 0xFE562315, 0xFE0F380F, 0x20123811, 0x053E2B2A, 0x2BFF6CCD, 0xCDEB1818, 0x18EBFF35, 0x0FFEE511,
	0xEB130720, 0x235E2323, 0x89CDEB56, 0x2CC3E1FE, 0xCF62CDDE, 0xC3DE1DC3, 0x4E23CB49, 0x79DE2CCD, 0x053840FE, 0x10DA4AFE, 0xDE19CDD1, 0x1EC68779, 0x3059FE4F, 0x381DFEE2, 0xD0D1CD09, 0xD102CDE5,
	0xE511C9E1, 0x06EBE5D1, 0x237E0900, 0xC9E36F66, 0x97114F87, 0x4FEE18D0, 0x48D130C4, 0x1DFA7ED1, 0x39D5C1D5, 0x68D161D1, 0x3AD12BD1, 0x0318AD91, 0xE5AD903A, 0xE1FF32CD, 0x0DCDE5C9, 0xFEA5CDBD,
	0xCDE5C9E1, 0x1418CBAA, 0xAE5E2AE5, 0xC9CD0E18, 0xCB4DD2D6, 0xFE78EBE5, 0xFB58CC03, 0xE1FE89CD, 0xC6CDE5C9, 0x0418EBBB, 0xBBC6CDE5, 0xE1FF35CD, 0xDE25CDC9, 0xB5CDEBE4, 0x0C3EEBDE, 0xCDCB55D2,
	0x73EBD6DB, 0xC3EB7223, 0xDBCDE9A3, 0xEBE5C5D6, 0xEB56235E, 0x123EB57C, 0xCDCB55CA, 0xFE7EDA2A, 0xCD282028, 0xCDE3DE2C, 0xCDE3DE19, 0xD5E3DA6A, 0xE3CF62CD, 0xD69FCD78, 0xDE41CDE1, 0xCDE30630,
	0xE718DE15, 0xE3DE1DCD, 0xCDDE1DCD, 0x21CDDA49, 0xCF62CDDE, 0xCDCB49C2, 0x8ACCFF66, 0xDA52CDFB, 0xFFC3F1E1, 0x8FF964FE, 0xE5F969F9, 0x43F9D3FA, 0xAAD23FD2, 0x6AF9D8C2, 0x71FA8DD2, 0x98C576C5,
	0xB0C2A1C2, 0x7EFA6EFD, 0xB6FA74D5, 0x14D574FE, 0x0ED560FF, 0x56FC53FE, 0x13F219D4, 0x69D470FE, 0x65D56AFA, 0x08F8ECD5, 0x2ACA50F2, 0xADD56FFF, 0x31D37BFA, 0x79F9BCD5, 0xFAFEEBD5, 0x06FABEF8,
	0x060218FF, 0xCF62CD01, 0xD2DE41CD, 0x74CDDE1D, 0xCF62CDFF, 0x62CD79E5, 0xCDE5C5F6, 0xC1E1FD49, 0xB80428B7, 0xE1FF6FC4, 0x62CDDE18, 0xFF74CDCF, 0x11DE41CD, 0xD8DC0000, 0xDE1DCDCE, 0x2721D5E5,
	0x4F111900, 0xFFD8CD00, 0xD1CB4DD2, 0xF662CD79, 0xFDD5CD43, 0xE5C0C9E1, 0xCDD300CD, 0x9BCDF72A, 0xCC37CABC, 0xF761C3E1, 0xCDD2C7CD, 0x69CDF725, 0xBC8CC3C4, 0xFED2BECD, 0x45CDC816, 0xC7CD19CB,
	0xF720CDD2, 0xCDBC77C3, 0x03CDF72A, 0xCDEBE3CF, 0x37CAD2DB, 0xCDD8E1CC, 0xD51BCB45, 0x0A28B778, 0x3E21FE7E, 0x23032000, 0x6BC32F05, 0x7ACDE5BC, 0x59C3E1BC, 0x8FCDE5F7, 0xCC37CABC, 0xF75DC3E1,
	0xCDE5D5C5, 0x59CDBC7D, 0xBC92CDF7, 0xE1F75DCD, 0xCDC9C1D1, 0x9932CEB8, 0xDE15CDAD, 0xEDD44CCD, 0xCDAD9C53, 0x1411DE41, 0xCED8DC00, 0xADA053ED, 0xCD100C01, 0x9F32D35F, 0xCD000EAD, 0x9A32D35F,
	0xD35FCDAD, 0x06AD9B32, 0xD35FCD20, 0xCDAD9E32, 0x21E5DE37, 0xAACDAD99, 0xF1D8E1BC, 0xCDDE5DC3, 0xD079DE41, 0x792CFE7E, 0xCEB8CDC8, 0x2B18D8B8, 0x69CD0806, 0xB3CDE5D3, 0xCDC9E1BC, 0xB77DFEB6,
	0x1F06381F, 0x301F0338, 0x0F20B412, 0xBCADCD7D, 0xCDFF32C3, 0x877BCED8, 0xC3C8BA9F, 0xC3CDCB4D, 0x3010FECE, 0xB711F5F6, 0xD425CDD3, 0xA221E5F1, 0xBCCD71AD, 0x7EC9E1BC, 0x1120EFFE, 0x06DE2CCD,
	0xD369CD10, 0xCD4F80F6, 0xF5C3DE15, 0xCD8006CE, 0x4018D369, 0x7AD393CD, 0x02287BB7, 0xB75F3C2F, 0x10FEB928, 0x11D5B530, 0x25CDD3FD, 0x21E5D1D4, 0xE67AADA2, 0x7B77B180, 0xE1BCBFCD, 0xEFFE7EC9,
	0x2CCD0D20, 0xD44CCDDE, 0x4FF0C67A, 0x060D1843, 0xD369CDF0, 0xDE15CD4F, 0x43D393CD, 0xCDDE15CD, 0x5857CEB8, 0x050001C9, 0x30DE41CD, 0xCDC5D51C, 0xC179FFFE, 0xA321E5C5, 0x090006AD, 0x23770909,
	0xE1722373, 0x10D10CC1, 0xDE37C3DF, 0x7ACED8CD, 0x9BC2F0E6, 0xB6CDC9D3, 0x005011FE, 0x30FFD8CD, 0x1ECD7D22, 0xFFFF21BB, 0x26690328, 0xFF35C300, 0xEBBB24CD, 0x7CFEB6CD, 0x530228B5, 0x7AB57C2B,
	0xC3FF32CA, 0x8DFECB4D, 0xB8CD1628, 0x15CDF5CE, 0xCF03CDDE, 0xE547F148, 0xBB0FCDEB, 0xC9E430E1, 0x06DE2CCD, 0xD369CD50, 0xDE15CD4F, 0x69CD0206, 0x479F1FD3, 0xCD79E5C5, 0xC1E1BB39, 0xCDBB2711,
	0x2D11D4CB, 0xD4CBCDBB, 0xCDBB3311, 0xD5D0DE41, 0x47CEB8CD, 0xFBCD79E3, 0xFEC9E1FF, 0xFE2628D9, 0xBB3F01A4, 0xA2FE0828, 0xC2BC3E01, 0xCDC5CB49, 0xC3CDDE2C, 0x15CD4FCE, 0xCEC3CDDE, 0xEBC1515F,
	0xEBFFFCCD, 0xDE2CCDC9, 0x69CD0206, 0xA721E5D3, 0x323E3D00, 0x0F290228, 0xE1BC68CD, 0x41CDE5C9, 0xFF45CDFF, 0xE1BD9ACD, 0x18FF3EC9, 0x97C3AF01, 0xBD9D01BD, 0xC5E51618, 0xEBFF14CD, 0xCDADB221,
	0xE3C1BD61, 0xFF6CCD79, 0xBDA001D1, 0xD8D559CD, 0xFACBB5CA, 0x4DC3CBBE, 0xCDD5C5CB, 0xC9D1FF14, 0x18BDA901, 0xBDA601E7, 0xA301E218, 0x01DD18BD, 0xD818BDAC, 0x18BDAF01, 0xBDB201D3, 0xB501CE18,
	0x52C918BD, 0x6F646E61, 0x756E206D, 0x7265626D, 0x65657320, 0x203F2064, 0xCD062800, 0x18E5CF62, 0x8321E518, 0xC38BCDD5, 0xCDCAECCD, 0x6FCDC398, 0xCDEF30ED, 0x20B7DE4D, 0xFF14CDE9, 0xE1BDBECD,
	0x28FE7EC9, 0x2CCD1B20, 0xCF62CDDE, 0xE5DE1DCD, 0xCDFF14CD, 0x0520BD94, 0xE1BD8BCD, 0xBDBEFCC9, 0x3ECDE5E1, 0xBD7FCDFF, 0xFACDC9E1, 0xAE662AD5, 0x22AE6822, 0x6C22AE6A, 0xB721C9AE, 0xCD363EAD,
	0xED21D607, 0x36063EAD, 0x203D2300, 0x0021C9FA, 0xADEB2200, 0x3EEA4DC3, 0x684BED5B, 0xC6870BAE, 0xADCE6F35, 0xEDC96795, 0x0BAE6A4B, 0x873D03E6, 0xCE6FEDC6, 0xC96795AD, 0x1E415A01, 0x38907905,
	0x213CE53D, 0x0006ADB2, 0x3D2B7309, 0xC9E1FB20, 0x0618031E, 0x0218021E, 0xCD7E051E, 0x1E30FF92, 0x2CCD474F, 0x202DFEDE, 0xDE2CCD0C, 0x30FF92CD, 0x2CCD4F0D, 0xD63DCDDE, 0x38DE41CD, 0x49C3C9DD,
	0xCB45CDCB, 0xCB45CD09, 0xCAF8FE0A, 0xBFCDF245, 0x21CDD5D6, 0xCF62CDDE, 0x9FCDE378, 0x47C9E1D6, 0xB8FF4BCD, 0xFEFFC478, 0xC2FF66CD, 0xCDE5FF83, 0xC3D1FB94, 0xE0CDFF87, 0xDE41CDD7, 0xCDC9F838,
	0x06CDD931, 0x184238D8, 0xD931CD28, 0x38D806CD, 0xCD79E538, 0x17CDD619, 0xCD2D18D7, 0x2138D931, 0xD617CDE5, 0xD4D732CD, 0x1C18D76F, 0x38D931CD, 0xCD79E510, 0x17CDD619, 0xB09F3AD7, 0x18D77BD4,
	0x682AE507, 0xEB192BAE, 0xB09F3AE1, 0xCDC94F47, 0x7ACDD931, 0xD5F218E9, 0xAE122AE5, 0x1028B57C, 0x01C52323, 0x40CD0000, 0x0430C1D7, 0xC937F1F1, 0xE5D5D1E1, 0xE1D740CD, 0xC9D10238, 0xD79EC3E1,
	0x6F66237E, 0xE509C8B4, 0x5BED2323, 0xBE1AAE0E, 0x13230D20, 0x3AF73017, 0xAE3DB09F, 0xE1EB07E6, 0x3713DE20, 0x235D54C9, 0x237ECB23, 0x3EC9FB28, 0xD77BCD02, 0x40F61A1B, 0xD5C91312, 0xCDF5C5E5,
	0x2AF5D7A8, 0xCDEBAE6A, 0x1ACDF6B8, 0xB8CDF1F6, 0x2BAFC1D7, 0xC1FC1077, 0xD7D0CDE3, 0x7B23E1D1, 0x7A237791, 0xC9377798, 0x2A4F03C6, 0x47AFAE0E, 0x7ECB3C03, 0xC9F92823, 0xE5096B62, 0x2A1313D5,
	0xECCDAE0E, 0xB09F3AFF, 0x4213123D, 0xC9E1D14B, 0x917B127E, 0xF57E2377, 0xF177987A, 0xC9131213, 0x7ED931CD, 0x052828FE, 0x49C25BEE, 0xD883CDCB, 0x9F3AC5E5, 0xD627CDB0, 0xDAD740CD, 0x3EC1D685,
	0xD8B3CDFF, 0x7EF5C9E1, 0x102828FE, 0x0C285BEE, 0x2AE5D0F1, 0x192BAE68, 0xC937E1EB, 0xF1D883CD, 0x2A0730E5, 0x192BAE6A, 0xD5C51518, 0xCDB09F3A, 0x40CDD627, 0x130F30D7, 0x9ECDE113, 0x78EBC1D7,
	0xD681C296, 0xC1E10A18, 0xD8B3CDAF, 0xEBD79ECD, 0x46000011, 0x5ED5E523, 0x27CD5623, 0xFFD8CDD9, 0xE3D681D2, 0xD1DD72CD, 0x23E1EB19, 0xEBE61023, 0x9F3A4D44, 0x3803D6B0, 0x01282904, 0xEB190929,
	0xD5C937E1, 0x3ADE2CCD, 0x06F5B09F, 0xCECECD00, 0xCD023EE5, 0x2373F672, 0xCD04E172, 0xED38DE41, 0x2829FE7E, 0xC25DFE05, 0x2CCDCB49, 0x9F32F1DE, 0xE5C9D1B0, 0xC5AE0D32, 0x03C68778, 0xF5D7A8CD,
	0xEBAE6C2A, 0xF1F6B8CD, 0x60D7B8CD, 0x23D5C169, 0xB09F3A23, 0x7000165F, 0x3AD523E5, 0x11B7AE0D, 0xC4EB000A, 0x13EBD927, 0x23722373, 0xDD72CDE3, 0xEBD681DA, 0x42E310E1, 0xCD5D544B, 0x6C22F6BB,
	0x362BC5AE, 0xB1780B00, 0xE1C1F820, 0x29EB575E, 0x2BEB0923, 0x7223732B, 0x3AEBE323, 0x27CDB09F, 0xD7D0CDD6, 0x3EC9E1D1, 0xF662CD02, 0x6F66237E, 0xD9AFCDC9, 0x56235E23, 0x0A28B37A, 0x30177E23,
	0xDE2CCDFB, 0x2B2BC937, 0x0E2AC1EB, 0x5E21E5AE, 0xEBC5E5D9, 0xD96CCDE5, 0xAE0E53ED, 0x2AE5C9D1, 0x6ECDAE0E, 0x22E3E1F6, 0xC9E1AE0E, 0x23237EE5, 0xA9CB4E23, 0x380BFEE3, 0x1FE67917, 0xCE5FF2C6,
	0x1A5793AD, 0x36B09F32, 0x2805FE0D, 0x7709C603, 0xCD283ED1, 0x06E5F672, 0x49CA0529, 0xE6131ACB, 0x172377DF, 0x6ECDF330, 0xD12BEBF6, 0x7EDE2CC3, 0x02380BFE, 0x04FEF7C6, 0x04300928, 0x053002FE,
	0x3ECB49C3, 0xB09F3205, 0xD602CDC9, 0xEBAE6C2A, 0xCDAE6A2A, 0xD5C8FFD8, 0x7ED765CD, 0x3C07E623, 0xD627CDE5, 0xE1D7D0CD, 0x2356235E, 0xE318D119, 0xCDEA4DCD, 0x41CDD9FC, 0xC9F838DE, 0xE5D931CD,
	0xCDB09F3A, 0x40CDD627, 0xCB4DD2D7, 0x46234EEB, 0xE4CD0923, 0xF6E5CDFF, 0xCDF621CD, 0xC9E1D9C9, 0x22000021, 0x1022AE12, 0x2AE5C9AE, 0x3EEBAE10, 0xF672CD06, 0x73AE1022, 0xAF237223, 0x23772377,
	0xAE125BED, 0xE1722373, 0x102AE5C9, 0xAE1222AE, 0x102AC9E1, 0xF66ECDAE, 0x2356235E, 0xAE1053ED, 0x235E2323, 0x1222EB56, 0x3EE5C9AE, 0xF672CD02, 0xD9AFCDE3, 0xE3D96CCD, 0xAE102AEB, 0x00012323,
	0xD7D0CD00, 0x47B09F3A, 0xF672CD3C, 0x23773D78, 0x2AC9E1EB, 0xB57CAE10, 0x234E0E28, 0x01C52346, 0xE9CD0000, 0xEE18E1DA, 0xC51A4101, 0xD619CD79, 0xC1DAE9CD, 0x3EF4100C, 0xD627CD03, 0x234EE1E5,
	0xC8B17846, 0x2BAE6A2A, 0xCDD5E509, 0x23D1D765, 0x2346234E, 0x4EE309E5, 0x09000623, 0xDECDC109, 0xCDDA28FF, 0x1823DB02, 0x66237EF5, 0x09C8B46F, 0x65CDD5E5, 0x237ED1D7, 0x02FE07E6, 0xE1DB02CC,
	0xD5C5E718, 0x234E237E, 0xB7EBE546, 0xE1FFFBC4, 0xCDC9C1D1, 0xCDA3DE25, 0x8BCDC1D4, 0xD6BFCDDB, 0xE5FF5ECD, 0xDB31CDD5, 0xE1F88ACD, 0xE1D6A8CD, 0xC1C4CDC9, 0xCDDC57D2, 0x143ACAEC, 0xC43BFEAE,
	0xCDC9C398, 0x5BCDC1D4, 0xBFCDD5DB, 0xCDAFE3D6, 0xE323DBBD, 0x38DE41CD, 0xCDC9D1F1, 0x2B30C1C4, 0xDB8BCDE5, 0xDB37CDE5, 0xCDCDE1EB, 0xC5D8C1DB, 0xCDDB7921, 0x18E1C38B, 0x65523FE7, 0x66206F64,
	0x206D6F72, 0x72617473, 0x7E000A74, 0x14323BFE, 0xDE2CCCAE, 0x0B2022FE, 0xCDDBB1CD, 0xCDD8DE41, 0xCD3BDE25, 0x3ED0C1C4, 0xC3A0CD3F, 0xA0C3203E, 0xF879CDC3, 0xD2C1C4CD, 0xD0C3FBF5, 0xF7CDD5F8,
	0xE30630DB, 0xE1D69FCD, 0xCB45CDC9, 0xD5E5D50D, 0xE3D70FCD, 0xDBF7CDAF, 0x03FE1930, 0xE3FBF5CC, 0xE3DE41CD, 0xEE08307E, 0x2308202C, 0xB7E118E3, 0xE1370120, 0x5FC9D1E1, 0xF5FF66CD, 0x15CD0620,
	0x091837DC, 0xD4C1C4CD, 0x6FCDDC2C, 0x4DDCF5ED, 0x7AD1F1DE, 0xC1C4CDC9, 0x38CD0638, 0xF88AC3DC, 0xFEDE4DCD, 0xF879CA22, 0xF894C37B, 0x11DC8ECD, 0x2B38DCB5, 0x18CB45CD, 0x30DC8ECD, 0x2822FEF7,
	0xDCB91105, 0x99CD1918, 0xDC5411DC, 0x8A211138, 0xC90036AC, 0xCDC922FE, 0xD830DC99, 0x21DCBC11, 0x06E5AC8A, 0xFFFECDFF, 0x23770C28, 0xCD052805, 0xF138DC99, 0x0036FFF6, 0x0DFEC0E1, 0xC422FEC8,
	0xCDC0DCBF, 0xCDD0DC8E, 0x86C4DCB9, 0x99CDC9BC, 0xBFCDD0DC, 0x37F728DC, 0xC45CCDC9, 0x01C5F5D0, 0x28B90A0D, 0x0A20B804, 0xC45CCD41, 0xC4B80430, 0xF1C1BC86, 0xDCBFCDC9, 0xC82CFEC8, 0xFEC90DFE,
	0x09FEC820, 0xC90AFEC8, 0x48CD0A28, 0x5CCDE5CF, 0x31182BE8, 0xAE642AE5, 0x2AE52B18, 0x0ACDAE17, 0xBFCDE3DD, 0x3E23E3D6, 0xDBBDCD01, 0x3802FE7E, 0x282CFE0D, 0xAE152A09, 0xC3DEADCD, 0xCDE3CB49,
	0x38E3DE41, 0xAE1722D9, 0xFE7EC9E1, 0xA3CDC82C, 0x0E20B7E9, 0xB6237E23, 0xCA043E23, 0x1522CB55, 0x2CCD23AE, 0x208CFEDE, 0xCD44C9E5, 0x0218DDEA, 0x001E0006, 0x7CC9020E, 0xDD42FAB7, 0xDDEDFAB0,
	0x80EEC937, 0x3778C0B5, 0xEDB7C98F, 0xF6E0375A, 0xB7EBC9FF, 0xE03752ED, 0xCDC9FFF6, 0x72CDDD67, 0xDD37D2DD, 0x7CC9FFF6, 0xCDEB47AA, 0xC3EBDDEA, 0xB77CDDEA, 0xB77A0528, 0xB5EBC037, 0x7DB37AC8,
	0xFEC8626B, 0x37103803, 0x29FD308F, 0x023087D8, 0x80FED819, 0xFEC9F520, 0xC929C801, 0xDADDABCD, 0x4CC9DD37, 0xEBDDABCD, 0xCDF41841, 0xB37ADD67, 0x06EBC5C8, 0x20B77C01, 0x38BD7A09, 0x002E6505,
	0x957B0906, 0x05389C7A, 0xF6302904, 0x44783F3F, 0x0000214D, 0x18CB0E18, 0xEDEB19CB, 0x09013042, 0x6AED3FEB, 0x37EF203D, 0xB77CC9C1, 0x6F95AFF0, 0x67BC959C, 0x01FEC037, 0xC8B57CC9, 0xD89F877C,
	0xAA7CC93C, 0xDE0DF27C, 0x3CD89F87, 0xF920BAC9, 0xF520937D, 0x182C3EC9, 0x18283E10, 0x18293E0C, 0x18EF3E08, 0x237EE304, 0x0F20BEE3, 0x20FE7E23, 0x01FEFA28, 0x7EC9B7D0, 0x18D802FE, 0x02FE7E6A,
	0x2CCD2BC9, 0xC02CEEDE, 0x37DE2CCD, 0xFE237EC9, 0xFEFA2820, 0xFEF62809, 0x2BF2280A, 0xAE1B2AC9, 0xCDAE1B22, 0xB2DCB921, 0xDE2CCDC8, 0x7EDE8FC4, 0xEC2801FE, 0x7E233130, 0x2823B623, 0xAE1D220F,
	0xAE1F3A23, 0xCDD928B7, 0xD418DECA, 0x87CC49C3, 0xFED689D2, 0xEB1030C3, 0xCE6FE0C6, 0x4E6795DE, 0xEBC54623, 0xC3DE2CC3, 0x0021CB49, 0xAE1D2200, 0xAE1D2AC9, 0xAE1D2AC9, 0x7EC8B57C, 0x376F6623,
	0x18FF3EC9, 0x1F32AF01, 0x5B3EC9AE, 0xE5C3A0CD, 0x7EAE1D2A, 0xCD6F6623, 0x3EE1EF44, 0xC3A0C35D, 0xC0EACA22, 0xF25CC248, 0xEAFDD296, 0xC506C12F, 0xD2F5D2ED, 0xCC93C280, 0xD171E9A3, 0xD658D654,
	0xD529D650, 0xD6B6E7EE, 0xC53EC539, 0xE9ADC046, 0xD3D4CC31, 0xD9F0D39E, 0xCA2ACB51, 0xC78CC5D4, 0xC767C786, 0xDB43C251, 0xD68ED486, 0xE1CDDB13, 0xC2FFEAB5, 0xEB54F56B, 0xC275FA07, 0xC534C52F,
	0xC128C6A2, 0xC976C882, 0xC9F5CCCA, 0xD2A8D2B4, 0xF223C4DE, 0xC224C239, 0xC548C543, 0xF2A4F20F, 0xD52DE9A7, 0xDCDAD599, 0xE9A7D370, 0xDCC8E89E, 0xC7B0CCD5, 0xECDCEA78, 0xD4DBD313, 0xF784CC26,
	0xC34AC343, 0xDEC1DEC5, 0xC81AF229, 0xC42AC7E7, 0xF508C30E, 0xC997F29D, 0xC512C99D, 0xC5C0C59A, 0xC360BD19, 0x625BEDD5, 0x35CDD5AE, 0x012C01E0, 0x7EDFC8CD, 0x3EF920B7, 0x3E4F912D, 0xAF479801,
	0x13121312, 0xC9D1E112, 0xCDC8B77E, 0x1C38FF92, 0xDAFFA0CD, 0x26FEE0E2, 0x23E136CA, 0x21FEF8B7, 0x3AE15CD2, 0xC0B7AC00, 0x1C18203E, 0xD8E03ACD, 0xC3CAC5FE, 0x1221E5E1, 0xFFCACDE0, 0xF51838E1,
	0x013E97FE, 0xF1E008CC, 0x790B1312, 0x45CDC0B0, 0x8E8C17CB, 0xCD008F90, 0xB77EE008, 0x283AFEC8, 0xFAB72314, 0x20FEE01A, 0x203E0230, 0xE72022FE, 0x18E195CD, 0x2032AFE5, 0xD5C5C9AE, 0xCD237EE5,
	0xA8CDFFAB, 0xE3EBCDE3, 0xE6792630, 0xFF9CCD7F, 0xFE1A0930, 0x9CC47EE4, 0xF11538FF, 0xAFFAB71A, 0xF5C1D1E0, 0x08CDFF3E, 0x08CDF1E0, 0x3A18AFE0, 0xE5C1D1E1, 0xCD7E232B, 0xF938FF9C, 0x38E0D1CD,
	0x180D3E04, 0x05FE2306, 0xCD3D0120, 0xCDAFE008, 0xCDAFE008, 0x7EE3E008, 0x30FF9CCD, 0x08CD7E07, 0xF31823E0, 0xE1E1B5CD, 0x2032FF3E, 0xE5C937AE, 0xE0C3214F, 0x9FFFCACD, 0x203201E6, 0xD1E179AE,
	0xC7C9B7C1, 0x9692C681, 0xCA97E3C8, 0x9FEBA0A7, 0x2821FE00, 0xD026FE07, 0xD03F24FE, 0x07EE1FDE, 0x203AC937, 0x1028B7AE, 0x08FA237E, 0xCDD52BE0, 0x3230EECF, 0x4D181E3E, 0x8ACDC5D5, 0x2630C1ED,
	0x3EFF66CD, 0xED3E301F, 0x7AB0A05B, 0x201A3EB7, 0x7DEBE334, 0x04300AFE, 0x06180EC6, 0x08CD193E, 0xC3E17DE0, 0x237EE008, 0x08CDEBE3, 0xCDE3EBE0, 0xF220FFD8, 0xC5D5C9D1, 0xC1ED8ACD, 0x02FEE830,
	0x01281B3E, 0x08CDD13C, 0xA021E5E0, 0xFF4BCDB0, 0xCD237EF5, 0x3DF1E008, 0xC9E1F620, 0x352822FE, 0x3F287CFE, 0x3FEED5C5, 0x1028BF06, 0xE736112B, 0x1AE3EBCD, 0x237E0238, 0xE189CD47, 0x78AE2032,
	0xC0FEC1D1, 0x08C33628, 0xEEC83DE0, 0x203AC822, 0x3DC83CAE, 0xE008CDC9, 0x23C8B77E, 0xF52022FE, 0xCDE008C3, 0x32AFE008, 0x08CDAE20, 0xCD237EE0, 0xF638FF9C, 0xF61A1B2B, 0xC9131280, 0x08CD013E,
	0xCDC03EE0, 0x237EE008, 0x2BF820B7, 0xCF0FCDC9, 0xCACDD5C5, 0xDE37CDC1, 0xD1DEAACD, 0xE1E3CDC1, 0xD5C058C3, 0x64CD5950, 0x234ED1E8, 0xB1782B46, 0xC472CDC8, 0xD5E309E5, 0x5E2323E5, 0xE3E15623,
	0xE3FFD8CD, 0x54CD1238, 0xAC8A21E2, 0x23E21DCD, 0xF820B77E, 0xB7C398CD, 0xCE30E1D1, 0xC1BECDC9, 0xCD0A387E, 0x0AFEC3B8, 0x180D3EC0, 0x3E20FE08, 0xC3B8DC01, 0xC3B8C37E, 0x38E864CD, 0x4ACDEB17,
	0x010011EF, 0x7EAC8A01, 0x15030223, 0x02F820B7, 0xE2E8C30B, 0xE23DCDE5, 0x232323E1, 0xB7027E23, 0xE266CDC8, 0xF8FAF718, 0x3802FEE2, 0x3805FE1C, 0x380EFE42, 0x3820FE3E, 0x287CFE31, 0xE0D1CD54,
	0xDCFF9CD4, 0x187EE2E6, 0xFE7E230D, 0xFE5D28C0, 0x2B692897, 0x001E3A3E, 0x0B2022FE, 0x23E2CACD, 0xFEC8B77E, 0x23F52022, 0xE6CD2018, 0xE32FCDE2, 0xCDC9011E, 0xF57EE2E6, 0xCD232323, 0x1EF1E2DB,
	0xD00BFE01, 0x27EE001E, 0x0302FDE6, 0x140BC015, 0xCD011EC9, 0x7E23E2CA, 0x7EC0B723, 0xCACD7FE6, 0x3023BEE2, 0xC01DC9F6, 0xDE18203E, 0x7EE2FCCD, 0xCACDC8B7, 0xF71823E2, 0xF028C5FE, 0x20FFFE23,
	0xF5237E02, 0xE3B8CDE5, 0xF50828B7, 0xF1E2E6CD, 0x7EE2CACD, 0x09FE7FE6, 0xBEE2CAC4, 0xCDF42823, 0x001EFF9C, 0x011E0230, 0xE4D6F1E1, 0xD5C95FC0, 0x1FFE237E, 0x235E5D28, 0x1BFE2356, 0x1CFE3228,
	0x1EFE3928, 0x1DFE2328, 0x1AFE1628, 0x162B0B28, 0x2819FE00, 0x0ED62B04, 0xCDEBE35F, 0x3A18FF35, 0x2323EBE5, 0x56235E23, 0xCDEBE3E1, 0x2D18EF4A, 0x37583EE3, 0x0101C5F5, 0xE3071801, 0x01C5F5B7,
	0xAFEB040F, 0xC1F1DFCD, 0xCACD263E, 0xCADCF1E2, 0x3E0A18E2, 0xFF6CCD05, 0x5ACDEBE3, 0xCD237EEF, 0xB77EE2CA, 0xC9E1F720, 0x8741D6E5, 0xCE6F18C6, 0x5E6795E4, 0xC9E15623, 0x1A064FC5, 0xCDE44C21,
	0x0E38E3D7, 0x21F81023, 0xD7CDE736, 0xCB49D2E3, 0xC678C006, 0x7EC9C140, 0x7EE5C8B7, 0xFB301723, 0x28B9237E, 0xEF18F103, 0x1AC937E1, 0x1AE5C8B7, 0x2809FE13, 0x2020FE04, 0xDE4DCD05, 0x7E4FF118,
	0xFFABCD23, 0xE6E828A9, 0x1B0A287F, 0x3017131A, 0x18E113FB, 0xC937F1D6, 0xE715E720, 0xE692E6CC, 0xE64EE663, 0xE630E63A, 0xE60BE60F, 0xE5DBE607, 0xE5A6E5B1, 0xE54AE56E, 0xE4FFE549, 0xE493E4BF,
	0xE47CE484, 0xE456E45E, 0xE44CE451, 0xDAC54E4F, 0xD34F5000, 0x4F500048, 0xD24F47D3, 0x495200FD, 0x49D9C554, 0xD74F444E, 0x544449D8, 0x4948D7C8, 0x45D6C54C, 0x41D5C44E, 0x00D4D449, 0x7FD34F50,
	0x001DCC41, 0xC74E4953, 0x455050ED, 0x4E1CA452, 0x52001BD4, 0x52D3CE4F, 0xD2C6464F, 0x4D49ECCF, 0x454846C5, 0x5345EBCE, 0x457DD254, 0x417CD453, 0x47411ACE, 0xD1C6464F, 0x41D0C741, 0x5900EAC2,
	0xCC4F424D, 0xD04157CF, 0x495254E7, 0x7BA4474E, 0x19A45254, 0xCED04F54, 0xE6D04554, 0xD118D251, 0x45455017, 0xC350CDC4, 0x434150E5, 0x4F16A445, 0xCCC44E55, 0x4715CE49, 0x564114CE, 0x5500CBC5,
	0x554FCACE, 0x4E7AC44E, 0x474945C4, 0x79A45448, 0x52555445, 0x5345C9CE, 0xC8C54D55, 0x4F545345, 0x45C7C552, 0xC6CD554E, 0x49414D45, 0xCD4513CE, 0x454C45C5, 0xC4C55341, 0xC3C44145, 0x4F444E41,
	0xC55A494D, 0xC1C441C2, 0x49520000, 0x4FBFD44E, 0x4B4F78D3, 0x4F4CBEC5, 0x4CBDD254, 0xC9BCD44F, 0xBBCE4544, 0x12CB4545, 0xD2455041, 0xD45500BA, 0x474952B9, 0xD2B8CE49, 0x4E4550FC, 0xB7D4554F,
	0x494E4550, 0x204EB6CE, 0x4EB5D153, 0x52524520, 0x4720524F, 0x4F54094F, 0x4EB4B020, 0x45524220, 0xCEB3CB41, 0xD44F00B2, 0xB1D745FE, 0xB0D45845, 0x45564F00, 0x564FAFD2, 0x444FAEC5, 0xC44FADC5,
	0x77CE49FB, 0xACA44449, 0xC5475245, 0x4F4D45AB, 0x41AAD952, 0x534176D8, 0x4F00DFCB, 0xA4524557, 0x31474F11, 0xC74F10B0, 0x41434F0F, 0x4FA9C554, 0x49A8C441, 0x49A7D453, 0x45A6C54E, 0xCE45A5D4,
	0x5446450E, 0x450075A4, 0x4F00A4D9, 0x4E000DD9, 0x534E0CD4, 0x4E74D254, 0xA3D45550, 0x4E0BD04E, 0xA459454B, 0x454B4E43, 0xCB4E0AD9, 0x00A1C6A2, 0xCD454D49, 0xA4584542, 0x41520073, 0x43494850,
	0x094FDED3, 0x4FA0CF54, 0xC2555309, 0xC552009F, 0x4D415209, 0xD24FE0C5, 0x49E4CE9E, 0x4C4908D8, 0x5800DDCC, 0x455607D0, 0x529DD952, 0x9CD24F52, 0x5241D252, 0x4152E3CC, 0x4F9BC553, 0xD64E40C6,
	0x99D44E9A, 0x4C98C44E, 0xC997C553, 0xD44944DC, 0x41520096, 0x5295D257, 0x4994D741, 0xDBC993CD, 0x49D25245, 0x54454C45, 0xC74592C5, 0x53464591, 0x4590D254, 0x41455246, 0x46458FCC, 0x8ED44E49,
	0x458DC645, 0x4172A443, 0x008CC154, 0x4F535255, 0x4552E1D2, 0x4F06CC41, 0x504F05D3, 0x52484359, 0x4E4F7EA4, 0xD34C8BD4, 0x534F4C8A, 0xD4554F45, 0x534F4C89, 0x88CE4945, 0x4C87C74C, 0x86D24145,
	0x04D44E49, 0x03A45248, 0xCE494148, 0x84D44185, 0x83CC4C41, 0x44524F00, 0x4982D245, 0x0071A44E, 0x81CF5455, 0x5302CE54, 0xC44E01C3, 0x455446FA, 0xD34280D2, 0xF8DE0000, 0x093EF9DC, 0x203DF0BD,
	0xEEBEF0BE, 0xF2BE093C, 0xF3BD093C, 0xF3BC203D, 0xF1BCEFBD, 0x01BAF7AF, 0xF5ADF6AA, 0xC0A7F4AB, 0x642AAF00, 0x772377AE, 0x22237723, 0x1118AE66, 0xB7AE213A, 0xE5D5C5C8, 0xCDE78601, 0xD1E1E9B9,
	0x2132AFC1, 0xFDCDC9AE, 0xD802FEE9, 0xF6201DFE, 0x2B5E2B56, 0x2323EBE5, 0x56235E23, 0x231E36E1, 0x18722373, 0x20FE7EE1, 0xCD230120, 0xA4CDE770, 0x4DCDE5DF, 0x2828B7DE, 0x0421D5C5, 0xE5E50900,
	0xE5E864CD, 0xD1E7E4DC, 0xF6B8CDC1, 0xEBF607CD, 0x722373D1, 0x2373D123, 0xEBC12372, 0xC9B0EDE1, 0xE85CCDE1, 0xEBC8B178, 0xC3F6E5CD, 0x00CDF607, 0xDE37CDE8, 0xCDFB4DCD, 0x8FCDE81A, 0xC058C3C1,
	0xE5CF0FCD, 0xE882CDC5, 0x64CDE5D1, 0xAE2222E8, 0xEDB7E1EB, 0xAE242252, 0x70CDC9E1, 0x244BEDE7, 0xAE222AAE, 0x23E7E4C3, 0xFE56235E, 0xFE2A281D, 0xCB49C21E, 0xDEB5CDE5, 0x30FFD8DC, 0x23E5E10A,
	0x23E9ADCD, 0xD4E868CD, 0xEB2BE85C, 0x3E2B2BE1, 0xAE21321D, 0x23732377, 0xDE2CC372, 0xD8E864CD, 0x08CB45CD, 0x23AE642A, 0x2B46234E, 0xE5C8B178, 0x237E2323, 0xCDEB6F66, 0xE1EBFFD8, 0x09C8D03F,
	0x642AE618, 0x4EE523AE, 0x78234623, 0x092837B1, 0x6F66237E, 0xFFD8CDEB, 0x09D8E1EB, 0x0A11E818, 0xE91AC400, 0x000011D5, 0xDCDE41CD, 0x11D5E91A, 0x41CD000A, 0xCF48DCDE, 0xE1DE37CD, 0xD5EBE3EB,
	0xE864CDE5, 0x64CDE5D1, 0xCDE1EBE8, 0x1D38FFD8, 0xD5C1D1EB, 0x234EC5E5, 0x28B17846, 0x7E092B13, 0x0C28B623, 0xEBE5C12B, 0x30E1EB09, 0xCB4DC3E8, 0xCDE92001, 0xE1C1E9B9, 0x4EE5C5D1, 0x78234623,
	0x730C28B1, 0xE1237223, 0x09EBC109, 0xE1EA18EB, 0xE94401E1, 0xC3E9B9CD, 0x2CFEC058, 0xC9CF48C4, 0xFEE9FDCD, 0x1EFED802, 0x56E5F620, 0x64CD5E2B, 0x2B0E30E8, 0x72E5E1EB, 0x3E2B732B, 0x2132771D,
	0xDC18E1AE, 0xFEE9FDCD, 0x1EFED802, 0x56E5F620, 0xB5CD5E2B, 0xCBE6CDDE, 0x06E918E1, 0xCD042B00, 0xA1FEE9FD, 0x02FEF828, 0xC8B7F530, 0xFEE9FDCD, 0x10EF2097, 0xDE2CCDEA, 0xFE7EC904, 0xFE03285B,
	0x0006C028, 0xE9FDCD04, 0xF8285BFE, 0xF42828FE, 0x0B285DFE, 0x072829FE, 0xE93002FE, 0x10CB49C3, 0x06C923E4, 0x7E081801, 0x1823C8B7, 0x2B0006FA, 0xB7E9FDCD, 0xF820B8C8, 0xDEB1CDC9, 0xAE642AE5,
	0xB6237E23, 0xCD231328, 0xC523DEAD, 0xC1FFFCCD, 0xE9EFCD2B, 0x18F420B7, 0xADC3E1E7, 0xE9EFCDDE, 0x7E23C0B7, 0xCA79B623, 0x5423CB55, 0xCDC9235D, 0x02FEE9FD, 0xC897FED8, 0xF320EBFE, 0xDE2CCDC9,
	0x380EFEC8, 0x3820FE25, 0x2822FE2B, 0x287CFE11, 0x28C0FE1B, 0x28C5FE30, 0xC0FFFE2C, 0x7E23C923, 0xB7C822FE, 0x3E2BF820, 0x2323C922, 0x177E23F5, 0xC9F1FB30, 0xFED818FE, 0xFE082819, 0x2303381F,
	0x23232323, 0xCD23F5C9, 0x2BF1E9A7, 0xE5D5C5C9, 0xCDEA5A01, 0xD1E1E9B9, 0xCDE5C9C1, 0xFED1E9FD, 0x0EFED802, 0xCDEBF430, 0x0DFEDE2C, 0x0D360238, 0x2377AF23, 0xE218EB77, 0xEDDE3DCD, 0x38AE645B,
	0x281EFE1D, 0x281DFE15, 0xEAD1CD11, 0xD2EAF121, 0x6DCDBD13, 0xAE642AEC, 0x27CD1218, 0xCDD5C0E8, 0x78CDD300, 0xC18FCDC1, 0xE1C162CD, 0xBD43CDE3, 0x77C323E1, 0xEAD1CDDE, 0x6DCD0630, 0xC058C3EC,
	0xF5ABCDE5, 0xCDAE262A, 0x37CABC83, 0xEDC3E1CC, 0xEC54CDD2, 0x02EE0EE6, 0x37CD0828, 0xC16FCDDE, 0x41CDC937, 0xCEF5DCDE, 0xAE2653ED, 0xB7DE37CD, 0xAE262AC9, 0xE5BC83CD, 0xE1BC7ADC, 0x32ABEEC9,
	0x2CCCAE28, 0xEC54CDDE, 0xCD000011, 0x0630DE41, 0xC42CFE7E, 0xCDD5CEF5, 0x0830DE41, 0x92DE25CD, 0x37E800CD, 0xDE37CDF5, 0xCDFB4DCD, 0x0ECDFC64, 0x1ADCF1D6, 0xEB47CDE8, 0xAE642AD1, 0xCDC8B37A,
	0x5CCDDEAA, 0x3AC92BE8, 0xCAB7AE28, 0x89CDEC62, 0xEC6DC3C1, 0xCDEC54CD, 0x78CDDE37, 0xEC62CDC1, 0xCDC058C3, 0x70CDC18F, 0xF629CDE7, 0xEBAE662A, 0x23AE642A, 0xEBAE6622, 0xEBFFE4CD, 0xEBF707CD,
	0x13B8ED2B, 0xEC4BCDEB, 0x28B34A30, 0x4BCDD54C, 0x7E4130EC, 0x282BB623, 0x2323E51B, 0x6F66237E, 0xE1FFD8CD, 0x0C300728, 0x18EC19CD, 0x235ED5E6, 0xD1192B56, 0xAE662AE5, 0x23732323, 0x001D1172,
	0xE3E1EB19, 0xE3EB19EB, 0x38FFD8CD, 0x2CCDE325, 0xB138E1EC, 0x18EBD9CD, 0xB6237E36, 0xCD05282B, 0xF518EC19, 0xAFAE662A, 0x23772377, 0xC3AE6622, 0xD9CDECB0, 0xDEAACDEB, 0x1518073E, 0xD8BC80CD,
	0xC8371AFE, 0x3FAD9132, 0xAD9132C9, 0x3EC16FCD, 0x00CDF518, 0x55C3F1D3, 0x4ED5C5CB, 0xED2B4623, 0xEDAE665B, 0x6653EDB0, 0xC9C1D1AE, 0xAE662AEB, 0x23722373, 0x1B1B2323, 0xB37A1B1B, 0xFCCD0828,
	0x382377EB, 0x6622C9F4, 0xCDC937AE, 0xDC5FEBFC, 0xC957EBFC, 0xCDD300CD, 0x2932D2BE, 0x2A43EDAE, 0x293AC9AE, 0x63CAB7AE, 0x2016FEEB, 0xAE293A0B, 0x422816FE, 0x0428FEE6, 0x19CB45CD, 0xCDC18FCD,
	0x4BEDF629, 0xCD03AE64, 0x8011F707, 0xEDB719FF, 0x2A5BED42, 0xFFD8D4AE, 0xEBEBF5DA, 0xAE662209, 0x1FAE293A, 0xAE2C329F, 0x83CD6960, 0xEC09CABC, 0xC3F63CCD, 0x8FCDD2ED, 0xDEAACDC1, 0xCDF629CD,
	0xEC30CB0A, 0xB7DE4DCD, 0x18ECCDC4, 0xEECFCDF2, 0x3EE7A5DA, 0x3E022815, 0xCB55C306, 0xCDD300CD, 0x0006D2A8, 0x30DE41CD, 0xDE25CD25, 0x7E23230D, 0x49F2DFE6, 0x0021E5CB, 0xFFB4CDED, 0xDE2CC3E3,
	0xC1CB4903, 0x2BC2ED53, 0xED0CD0ED, 0x37CD0106, 0xCDC5E5DE, 0x4DCDE770, 0xAE642AEA, 0x662AEB23, 0x52EDB7AE, 0x0001F1EB, 0xCD201800, 0xF5CDDE15, 0x15CDD5CE, 0xCEF5CDDE, 0xDE41CDD5, 0xDC000011,
	0xCDD5CEF5, 0x023EDE37, 0xCDE3D1C1, 0x37D2BC98, 0xCD1718CC, 0x3EE5DE37, 0xC1A6CD09, 0x000101F5, 0xCDFFFF11, 0xCDF1E1E3, 0xF5CDC1A6, 0xCDC9E1D2, 0x0520EE0F, 0x18DE4DCD, 0x2826FE2F, 0xFFA0CD1C,
	0x38CD2638, 0xFF1BCDFF, 0xCDE5C937, 0xD8D1ED92, 0x0016C9EB, 0x2026FE7E, 0xEEE7CD0F, 0x35CDF5EB, 0xD8EBF1FF, 0xCBBEC3C8, 0xFE237EE5, 0xDE4DCC2E, 0xE1FFA4CD, 0xEE7E0638, 0xC923C02E, 0xD5FF38CD,
	0x11000001, 0x1ECDAE2D, 0x202EFEEE, 0xEE94CD0B, 0x0CFF41CD, 0x0DEE1ECD, 0xEBFF36EB, 0xD1EE42CD, 0x21D5E55F, 0x99CDAE2D, 0x66CDD1EE, 0xE50830FF, 0xFE2CCD42, 0x7A1138E1, 0xB8CD234E, 0x79CD7BBD,
	0x3ECDEBBD, 0xBD61DCFF, 0xD8E10A3E, 0xCDCBBEC3, 0x1623DE4D, 0xC82DFEFF, 0xC82BFE14, 0xCDE5C92B, 0xCD23DE4D, 0x0438FFA4, 0xFFABC3E1, 0x30D6E1E3, 0x0728B012, 0x0CFE0478, 0x79130130, 0x0CDF28B7,
	0x45FEDC18, 0xCDE51020, 0x0FCDEE94, 0xDE4DCCEE, 0x38FFA4CD, 0x18AFE104, 0xCDE1E31E, 0xC5D5FF41, 0x30EF00CD, 0x64D67B09, 0x7B00DE7A, 0x7F3E0238, 0x2014D1C1, 0xC63C2F02, 0xD6785F80, 0xAF01300C,
	0x83093091, 0xFEAF0138, 0xC980CE01, 0x3E023083, 0xC980D6FF, 0x23DE4DCD, 0x3F21EBC9, 0x050101AE, 0x1000362B, 0xFFFE1AFB, 0x3A2177C8, 0xFE1A13AE, 0x41D5C8FF, 0x5EE50016, 0x29296B62, 0x195F2919,
	0x73E17C5D, 0xD1EF1023, 0x77DF28B7, 0xC5DB180C, 0xEF00CDE5, 0xFF35CDEB, 0x0630C1EB, 0xFFC6B37A, 0x59500338, 0x23C9C1EB, 0xCDDE4DCD, 0x0206FFAB, 0x062858FE, 0x48FE1006, 0xCD230420, 0x0218DE4D,
	0xCDEB0A06, 0x0026EF2C, 0x0E1E306F, 0xEF2CCD00, 0x16D51430, 0x58D55F00, 0xD1DD72CD, 0x30190338, 0xD1FF0E02, 0xFE79E718, 0xC978EB01, 0xA4CD131A, 0xCD0A38FF, 0x41FEFFAB, 0xD605303F, 0xB830D607,
	0xC9AF1BD8, 0xC3EF4ACD, 0xC5D5C38B, 0xCDFF35CD, 0xCDAFFD03, 0xC123EF72, 0xC5D5C9D1, 0xEF6ACDAF, 0xFE7ED1C1, 0xC923C020, 0x5222403E, 0xF3CDF5AE, 0x57C5F1FC, 0xF18ACDD5, 0xEF96CDD1, 0xF1F11ACD,
	0xCCB7785F, 0x45CDF12C, 0xF14FCDF1, 0x7AF16FCD, 0x362BD01F, 0x877AC925, 0xEDFA2D30, 0xD6817BEF, 0xF056FA15, 0x5741F67A, 0x79414318, 0x831528B7, 0xDECD5F3D, 0x790106F0, 0x043807FE, 0x262072CB,
	0xF074C4B8, 0x7BF032C3, 0xEFD0FAB7, 0xC941DC20, 0xF0DECD43, 0xF628B778, 0x81475893, 0xEFAAFA83, 0xC3F087CD, 0x063EF074, 0x18AE5232, 0xF0FBCD2E, 0xC2CB0330, 0x13CC47AF, 0xCB0E20F1, 0x523A04C2,
	0x0528B7AE, 0x52323C05, 0x284ACBAE, 0x05047807, 0xFB3004D6, 0x0428B779, 0x785F9083, 0x59CD47F5, 0x28B8F1F0, 0x05231C0D, 0x2EFE7EE5, 0x36230120, 0x043EE131, 0xE5F0C2CD, 0x7B2B4521, 0xF044F2B7,
	0x2D2693AF, 0x2EAE4C22, 0x0AD62C2F, 0x3AC6FB30, 0xAE4E2267, 0x87CDC9E1, 0xF113CDF0, 0x0530B980, 0x18F09ACD, 0x3815FE0A, 0x91143E02, 0x3AF0C2C4, 0xC8B7AE52, 0xC5782E0E, 0x6F850447, 0x2B67958C,
	0x10774E79, 0x7BC9C1FA, 0x2FF04781, 0xB814063C, 0x2B470130, 0x100C3036, 0x854FC9FA, 0x67958C6F, 0xFE7EC5E5, 0xF0B4D435, 0x2B0538C1, 0x0C043136, 0x38182BE1, 0x2BC8B779, 0xFE347E0D, 0x3036D839,
	0xC5D5F218, 0x907B47EB, 0x67829F6F, 0x77131AE5, 0xF920B723, 0x2330362B, 0xC1E1FB10, 0x5021C9D1, 0xFE7E2BAE, 0x0D052030, 0x2BF62004, 0x4F11C5D5, 0xCD0006AE, 0x23EBFFF5, 0xC5C9D1C1, 0x1F04E67A,
	0x62CB471F, 0x877A0720, 0xF10DF2B3, 0xAE533A04, 0x3AC9C190, 0xC8B7AE52, 0x4ACBC93D, 0x03D678C8, 0x0EF5C8D8, 0xF077CD2C, 0xF218F104, 0x3D237EE5, 0xF93830FE, 0x5F01203C, 0x80EE7AE1, 0xD8F0FBF4,
	0x18303EC8, 0xC852CB06, 0x04AE543A, 0x7BC9772B, 0x382D3E87, 0x98E67A0E, 0xE6C880EE, 0x202B3E08, 0xCB203E02, 0x32E42862, 0x32AFAE50, 0x7AC9AE51, 0x533AF0B7, 0x38C890AE, 0x6ACB470E, 0x02202A3E,
	0x772B203E, 0xCBC9FC10, 0x2D11C9C2, 0xB647AFAE, 0x0D05202B, 0x2818F920, 0xFD308F37, 0x1857D5EB, 0xD51B1A11, 0x58578F37, 0x77278F7E, 0xF8201D23, 0x36040330, 0xAE2D2101, 0xEA20877A, 0xE1200DD1,
	0xAE5021EB, 0x87780036, 0x303EC84F, 0x1B67EDEB, 0x1B67ED12, 0xF5102312, 0xC030FEEB, 0xD5C9230D, 0xAE3E21EB, 0xF53D0036, 0xF0F6A17B, 0xCEA0C627, 0x78772B40, 0x1BCB3ACB, 0xF147FA10, 0xF1E7F23D,
	0x003EB37A, 0xC9D1E120, 0xE7FEEBCD, 0xCDFF32C3, 0xCDD5CEF5, 0x12D1F23F, 0xFEB6CDC9, 0x78ED4D44, 0xCDFF32C3, 0x79EDF23A, 0xF23ACDC9, 0xC4003E57, 0xED5FF23F, 0x28A2AB78, 0xF5CDC9FA, 0xCD4B42CE,
	0xB8C3DE15, 0xB77E23CE, 0xD4CCE523, 0x30E1EBBC, 0x17237E07, 0x0918FB30, 0x1CCB45CD, 0x0ECEF5CD, 0x5553EDFF, 0x573279AE, 0x5A73EDAE, 0xCD2006AE, 0x0830DE41, 0xCEE3CDC5, 0xF310D5C1, 0x22DE37CD,
	0x203EAE58, 0x0021DD90, 0xDF39DD00, 0x7BEDAE55, 0xCCCDAE5A, 0xAE582AFB, 0x180D3EC9, 0xCEC3CD03, 0xC9AE5C32, 0xCDC1CFCD, 0x98DADE3D, 0xCAEDFEC3, 0x21EBF37E, 0xB4CDF2C3, 0xFECDEBFF, 0xDE3DCDFF,
	0x04C9EB30, 0x192CF2D2, 0xF334E5F3, 0x3BF33DEA, 0x62CDDE2C, 0xCDE5F5CF, 0x0F28FF66, 0xCDEF68CD, 0x2036F88A, 0x34B0A02A, 0x2A1F187E, 0x0E46B0A0, 0x237E2300, 0x18046F66, 0x20FE7E0E, 0x3D073023,
	0x28050720, 0x100C2304, 0xE7CD79F0, 0xC398D4C2, 0xE1F8D0CD, 0xC398CCF1, 0xDE2CCDC9, 0x4FAE5C3A, 0x3DC2B9CD, 0x2FFD3091, 0xCD81473C, 0x98D2C2E7, 0x1E1878C3, 0xCDF35DCD, 0x187BF369, 0xF35DCD15,
	0xF369CD1B, 0x2FC2B9CD, 0x38831C3C, 0xC398CD05, 0x04477B1D, 0x203EC805, 0x18C3A0CD, 0xDE2CCDF7, 0xCDDE19CD, 0x1DC3CED8, 0x30177ADE, 0x00001103, 0xD0C2CFCD, 0x165FEBE5, 0xDDAECD00, 0x2CCDC9E1,
	0xCF09CDDE, 0x3BDE25CD, 0xB0A02AE5, 0xCF62CDE3, 0xAE5D32AF, 0x46EBD5D1, 0x66237E23, 0xCDCDEB6F, 0xF4ABD2F3, 0x38DE3DCD, 0xF3EFCD11, 0xCDD50C28, 0xCDD1CF62, 0xDC30F3CD, 0x3EF5EA18, 0xAE5D32FF,
	0xF1F3CDCD, 0xE3C398DC, 0xE1FC03CD, 0xC8B778C9, 0x5FFE1AE5, 0x10130720, 0x181B040C, 0xF3F7CD08, 0x38F431D4, 0xA0CD1A08, 0xE51013C3, 0xFEC9E1B7, 0xDE2CCA3B, 0x1ADE15C3, 0x26FE000E, 0xFE0C1E28,
	0xEE192821, 0xD5C5C05C, 0x0A280513, 0x5CFE1A0C, 0x20FE0828, 0xC1D1F228, 0xF1F1C9B7, 0xD5C50513, 0xB7AE5D3A, 0xDCCD0720, 0x52CD79F8, 0x37C1D1F3, 0xF448CDC9, 0xAE5D3AD0, 0xC50B20B7, 0x6ACD79D5,
	0xC38BCDEF, 0xC937C1D1, 0x800ED5C5, 0xFE1A0026, 0x1307202B, 0x24242805, 0xFE1A880E, 0xFE20282E, 0x133E2823, 0xEB142805, 0x0F20EBBE, 0x042E2424, 0x28F502CD, 0xFE202E24, 0xD111282A, 0x13C9B7C1,
	0x1AF82805, 0xF32023FE, 0x1518041B, 0x0E280513, 0xF502CD1A, 0x2E240820, 0xAE543224, 0xB5790513, 0xCDF1F14F, 0x857CF4AE, 0xC3D815FE, 0x6FAFCB4D, 0xFE1AC8B0, 0xFE0F282E, 0xFE062823, 0xCB10202C,
	0x101324C9, 0x132CC9ED, 0xFE1AC805, 0xEBF72823, 0x205EFEE5, 0x20BE2316, 0x20BE2312, 0x20BE230E, 0xD678230A, 0x47043804, 0xE1F1CBE3, 0xC80504EB, 0x1AC059CB, 0x05282DFE, 0xCBC02BFE, 0x13E1CBD9,
	0x24FEC905, 0xC9A3FEC8, 0xCDC1CFCD, 0x98DADE3D, 0xCF62CDC3, 0x66CDE5F5, 0xCD0828FF, 0x8BCDEF5A, 0x3E0D18C3, 0xC3A0CD22, 0x3EF8D0CD, 0xC3A0CD22, 0x98CAF1E1, 0xF3EFCDC3, 0xA0CD2C3E, 0x01D218C3,
	0xDECDAC00, 0x5E22D0FF, 0xB07322AE, 0xEBAE6022, 0x01AE6222, 0xD809012F, 0xEBAE6422, 0xD852ED23, 0xD804FE7C, 0x32F77FCD, 0xCDC9AE6E, 0x2AE5CEF5, 0xD8CDAE60, 0x133138FF, 0xDCF5ECCD, 0xCDEBF58A,
	0x762AF808, 0xB07822B0, 0xAECDC9E1, 0x5E4BEDBB, 0xF5E0DCAE, 0x762A1238, 0xE0CD2BB0, 0x753AD0F5, 0xFEC8B7B0, 0xF77FCA04, 0xD5F875C3, 0xED2B09EB, 0xE3AE624B, 0xAE5E2AEB, 0xEBF5E0CD, 0xE0DCEBE3,
	0xEDE530F5, 0x21B0764B, 0xCD090FFF, 0xDCD1F5E0, 0xEBD8F5E0, 0xECCD5950, 0xF77FC2F5, 0xC9B07822, 0xEDB7E5D5, 0xEDB7EB42, 0x0618EB42, 0x5E2AE5D5, 0xD8CD23AE, 0xC9D1E1FF, 0x712AE5D5, 0x732AEBB0,
	0xFFE4CDB0, 0x2AC9D1E1, 0x2209AE66, 0x6E3AAE66, 0x2AC0B7AE, 0x2209AE68, 0x6A2AAE68, 0x6A2209AE, 0xAE6C2AAE, 0xAE6C2209, 0xF6FCCDC9, 0x662A4D44, 0xB8CDEBAE, 0x32FF3EF6, 0xD718AE6E, 0xAE6E32AF,
	0xEBAE662A, 0xCDAE682A, 0xE5CDFFE4, 0x21C418F6, 0x6F22AE6F, 0xCD013EB0, 0x0036F672, 0xAE192223, 0x6F2A0C18, 0xC83C2FB0, 0xFF3E6F85, 0x6F22678C, 0x6F2AC9B0, 0x6F85E5B0, 0x2267958C, 0x943EB06F,
	0x8C4F3E85, 0x4FCDD0E1, 0xF875C3F6, 0x22B0732A, 0x06C9B071, 0xAE6C2A00, 0xB0712AEB, 0x2B42EDB7, 0xFFD8CD2B, 0x64CD0930, 0xCDEA38FC, 0x220ECB45, 0x7123B071, 0xC9237023, 0xC5F714CD, 0x09E5D5D5,
	0xCDEB0E38, 0xD8CDF707, 0xCD0830FF, 0xF338FC64, 0xE1F875C3, 0x917DD5C1, 0x47987C4F, 0xF5CD1B2B, 0xC1D1E1FF, 0xEBD5C5C9, 0x14CDEB09, 0xFFE4CDF7, 0xEFCDD1EB, 0x0021D1FF, 0xFFE4C300, 0xEBAE6C2A,
	0xB7B0712A, 0x3AC952ED, 0x2AB7AE6E, 0x2AC8B071, 0xC92BAE68, 0xB7AE6E3A, 0xC8AE6C2A, 0xC9AE662A, 0x18000111, 0x08021108, 0x00110318, 0x3AE5C508, 0x20B7B075, 0x5E2AD518, 0x782223AE, 0xF00011B0,
	0xF875D219, 0x22F808CD, 0x3ED1B076, 0x762AB304, 0x19001EB0, 0x18C1E1EB, 0x18FE3E27, 0x18FD3E06, 0xE5FF3E02, 0xA6B07521, 0x2004FE77, 0xB0762A09, 0xF5ECCDEB, 0xC9E10228, 0xCDB0782A, 0xAFE1F808,
	0xC9B07532, 0x292880FE, 0x4FCEB8CD, 0x06DE15CD, 0x41D43708, 0xB8DC9FDE, 0x10B7F5CE, 0xCD79EBF5, 0x4DD2BBA5, 0x000801CB, 0x772BF109, 0xEBFA200D, 0xDE2CCDC9, 0xE5CED8CD, 0xCD010021, 0x4DDAFFD8,
	0xAECDD5CB, 0x1B30EBBB, 0x00266F2F, 0x29292923, 0xC2F5ECCD, 0xCD19CB4D, 0x61CDF808, 0x010011F7, 0xD1BBABCD, 0xE1F7E9CD, 0x010021C9, 0xC852EDB7, 0x292929D5, 0xAE5E2AEB, 0xFE7C52ED, 0xF875DA40,
	0xF808CD23, 0xBBABC3D1, 0xFC64CDE5, 0xF5F8CDEB, 0x09AE6C2A, 0xFFD8DC3F, 0x5E2A5B30, 0xED37EBAE, 0xB07A2252, 0xF86511E5, 0xC1DA93CD, 0x14380778, 0x2A2B28B1, 0x5D54B073, 0xF8CDE509, 0xF5CDEBF5,
	0x1318E1FF, 0x54B0712A, 0xCDE5095D, 0x23EBF5F8, 0xFFEFCD13, 0x22D12BEB, 0x22EBB073, 0x2BE1B071, 0x23AE5E22, 0xAE662AC9, 0xD0FFDECD, 0x09B07A2A, 0x732B72EB, 0xCB45CDC9, 0xA7CD2307, 0x22FE7EF8,
	0xB7DE2CCA, 0x23043128, 0xA7CDF318, 0xC8B77EF8, 0xF9180423, 0x4FF8A7CD, 0x1B28B77E, 0xFE1828B9, 0x2314282C, 0x22F11804, 0x06D1B09D, 0xFFFECD00, 0xB09C3278, 0xE5FBD6C3, 0x12280504, 0x20FE7E2B,
	0x09FEF728, 0x0DFEF328, 0x0AFEEF28, 0xC9E1EB28, 0xC8FBF5CD, 0xB8CD131A, 0xC9F910C3, 0xC8FBF5CD, 0x05309079, 0x47022880, 0xE8184FAF, 0x18F8F101, 0xD841FE0C, 0xC6D05BFE, 0xAB01C920, 0xA02AC5FF,
	0xFBF5CDB0, 0x23FC41CD, 0x6966234E, 0xCA3D3CC1, 0x7EF5FBD6, 0xFFFCCD23, 0x18F11312, 0xA05BEDF1, 0x30861AB0, 0xCB45CD04, 0xFC41CD0F, 0xD5F959CD, 0xD6CD48C5, 0xECCD79FB, 0x79E1C1FF, 0xCDFFECC3,
	0xB9AFF959, 0x28B80F28, 0x1A0D050A, 0x28239613, 0x3CC09FF2, 0x9FC8B8C9, 0xFBF5CDC9, 0x03CDD548, 0xC9D1EBFC, 0x18010101, 0x040F0103, 0xCF62CDC5, 0xFEEBCDE5, 0xDE41CDE3, 0xCEB8DC9F, 0x4DD211FE,
	0x1DCD47CB, 0xE1EB78DE, 0xDFCDD5C1, 0xCD3118F1, 0x15CDCF62, 0xFF74CDDE, 0xCDCF03CD, 0x79E5DE1D, 0xD5F662CD, 0xFF6CCD79, 0xC4B778D1, 0x4DD2F448, 0xC2B778CB, 0xCD79CB4D, 0x0418EF6A, 0xEF68CDE5,
	0xAFFF0EE5, 0x2023BE0C, 0xCD79E1FB, 0xECCDFBD3, 0xCDC9E1FF, 0x1818FA43, 0x1AFA43CD, 0x4F113890, 0x19CD0E18, 0xFA43CDDE, 0x05CB4DCA, 0xFA4FCD48, 0xE5DE1DCD, 0xFA60CDEB, 0xCDFC41CD, 0xCDEBFC03,
	0x0006FBD6, 0xCD371809, 0xBFCDDE19, 0xFF5ECDD6, 0x58CDEBE5, 0x55CDE3FB, 0xCB4DCAFA, 0x4FCD4F3D, 0xDE1DCDFA, 0xC5DE21CD, 0x78CF03CD, 0x30B8E3C1, 0x60CD4701, 0x234623FA, 0x00066866, 0xECCDEB09,
	0xCDC9E1FF, 0x2AEBCF09, 0x0EEBB0A0, 0x06061800, 0x29FE7EFF, 0x15CDD5C8, 0xCEB8CDDE, 0xC9B7D147, 0x0130917E, 0x78D8B8AF, 0xFBF5CDC9, 0xA6CD0318, 0xFF32C3FA, 0x37FAD9CD, 0x01E69F4F, 0x6FCD3418,
	0xFEF530C4, 0xFEF128FC, 0x18ED28EF, 0xCEB8CDEA, 0xDE15CDF5, 0xCDCF62CD, 0xA1CDDE1D, 0x18F14FFA, 0xFF66CD11, 0xF5CD3320, 0x1A3728FB, 0xFAD9CDC9, 0xCD47200E, 0x0479FBD3, 0x1312C805, 0xF5CDFA18,
	0xFF32CAFB, 0x165FE5EB, 0x725E1900, 0x6FCDD5E3, 0x73E1D1ED, 0xCB45CDD8, 0xB6CDE50D, 0x7DB77CFE, 0x4DC3C8E1, 0xCF62CDCB, 0x0EFF66CD, 0xCD0E2801, 0xCAB7FAD9, 0xCD4FCB4D, 0x09CDDE15, 0xDE15CDCF,
	0xB0A02AE5, 0xCF03CDE3, 0xE3DE1DCD, 0xF5D54879, 0xEBFC03CD, 0x00165FF1, 0x93782B19, 0xD17B473C, 0x0D0C2538, 0x78F52228, 0xE51B38B9, 0xBE1AC5D5, 0x13230B20, 0xC1F7200D, 0x18F1E1D1, 0xE1D1C10B,
	0x05233CF1, 0xAFF1E018, 0xE1FF32CD, 0x11E5D5C9, 0x93CDFB65, 0xC9D1E1DA, 0x4E237EE5, 0xB7EB4623, 0xE1FB65C4, 0xB0712AC9, 0x30FFDECD, 0xB0732A07, 0xD0FFDECD, 0xE52B2BEB, 0xE1FBB9CD, 0x77B09C3A,
	0x9D5BED23, 0x722373B0, 0x37CDC923, 0xB9CDD8FC, 0xFBD6C3FB, 0xCDB0A02A, 0xB778FC1F, 0x642AE5C8, 0xFFD8CDAE, 0xEBB0732A, 0x30FFD8DC, 0x665BED0A, 0xFFD8CDAE, 0xE1FC37D4, 0x41CD7ED8, 0x237ED5FC,
	0x6966234E, 0xD1FFECCD, 0xC9B09C21, 0x22B07E21, 0xCDC9B07C, 0x3EE5FC41, 0xB09F3203, 0x22B07C2A, 0x9C11B0A0, 0xFFD8CDB0, 0x55CA103E, 0xFB7CCDCB, 0xE1B07C22, 0x5ECDE5C9, 0xB0A02AFF, 0xE1FC03CD,
	0xCDC9B778, 0x78C0FC1F, 0x712AC8B7, 0x232323B0, 0xC0FFD8CD, 0x266E2B2B, 0x222B1900, 0xE5C9B071, 0xB07C5BED, 0xCD1B1B1B, 0x0420FFD8, 0xB07C53ED, 0x235E2346, 0x2AC9E156, 0x7D3EB0A0, 0x9CB03E95,
	0x4FC5E5C9, 0xF693C4B7, 0xB09C3279, 0xEBB09D22, 0xCDC9E1C1, 0x0620FF66, 0xCDFBF5CD, 0xFCCDFC64, 0xFE89C3F6, 0x21C5D5E5, 0x0C18B07E, 0x234E237E, 0xC4B7EB46, 0x23EBFCE3, 0xB07C5BED, 0x20FFD8CD,
	0xFCE311EB, 0x2ADA93CD, 0x2AE5B073, 0x5D23B071, 0x4E141854, 0x05044623, 0x0A2B0B28, 0x0300064F, 0x18B0ED03, 0xC1092302, 0xFFDECDC5, 0x2A1BE538, 0xCDEBB071, 0xCDD1FFE4, 0xD5F5FFD8, 0xEBFFF5CD,
	0xC1B07122, 0x5E121823, 0x1A2B5623, 0x00362377, 0x2B72EB23, 0x00266F73, 0xFFDECD19, 0xC1F1E938, 0x2AC9E1D1, 0xDECDAE6C, 0x7A0BD0FF, 0x120A0B02, 0xCDC9027B, 0x76D2FF4F, 0xDD2ACDBD, 0x21B0A022,
	0xCDC9B0A1, 0xA121FEEB, 0xDD30C3B0, 0x30FE3BCD, 0xDD4ACD09, 0xCDFF35DA, 0x7CCDFE78, 0xBEC3D8BD, 0xFE3BCDCB, 0x52CD0930, 0xFF35DADD, 0xCDFE78CD, 0x18D8BD82, 0xFE3BCDE9, 0x5BCD0930, 0xFF35DADD,
	0xCDFE78CD, 0x18D8BD85, 0xFE3BCDD5, 0xC3DE02DA, 0x70CDBD8E, 0xCDD5EBFE, 0x01D1BD88, 0xB0ED0005, 0xCBB5CAD8, 0xCDCBBEC3, 0xCDEBFEC3, 0x35DADD9C, 0x211028FF, 0x89C38000, 0xFEC3CDFE, 0xDDA3CDEB,
	0xCDFF35DA, 0xCD0BCB45, 0xA57BFEC3, 0xC3A27C6F, 0xC3CDFF34, 0x6FB57BFE, 0xF318B47A, 0x7BFEC3CD, 0xAA7C6FAD, 0xB6CDE918, 0x6F2F7DFE, 0xDF182F7C, 0xF0FDC4CD, 0xD2FF4FCD, 0xEDCDBD91, 0xB0A022DD,
	0xC9FE89D4, 0xCCCDE5C5, 0xC9C1E1FD, 0xDAFF4FCD, 0x94C3DDF9, 0xCD79E5BD, 0xCDD1FF6C, 0x3078FF4F, 0xCDF0B70B, 0xF4CDFE93, 0xFEB6C3FD, 0x110520B7, 0x2618BD6D, 0xCD78C5D5, 0x6DDCBD79, 0xD1C178BD,
	0x67CD0830, 0xC390AFBD, 0xC3EBBD79, 0x7011FF6F, 0x110318BD, 0x4FCDBD73, 0xFECDD8FF, 0x9F3AD0FF, 0xFE2CCDB0, 0xFF45CDD8, 0xBD67C378, 0xD003FE79, 0x6F66237E, 0xD0DD37CD, 0x79FF35C3, 0x2D2803FE,
	0xFEB09F3A, 0xB9262803, 0x0C301728, 0xB09F21E5, 0x8CCD2371, 0xC9B7D1FE, 0xB7FE8CCD, 0xB0A021EB, 0x2002EEC9, 0x56235EF7, 0x37B0A02A, 0xFF62C3C9, 0xB1B09F3A, 0xC32002FE, 0xCDB0A02A, 0x6F2AFE93,
	0xFE8CCDB0, 0xB0A021EB, 0x0818AFC9, 0x2B56235E, 0x7C08187A, 0xB09F21EB, 0xEB230536, 0xEDFCB7F5, 0x64C3F1DD, 0xB0A022BD, 0xB0A222EB, 0x36B09F21, 0xC3AF2305, 0xBCCDBD67, 0x3F18D8FE, 0x22FECECD,
	0x79C9B0A0, 0xEBFED5CD, 0xD8FECEDC, 0x9F212D18, 0x02367EB0, 0x3803FE23, 0xFF62CA0D, 0xBD6ACDC5, 0xDD37DC47, 0x237EC9C1, 0xCDC96F66, 0xCDD8FF4F, 0x0730BD6A, 0xDD37FC47, 0xCDFF35DA, 0xE506CB45,
	0x9F21C5D5, 0x0DC4BEB0, 0xE1D1C1FF, 0x3803D6C9, 0xFF5ECAA5, 0xDAFF4FCD, 0xE5C9FE93, 0x22000021, 0xA222B0A0, 0xB0A322B0, 0xC4CDC9E1, 0x9F876FFD, 0xAF6F0218, 0xB0A02267, 0x9F32023E, 0xA021C9B0,
	0x18053EB0, 0xB09F21F5, 0x3AC9234E, 0x3AC9B09F, 0x03FEB09F, 0xA02A0C28, 0xA021D8B0, 0x66CDC9B0, 0x45CDC8FF, 0x9F3A0DCB, 0xC903FEB0, 0x11B09F32, 0x1318B0A0, 0x9F3AE5D5, 0x72CD4FB0, 0xFF83CDF6,
	0xEBC9D1E1, 0xC5B0A021, 0x4FB09F3A, 0xB0ED0006, 0xABCDC9C1, 0x3F41FEFF, 0xC95BFED0, 0xD8FF92CD, 0xC8372EFE, 0xD03F30FE, 0xFEC93AFE, 0x7BFED861, 0xC920D6D0, 0x2346C5F5, 0xBE2323E5, 0x10032823,
	0x7EF1E3F8, 0xC16F6623, 0x4FC5C9F1, 0x0528B77E, 0xF820B923, 0xC9C17937, 0x7DC0927C, 0x907CC993, 0xC9917DC0, 0x52EDB7E5, 0xC9E14D44, 0x7800064F, 0xB0EDC8B1, 0xC8B178C9, 0xE9C9B8ED, 0xC9D5C9C5
};

static int ROMDISC[ 0x1000 ] = {
	0x00050001, 0xBCC3C072, 0xC1B2C3C1, 0xC3CCD1C3, 0xE4C3CCD5, 0xCCFDC3CC, 0xC3CD01C3, 0xDAC3CD18, 0xCDDDC3CD, 0xC3CDE4C3, 0x2EC3CDFE, 0xD48AC3D4, 0xC3D4C4C3, 0x0DC3CA72, 0xC581C3C6, 0xC3C666C3,
	0x52C3C64E, 0xC763C3C6, 0xC3C630C3, 0x68C3C603, 0xC0DBC3C1, 0xC3C389C3, 0xDBC3C301, 0xC3F7C3C3, 0xC3C435C3, 0xE3C3C445, 0xC3FFC3C3, 0xC3C43AC3, 0x5043C44B, 0x4F52204D, 0xCD5043CD, 0xC3534944,
	0x43534944, 0x44CE492E, 0x2E435349, 0x54D4554F, 0x54C55041, 0x2E455041, 0x4154CE49, 0x4F2E4550, 0xC2C1D455, 0x56495244, 0x455355C5, 0xD24944D2, 0x52C15245, 0x8281CE45, 0x86858483, 0x00898887,
	0x2200392A, 0xC33EAD3E, 0xAFAD3332, 0xF3AD4032, 0x3C43EDD9, 0xFA21D9AD, 0x211A18C0, 0xC8BEAD40, 0xB77746C5, 0xE728C178, 0xEDD908F3, 0xB7AD3C4B, 0x3221D908, 0xAD3422C1, 0x08F3C9FB, 0xAD3822D9,
	0x3673EDE1, 0xC00031AD, 0xFDF5C5D5, 0x3C4BEDE5, 0x4FCDB7AD, 0xD908F3C1, 0xAD3C43ED, 0x22C16321, 0xE1FD0039, 0x2AD1C1F1, 0xD908AD38, 0xAD367BED, 0x08F3C9FB, 0x73EDE1D9, 0x0031AD36, 0xC14FCDC0,
	0x6321D9F3, 0x003922C1, 0x367BEDD9, 0xEDC9FBAD, 0xEDAD3E5B, 0xFD003953, 0x5EAC4821, 0x08D55623, 0xCDC9FBD9, 0x0038AD33, 0xE1AD3A22, 0xE32323E5, 0xAD3A2AE5, 0x21AD33C3, 0x3922C163, 0xC3E9EB00,
	0xBEC3C1B2, 0xC2E1C3C2, 0xC3C2C3C3, 0xD2C3C2C8, 0xC2D7C3C2, 0xC3C2DCC3, 0xF2C3C2E9, 0xC524C3C2, 0xC3C529C3, 0xF7C3C51A, 0xC2FCC3C2, 0xC3C2CDC3, 0x12CDC55A, 0xDC214FB9, 0xBD16C3C1, 0x12CD0638,
	0x1828B7B9, 0x11D5E5FD, 0xE519FB00, 0xE1FDE523, 0xCDC5DDCD, 0xD1E1CCA0, 0xC937E1FD, 0xFDC00031, 0x11AC4821, 0xA501AD33, 0xCAAFCD00, 0x35AD4121, 0x0332813E, 0x0432AF00, 0xC0332100, 0x01BE8011,
	0xB0ED003F, 0xCDC0C0CD, 0x410EC5DD, 0x21000011, 0x66CD0100, 0xC2ACDCC6, 0x01EB0A30, 0x3331C17F, 0xC177C3AD, 0xB8CD0F3E, 0xCDDF18CA, 0xB0CDC86F, 0x014801C8, 0xE5000011, 0xE1C299CD, 0x30C2ACDC,
	0x5E23E551, 0xA4215623, 0xE1EB19FC, 0xED020001, 0x4901EBB0, 0x0000110A, 0x30C299CD, 0x0021EB35, 0x21E519EA, 0x3E19F206, 0x000532C3, 0x32000622, 0x03210000, 0x01221900, 0xC17F2100, 0xED003301,
	0x000421B0, 0xFE0FE67E, 0x36023802, 0xC3D14E00, 0x0E3EC177, 0x18CAB8CD, 0xC666CD92, 0xFE0C79D0, 0x0E033849, 0x24241441, 0xC937EF10, 0x000201E5, 0x3723BE7E, 0xF9100620, 0xB7F6200D, 0x33CDC9E1,
	0x21C22BAD, 0x1C18C486, 0x18C48F21, 0xC4982117, 0xA1211218, 0x210D18C4, 0x0818C4AA, 0x18C4BC21, 0xC47D2103, 0x6AAD33CD, 0xC168CDC4, 0x8921C51F, 0x33CDC9BE, 0xCDC4F0AD, 0xC54CAD33, 0x2EAD33CD,
	0xADC532C5, 0x11008101, 0xB0EDAD42, 0x22AD4321, 0x21C9ADC3, 0xB77EAD41, 0xCC350428, 0x09CDBB81, 0xBB0CDCBB, 0x4221C99F, 0x28B77EAD, 0xBB09CD1B, 0xC5210C30, 0xC03534AD, 0x36AD4221, 0x2A35C900,
	0x237EADC3, 0xC9ADC322, 0x7EAD4121, 0xBB81C4B7, 0x06C30036, 0xC91A3EBB, 0x21C9FF3E, 0xB77EAD41, 0x36BB84CC, 0x5ACD79FF, 0xD020FEBB, 0xCDBB78CD, 0xCDD8BB87, 0x8DC3BB8A, 0xBD2ECDBB, 0x79C99F3F,
	0xD8BD2BCD, 0x18C4D3CD, 0xDD01F3F6, 0xADC611FA, 0x03C3BDCD, 0xBDCD1303, 0x1E363EC3, 0xC3AECDDC, 0xCD1C763E, 0xB63EC3AE, 0xC3AECD1C, 0xDF01C9FB, 0x4B79EDFB, 0x79ED237E, 0x79ED237E, 0xED183EC9,
	0xED043E79, 0xED237E79, 0xED053E79, 0x23127E79, 0x033E79ED, 0x237E79ED, 0x01C979ED, 0xC621FADD, 0x010618AD, 0xC721FADF, 0x0F78EDAD, 0x24CDD89F, 0x0F78EDC4, 0x0129189F, 0xC621FADD, 0x010618AD,
	0xC721FADF, 0x0F78EDAD, 0x24CD1238, 0xC4C5CDC4, 0x0C281AFE, 0x300F78ED, 0xC420CDF4, 0xC978ED0B, 0x0218001E, 0xF5F3801E, 0x79ED053E, 0xB37FE67E, 0xFBF179ED, 0xFADD01C9, 0xDF010318, 0xE678EDFA,
	0x9F37C804, 0xDD0179C9, 0x790418FA, 0xF5FADF01, 0xCDC4D3CD, 0xF830C43D, 0x79ED0BF1, 0xC4B321C9, 0xBC210818, 0x210318C4, 0x2346C4A1, 0x0700033A, 0x06E6FD10, 0x195F0016, 0xEB56235E, 0xBEA701E9,
	0xC45DC313, 0xAA01BEB3, 0x62C326BE, 0x01BEB6C4, 0xC35BBEB0, 0xBEBCC467, 0x58BEAD03, 0xB9C379C3, 0xBEB003BE, 0xC37FC35B, 0xB005BEBC, 0xBCC35ABE, 0x07C35BBE, 0xC358BEA7, 0xC313BEB3, 0x55BEAA07,
	0x26BEB6C3, 0xC4D3CDC3, 0xE5C013FE, 0xC326CDC5, 0xE5C9E1C1, 0x13CDC5D5, 0x0F28B7C3, 0xFEC326CD, 0x3E082003, 0xCAEBCD0D, 0xC1C22BC3, 0xFFC9E1D1, 0x2102FE79, 0x7BD00000, 0x590F381F, 0x5CCD183E,
	0x0620B7CA, 0xC56CCDE5, 0x3279D0E1, 0x1021BE53, 0x0328B702, 0xC3022021, 0x43EDCA9F, 0xCDC9BE60, 0x000EC86F, 0xBE543279, 0x553279C9, 0x79C5C9BE, 0xEBCC02FE, 0xC800CDC7, 0xCDC81BDC, 0xD0C1C832,
	0x0DC8B6CD, 0xC86FCC37, 0xC9003ED0, 0xBE5932AF, 0xCDC832CD, 0x3ED0C8C7, 0x6960C900, 0xFB7E01C9, 0x5CCD4A3E, 0x5CCD7BC9, 0xC8F9C3C9, 0x3EC976CD, 0xCA5CCD16, 0x21100E57, 0xFFCDC55D, 0x513AD0C6,
	0xCDAFF5BE, 0xEBE5CA63, 0x01CA4321, 0xB0ED0016, 0xC0E6F1E1, 0xC83740FE, 0xFEC5CA11, 0x110328C0, 0x131AC5C0, 0x131A2377, 0x00040177, 0x77131A09, 0x77131A23, 0x09000701, 0x000601EB, 0xC937B0ED,
	0x009B0020, 0x08010001, 0x0024502A, 0x000000B3, 0x522A09C1, 0x00FA0032, 0x010C0FAF, 0xBE401103, 0xCD003D01, 0xF4CDCAAF, 0xC9E8CDC9, 0xCDC5D421, 0x12CDC60D, 0x80064FB9, 0x11BE6D21, 0xEFCDC9D6,
	0xE5103EBC, 0x32BE662A, 0xE17DBE66, 0xBE4411C9, 0xED000701, 0xFB7E01B0, 0x5CCD033E, 0xBE4A3AC9, 0x0707073D, 0xB6F0E62F, 0x23C95CCD, 0xC95CC37E, 0xD0C638CD, 0xC9BE4C3A, 0xF5C976CD, 0x01C947CD,
	0x043EFB7E, 0xF1C95CCD, 0xC3C95CCD, 0x453EC91C, 0x4D3E0218, 0x06C976CD, 0xC66DCD11, 0x3DBE483A, 0x20030303, 0x76CDC9FA, 0x06663EC9, 0xBE622210, 0x74226967, 0x7C2148BE, 0xC6FFC3C6, 0x01BE742A,
	0xCD7CFB7E, 0xCD7BC95C, 0xFE7CC95C, 0x3E16204D, 0xC959CD14, 0x59CD103E, 0xCD123EC9, 0x133EC959, 0x18CA5CCD, 0x5CCD7A1C, 0x5CCDAFC9, 0x5CCD7DC9, 0xCD143EC9, 0xCD7DC959, 0x113EC95C, 0x3EC959CD,
	0xC6D1CDFF, 0xC907CDFB, 0x4D3AC0D8, 0xAFD887BE, 0x5CCDF3C9, 0x622A7CC9, 0x2066FEBE, 0x0C061818, 0x0D7778ED, 0xF278ED23, 0x20E6C6E5, 0x0CC9F120, 0x0D79ED7E, 0xF278ED23, 0x20E6C6F5, 0x3AC9F120,
	0xCD47BE66, 0x28D8C72B, 0x04E67819, 0x16D50928, 0xC766CD27, 0xE5EC18D1, 0x63CD173E, 0xE10036CA, 0xC579E118, 0xC1CA7ACD, 0xCDC9D520, 0xC8D8C754, 0xCDC947CD, 0xC8D8C754, 0x0527FE7A, 0x14040A30,
	0x15C766CD, 0xD8C754CD, 0x20B77AC8, 0x15C90502, 0x14C766CD, 0xE5C766CD, 0x001ECDC5, 0x20D8E1C1, 0xCDC905F3, 0xD5E5C976, 0xBE663AC5, 0xCD173E47, 0xB77ECA63, 0x01C51F20, 0x073EFB7E, 0x7BC95CCD,
	0x3EC95CCD, 0xC7C7CD28, 0x163E2A30, 0x36CA63CD, 0xFF362300, 0x927E2BC1, 0x01C52828, 0x0F3EFB7E, 0x7BC95CCD, 0x7AC95CCD, 0x96C95CCD, 0x927E0230, 0xC7C7CD72, 0x200B38C1, 0xADCA05BD, 0xC947CDC9,
	0xD1C1B418, 0xF5C937E1, 0xCDBE4A3A, 0x3DF1C7E0, 0x493AF520, 0xC7E0CDBE, 0x5CCD083E, 0xC8F9C3C9, 0xC9C9C9C9, 0xC9C9C9C9, 0x21C9C9C9, 0x3E5EBE53, 0xCA5CCD03, 0xBE59113C, 0x03011312, 0xC9B0ED00,
	0x1ABE5911, 0x2113C8B7, 0x0306BE53, 0x0620AE1A, 0xF8102313, 0x32AFC937, 0xF5C9BE59, 0x35BE5921, 0x23235E23, 0x5CCDAF34, 0x0420BECA, 0x342B0036, 0xCDF5C9F1, 0x1938C854, 0xC1C86FCD, 0x80CDC5D0,
	0x0638F1C8, 0xCDC8A2CD, 0x9FF5C666, 0xF1BE5E32, 0xC937F1C9, 0xB7BE5E3A, 0xBE5301C8, 0x5EBE5621, 0x03C0AE0A, 0xC0AE0A23, 0xAEC892CD, 0x21C937C0, 0x0036BE5E, 0x37B77E2B, 0xA2CD34C8, 0xC64EC3C8,
	0x01BE5621, 0x770ABE53, 0x0A03235F, 0xC892CD77, 0x2303C977, 0x5CCD153E, 0xCB0A57CA, 0x3FCBD83A, 0x5BEDF918, 0x0F3EBE56, 0x21CA5CCD, 0x4F86BE58, 0xC302B021, 0xD5E5CA9F, 0xFF3EF5C5, 0xCDBE5D32,
	0x1BCDC8D6, 0xE50A18B9, 0xCDF5C5D5, 0xEDEBC8D6, 0xD1C1F1B0, 0x5321C9E1, 0x153E5EBE, 0x3DCA5CCD, 0x11A62323, 0x30210080, 0x3D193C02, 0xCDEBFC20, 0x602ACA98, 0x008001BE, 0xC91CCDC9, 0xBE4C3AD8,
	0x3EC808E6, 0xCD0D1813, 0xC0D8C8F9, 0xE6BE4D3A, 0x123EC802, 0xD8CA7ACD, 0xC9C9ADCA, 0x0016D5E5, 0xE5BE4C21, 0xC0FE78ED, 0xED0CFA38, 0x23770D78, 0x00000014, 0x78ED0000, 0xE82010E6, 0xC0E67EE1,
	0xE1D1722B, 0xC5C937C0, 0x3EFB7E01, 0xC95CCD08, 0xFEC91CCD, 0xC1F42080, 0xCA5CCDC9, 0x78EDF5F5, 0x87FB3087, 0xF1F10330, 0xED0CF1C9, 0xC9F10D79, 0x00000000, 0x7622C9F1, 0xC5D5E3BE, 0xBE6473ED,
	0xC9AD21E5, 0xC5D5E5E3, 0xC9DFCDF5, 0xB7BE5F3A, 0x7E011420, 0xED013EFA, 0x445BED79, 0xC9CDCDBE, 0x00000000, 0xC1F10000, 0xBE762AD1, 0x647BEDC9, 0x5BEDF5BE, 0xCDCDBE46, 0xD1C1F1C9, 0xD8003EE1,
	0x7EBE4C21, 0xB62308E6, 0x2B2B40F6, 0xBE6721C9, 0xC3000001, 0x5F21BCE9, 0x772F7EBE, 0x210628B7, 0xECC3BE67, 0xC9DFCDBC, 0x7E01003E, 0xAF79EDFA, 0xC9BE5F32, 0x11022021, 0x03CD01D0, 0x021021CA,
	0xCD019011, 0x53EDCA98, 0xCDD5BE42, 0x4022CA9F, 0x4321E5BE, 0x001901CA, 0x424BB0ED, 0x230036E1, 0x07110036, 0x30111900, 0xCA98CD02, 0x23722373, 0x722373D1, 0x70237123, 0x1021EB23, 0x73EB0900,
	0x24C97223, 0x00070300, 0x003F00AA, 0x001000C0, 0x09410002, 0x02E5522A, 0x00000004, 0xCA63CDE5, 0xD5C9E17E, 0x1DBE422A, 0x20004011, 0x195F1901, 0x782AC9D1, 0xBE7832BE, 0x3AF5C97D, 0x20B7BE78,
	0xC34BF105, 0xAFF1CAB8, 0xFFFFFFC9, 0xFFFFFFFF, 0x09E3E5FD, 0xC9E14D44, 0x19E3E5FD, 0xD5C9E1EB, 0x19D1E5FD, 0x61FEC9D1, 0xD07BFED8, 0xAFC9E0C6, 0x780B1312, 0xC9F820B1, 0x3ECAEBCD, 0xCAEBCD14,
	0x38BB09CD, 0xBB81CDFB, 0xCDBB06CD, 0x43FECAA6, 0x49FE1128, 0xFE0C2837, 0x3E072852, 0xBB5ACD07, 0xCDB7E618, 0x84CDBB5A, 0xE5003EBB, 0x7FE6F5C5, 0x47CB8621, 0x7E051804, 0xFB203C23, 0x237EF910,
	0x0B28FFFE, 0xCDC5D5E5, 0xD1C1CB13, 0xF1EF18E1, 0xB7C9E1C1, 0xFECB66F2, 0xFE4628FE, 0xFE1A28FC, 0x06C820FD, 0xCB2FCD08, 0x83CD2E3E, 0x130306CB, 0xCD7FE61A, 0xF710CB83, 0x2016EBC9, 0xCDFF9C01,
	0xF601CB4D, 0xCB4DCDFF, 0x1830C67D, 0xE5FF3E36, 0x0430093C, 0xF718E1E3, 0x0228B7E1, 0x18823016, 0x41C67922, 0xFEF51D18, 0xE5172020, 0xBB69CDD5, 0x7ABB78CD, 0x303F04D6, 0xE1D1BC01, 0xC3F10430,
	0xC3F1CAE9, 0x0A0DBB5A, 0x202020FF, 0xFF4BFCFF, 0x66208297, 0x97656572, 0x614280FF, 0x6F632064, 0x6E616D6D, 0x9BFF8064, 0x65726C61, 0x20796461, 0x73697865, 0xFF807374, 0x746F6E9B, 0x756F6620,
	0xFF80646E, 0x72696495, 0x6F746365, 0x9A207972, 0xFF9A98FF, 0x61686398, 0x6465676E, 0x6C63202C, 0x6E69736F, 0x80FD2067, 0x73699BFF, 0x6F209D20, 0x80796C6E, 0x95FFFDFF, 0x72657375, 0x2EFF80FC,
	0x435E2E2E, 0x504396FF, 0xFF804D2F, 0x6F6F6296, 0x65732074, 0x726F7463, 0x9D95FF80, 0x9C95FF99, 0x6998FF99, 0x209C2073, 0x746F7270, 0x65746365, 0x98FF8064, 0x7373696D, 0x80676E69, 0x655280FF,
	0x2C797274, 0x6E674920, 0x2065726F, 0x4320726F, 0x65636E61, 0xFF203F6C, 0x69724480, 0xFE206576, 0x80FF203A, 0x6C696146, 0x74206465, 0x6F6C206F, 0xFF206461, 0x95FF8080, 0x63736964, 0x6620FF20,
	0x806C6961, 0x6C7566FF, 0x80FF806C, 0x77FF20FD, 0x65746972, 0x616572FF, 0xFFFFFF64, 0xFFFFFFFF, 0x0077FDAF, 0x3D0177FD, 0xFD0877FD, 0x22FD2C77, 0x7721BE7D, 0x016411BC, 0x01CA98CD, 0xB0ED0027,
	0x233036EB, 0xCD23CD36, 0x3E77B912, 0xBE7F32C9, 0xCCE4CDAF, 0xBC7721D0, 0xE9CD0706, 0x9B21D0CC, 0x051804BC, 0x06BC8C21, 0x3F20B705, 0xCD018B11, 0xDF36CA98, 0x72237323, 0x37F71023, 0xCD18CDC9,
	0x016421D0, 0x01BC7711, 0x21CD0015, 0x8821D0CD, 0xBC9B1101, 0x0918030E, 0x11017921, 0x0F01BC8C, 0x0720B700, 0xEDCA9FCD, 0x3EC937B0, 0xCAEBC304, 0xBE7D2AFD, 0x79D908F3, 0xE3E1C1D1, 0x064FD5C5,
	0x10D2117F, 0x08D9E519, 0xBE7FC3FB, 0xC3CEAFC3, 0xBCC3D1B6, 0xCF64C3D1, 0xC3CFF5C3, 0x65C3D069, 0xCF37C3D0, 0xC3D1D8C3, 0x8FC3D1C2, 0xD0D8C3D0, 0xCDD513C3, 0xE5C9CD77, 0x39000621, 0xFD0675FD,
	0xC9E10774, 0xF5CD77CD, 0x18087EFD, 0xCD77CD07, 0x2C7EFDF5, 0x1228FFFE, 0xF1CE16CD, 0x087EFDC9, 0x7EFD0318, 0xCD77CD2C, 0x0E3EC83C, 0x3E0A18B7, 0xDBCACD04, 0x80F60CC6, 0x066EFDBF, 0xF90766FD,
	0xC83D3DC9, 0xCDCDAFC3, 0x2346CDCF, 0xDDDBF9C3, 0x66DD006E, 0xDD23DD01, 0x18AFC923, 0xCD013E02, 0x1318CD73, 0xCDCD73CD, 0xC7CDCDC2, 0xAFC205CD, 0xA6CD7ECD, 0xCD41D6CA, 0x77FDCE16, 0x73CDC900,
	0xCDC2CDCD, 0x11CDCFCD, 0xF3CD0010, 0xCDAFD2DB, 0xC90175FD, 0xD5E5030A, 0x1E4FF5C5, 0x087EFDFF, 0xFD0828B9, 0x28B92C7E, 0xD5001E02, 0xC4F0CDC5, 0xB57CD1C1, 0xFDCDAFCA, 0x74FD0375, 0x0573FD04,
	0xF10271FD, 0xC9E1D1C1, 0xCD005021, 0x11E5CE5A, 0x36190042, 0x21C9E180, 0xD5C5009A, 0x36CA9FCD, 0x23732300, 0x23732372, 0xC5E52372, 0xEB004501, 0xC1CAAFCD, 0xD5D16960, 0xED000C01, 0xE5D1E1B0,
	0x09001201, 0x23231636, 0x72237323, 0xE1FF3623, 0x21E5C9C1, 0x06540000, 0x237EE343, 0x10195FE3, 0xC9E1EBF8, 0xCE92CDE5, 0xE1722373, 0xCDD3F9C3, 0xCDD5CD9D, 0x14CDDA6F, 0x000921CE, 0x283C7E09,
	0xD651CD08, 0x18D50CD2, 0xD2A8CD1E, 0x38D651CD, 0xD2B3CD16, 0x38D651CD, 0xD2B7CD0E, 0xF5D651CD, 0xF1D2A8D4, 0xD1D50CD2, 0xE5CE48CD, 0xCD000811, 0x0A0BCA98, 0xD79CCD12, 0xCD00E421, 0x92CDCA9F,
	0xE51F30D3, 0xCE92CDD5, 0xCDDBF9CD, 0xE1D1DBF3, 0x55110D20, 0xCA98CD00, 0xED004501, 0xCD0318B0, 0xE5E1D79C, 0x19001511, 0x2356235E, 0x46234E23, 0xFD9F37E1, 0xCDC9677E, 0xCDD5CDA2, 0x14CDDA6A,
	0x57CDD1CE, 0xABCDE5CE, 0xD676CDD2, 0x112B6960, 0x98CD002C, 0x000D01CA, 0x1701B0ED, 0xCAAFCD00, 0xC99F37E1, 0xCDC5D5E5, 0xD1C1CF74, 0x1AFED0E1, 0xC9B7C037, 0xFDCD84CD, 0x5021D1E5, 0xFE7E1900,
	0xCDAACA02, 0x95210136, 0x237E1900, 0x28B623B6, 0x00682136, 0xB6237E19, 0xCFCBCC2B, 0x28B6237E, 0x4E2B4626, 0x7023710B, 0x19009521, 0xD67E0306, 0x03307701, 0x21F71023, 0x5E190053, 0xE7EB5623,
	0x2B7213EB, 0x3EC93773, 0xE5C9B70F, 0x5121E5D5, 0xF9CD1900, 0x1001E5DB, 0xD049CD00, 0x4791103E, 0x38CB000E, 0xE1D119CB, 0x01702371, 0x7309FFEA, 0xE1D17223, 0xCD84CDC9, 0x005021E5, 0x7ECA9FCD,
	0xAACA01FE, 0x110236CD, 0x5E190045, 0xD5E15623, 0x073EEBE5, 0x44DBEBCD, 0x49CDE14D, 0x1E30D1D0, 0x287FE67B, 0x21E5F519, 0x9FCD00E4, 0x0101E5CA, 0xD049CD00, 0x30C1D1E1, 0x00064805, 0x6F21B0ED,
	0xCA9FCD00, 0xF9C39F37, 0xCD1418DB, 0x11D0D392, 0x98CD0067, 0xDC1F1ACA, 0x8011D252, 0x780B1900, 0x37E820B1, 0xCF64CDC9, 0xF5D5E5D0, 0xCD005321, 0x235ECA9F, 0x2B721B56, 0x215D5473, 0xCD190042,
	0x1521D7AB, 0x20341900, 0xF1342302, 0xCDC9E1D1, 0xD5E5CD8D, 0xE5FDF5C5, 0x009A21D1, 0x02FE7E19, 0x36CDAACA, 0x00B22101, 0xF9CDE519, 0xF80001DB, 0x18DCD509, 0x34E1D1D1, 0x34012023, 0x1900DF21,
	0x21D7ABCD, 0xF119009D, 0x2B46234E, 0x02203402, 0xD1C13423, 0xC99F37E1, 0xF5CD8DCD, 0x9A21D5E5, 0xCA9FCD00, 0xCA01FE7E, 0x0236CDAA, 0x19002011, 0xC1712B70, 0x712B702B, 0x19002911, 0x11712B70,
	0x7119FFD3, 0x23C17023, 0x11702371, 0x7119FFE6, 0x11F17023, 0x77190015, 0x21D1E5FD, 0x7E1900B6, 0x211828B7, 0x7E1900B1, 0x06FE0FE6, 0x2C210D28, 0xEBD51900, 0xCDD7A7CD, 0x21D1D77D, 0xE51900B2,
	0x0156235E, 0xCD09FFE8, 0xCDE5DBF9, 0xE1C1D164, 0x36230036, 0x23232300, 0xE7110036, 0x237119FF, 0xC99F3770, 0xEB073ED5, 0xEBDBEBCD, 0x88CD4B42, 0xE679C1D1, 0x064FC87F, 0x00E41100, 0xD5CA98CD,
	0x3EB91BCD, 0x03E1121A, 0x11E52718, 0x98CD00B1, 0x301F1ACA, 0xE411C513, 0xCA98CD00, 0x008001D5, 0xE1B91BCD, 0xD252CDC1, 0xE1D3AFCD, 0x19008011, 0x20B1780B, 0x84CDC9D5, 0xC9E5CDCD, 0xFF0836FD,
	0x8DCD6E18, 0x002D11CD, 0xAFCA98CD, 0x1BD83CCD, 0xCD12FF3E, 0x5818C51F, 0xCD00DF21, 0x237ECA9F, 0x28B623B6, 0xCD8DCDDD, 0x11D118CD, 0x98CD002C, 0x8CCDD5CA, 0x009F01D7, 0x21CA90CD, 0x5E090012,
	0x09000921, 0x16203C7E, 0x200EE67B, 0xD2B3CD05, 0x02FE0C18, 0xB7CD0520, 0xCD0318D2, 0x6960D2A8, 0xFE0FE67B, 0xCEA4C406, 0x02FF3EC1, 0xD2DACD03, 0xFDC99F37, 0x36FD0266, 0x08110005, 0xD243CD00,
	0xCD002C11, 0xBC1ACA98, 0x12FF3EC0, 0xC3093E13, 0xC5E5DBCA, 0x010111E5, 0x0E188106, 0xAEE3E7E3, 0xE300AEDD, 0xDDE32377, 0x20152323, 0xDD0B1606, 0x1DD28121, 0x0D1E0520, 0x10D28C21, 0xE1D1E1DF,
	0x36B149C9, 0x061E2EF0, 0xEA19282A, 0x1ADB9DE2, 0xC6392942, 0x4590C6B3, 0x2020208A, 0x42242424, 0x41424B41, 0x4E494253, 0x3E0E18AF, 0x3E0A1803, 0x3E061806, 0x3E021809, 0x99C6D50C, 0x93D2CE5F,
	0xD5071857, 0xCD00A811, 0xC5E5CA98, 0x09000921, 0xEB000301, 0xE1C1B0ED, 0x0C21C9D1, 0xFF360900, 0xFF362323, 0xE5D683CD, 0xE3000021, 0xE3D6A2CD, 0xAFCD2830, 0xD7D8CDD2, 0x01260830, 0x38D9D9CD,
	0xC3CD2401, 0xD7D8CDD2, 0x012E0830, 0x38D9D9CD, 0xB77C2C01, 0xB77DD528, 0x7DF1D128, 0x3D4328B7, 0xB77C6628, 0x283D3C28, 0xD683CD45, 0xD0D6A2CD, 0x18D335CD, 0xD2AFCDF7, 0xDAD7D8CD, 0x51CDD4AA,
	0xC3CDD8D3, 0xD7D8CDD2, 0x4B42C5D0, 0x18D2AFCD, 0xD2ABCD0D, 0xD0D7D8CD, 0xCD4B42C5, 0xC3C1D2C3, 0x83CDD97A, 0xD6A2CDD6, 0xD33ECDD0, 0x83CDF718, 0xD6A2CDD6, 0xD37ACDD0, 0x51CDF718, 0xC3CDD8D3,
	0xD7D8CDD2, 0xC9D4AADA, 0x50D2C3CD, 0xC30A3E59, 0xD5E5CDB1, 0x0811E5C5, 0xCA98CD00, 0x30D410CD, 0xCDE3EB08, 0x18D1D9E8, 0xD1C1E148, 0xE5C9B7E1, 0x11E5C5D5, 0x98CD002C, 0xD6C8CDCA, 0x083E0B38,
	0xCDCDB1C2, 0xFACDD78C, 0xD72FCDD6, 0x1838000E, 0x93CDEBD5, 0x083EEBD8, 0x73CDB1D2, 0x0228B778, 0xCDD17223, 0x020ED72F, 0xF3CDE3EB, 0x7DCDD1D9, 0xD7A7CDD7, 0x37E1D1C1, 0x2C11E5C9, 0xCA98CD00,
	0xCDD79CCD, 0xE1EBD410, 0xF3DA000E, 0xCDAFC3D9, 0x38D6C8CD, 0xFACDC012, 0x4B42D5D6, 0xB3CDC503, 0xDCD1EBD7, 0xDCD1DBDF, 0x2FDAD70C, 0x73CDC9D7, 0xB70006CD, 0xC2CD0628, 0xCDC7CDCD, 0xCDDAA6CD,
	0xD0CDCE14, 0xCD0C3EDB, 0xE565D472, 0xCDD683CD, 0x1A30D698, 0x38D9DFCD, 0x7CC5E3F6, 0xDBC4C4BD, 0xCDCAE9CC, 0x202DDBC8, 0xE3C16C01, 0xC3E1E118, 0x03C6D571, 0xCDE5D567, 0xE17ABB69, 0x2E04C6D1,
	0x30942C00, 0x2EC02DFC, 0x73CDC901, 0xCDC2CDCD, 0xCDCDC7CD, 0x14CDDA8D, 0xD683CDCE, 0x30D698CD, 0xD4B1CD6B, 0x38D698CD, 0xB1CDC9F8, 0xCDB8D2D4, 0xD9D9CDC9, 0xD20A3E3F, 0xCDAFDBCA, 0xE53ED83C,
	0xD97AC312, 0xCDCD73CD, 0xC7CDCDC1, 0xDA5BCDCD, 0xCDC7CDC5, 0xE1DA60CD, 0xAFC2BE0A, 0xCE14CDCD, 0x44CDE523, 0x44C5E1D6, 0xD683CD4D, 0x30D698CD, 0xD9D9CD1B, 0xE3D38DDA, 0x0C01C5E5, 0xC1B0ED00,
	0x7ACDE3E1, 0xD698CDD9, 0xC9E1E738, 0x063E5950, 0xCDCDB1C3, 0xDDD5CD73, 0x080001E1, 0xCDCAAFCD, 0x14CDDA86, 0xDBD0CDCE, 0x83CDF5AF, 0xD698CDD6, 0xDFCD0C30, 0xE3F638D9, 0xE3D5AACD, 0x113EEF38,
	0x55D472CD, 0x1C001EF1, 0x82FC3092, 0xDD1D0120, 0x424BE1E5, 0xD57ACDE5, 0x0026EBD5, 0x19D63ACD, 0xE1F210D1, 0x000E11D5, 0x280DD119, 0x20B77E04, 0xD8C2CDE2, 0xC3B7033E, 0xB7E7CAEB, 0xC5D5E5C8,
	0xC4C4BA78, 0xCAE9CCDB, 0xDBC8CDEB, 0x3ED9D9CD, 0x3E02382A, 0xBB5ACD20, 0x19000C21, 0xE7235FE7, 0xCD023E57, 0xD1C1CAEB, 0x4CC5C9E1, 0xE5DD0006, 0x28B7E7E1, 0x23CD0450, 0x300F28D6, 0x0E11D526,
	0x78D11900, 0xE93892FE, 0xCDE55618, 0x11E3D8F2, 0xE719000C, 0x57E7235F, 0xEB19E32B, 0x722373E1, 0x793D1837, 0x382892FE, 0xEBC5D5E5, 0x6F3C9079, 0x3ACD0026, 0x194D44D6, 0x0E21EB2B, 0xCDEB1900,
	0xD1C1B91E, 0xD5C50CE1, 0x1323FF36, 0x000B01EB, 0xEBB91BCD, 0xF2CDEBE3, 0x73E1EBD8, 0x37C17223, 0xE5C9C161, 0x0B06C5D5, 0xE61A2313, 0xE6E74F7F, 0x0220B97F, 0xD1C1F210, 0x54D5C9E1, 0x2919295D,
	0xC9D12919, 0xCDD683CD, 0x2530D698, 0xB1C3053E, 0xD683CDCD, 0x30D698CD, 0x0921E518, 0xCA9FCD00, 0xDBDFCDEB, 0x057EFDE1, 0xCDC037B7, 0xFB38D6A2, 0x0536FD37, 0x83CDC9FF, 0xD698CDD6, 0xAACDF330,
	0xC5F618D4, 0xC1C51FCD, 0xFDFFFF21, 0xC0B7057E, 0xD814CDE5, 0xD9A8C3E1, 0xD0D6A2CD, 0x30D7D8CD, 0xFD23C9F7, 0x20B7057E, 0xD91CCD11, 0xE5FE1AD0, 0xA8CDC837, 0xC3FF3ED9, 0xB8CDD83C, 0x1CC3D0D9,
	0x002121D9, 0xDBF9C319, 0x19002321, 0xCDC0B77E, 0x1F7CD6C1, 0xE61F1F1F, 0x7C29470F, 0xC54F1FE6, 0x19000F21, 0x0F20A87E, 0x54CD043E, 0x2B472FDA, 0xA0A97E2B, 0xC1370120, 0x0D21C99F, 0x23711900,
	0xEB237023, 0xCD001101, 0xC9EBCAAF, 0xD6C1CDD5, 0x670FE67C, 0x001021E5, 0x00064E19, 0x662B2B2B, 0xCD013E68, 0xD109DBEB, 0xDBF3CD13, 0xCDC9D13F, 0x033ED6C1, 0xA5DA54CD, 0xCD023E4F, 0xEBCDDA54,
	0xCD063EDB, 0xB747DA54, 0x0011217D, 0xE60E2819, 0x6F858707, 0xE567958C, 0x18DBF9CD, 0x850FE60B, 0x67958C6F, 0x00266EE5, 0x0F28B57C, 0xCD023EF1, 0x3D29DA54, 0xB579FC20, 0xE1C9376F, 0x001021C9,
	0xB7347E19, 0x2B0136F0, 0xC9342B2B, 0xBBCDD5D5, 0xCD23E3D7, 0xCDE1DBDF, 0xC9D1D97A, 0x19002121, 0x772377AF, 0x21C97723, 0x34190021, 0xC03423C0, 0xCDC93423, 0x98CDD683, 0x211118D6, 0xCD23FFFF,
	0x073ED91C, 0x1ACDB1D2, 0xF220E5FE, 0x057EFDF5, 0xCA093EB7, 0xC9F1CDB8, 0x60E5D5C5, 0x20AE1A69, 0x0613232D, 0x3FFE7E0B, 0xAE1A0628, 0x1E207FE6, 0xF1101323, 0x0C283C7E, 0x54CD043E, 0x1A472FDA,
	0x0A20A0AE, 0x13231323, 0x02283C7E, 0xD1E1AE1A, 0xC937C0C1, 0x45CD053E, 0xCD033EDA, 0xEB23DBEB, 0x3FCD0E3E, 0x230036DA, 0x20B37A1B, 0xCD093EF8, 0x3EEBDA45, 0xDA3FCD0E, 0xC9722373, 0x4FC5D5E5,
	0x19001021, 0x235E1006, 0x54CD063E, 0x0328B7DA, 0x57237E05, 0xE50E28B3, 0x45CD053E, 0x7C937DDA, 0xD86CD49A, 0xC1DF10E1, 0xC937E1D1, 0xEBD5D5C5, 0xEBCD033E, 0x0E3EEBDB, 0x19DA3FCD, 0x07E67BD1,
	0x1C013E5F, 0xFC201D0F, 0x784FA147, 0x77B1A62F, 0xC5C9C1D1, 0xCD053ED5, 0x3EEBDA45, 0xDA3FCD0E, 0x7E088001, 0x0F0C28A1, 0x28B37A4F, 0xF3101B12, 0x7EED1823, 0x053E77B1, 0xB7DA45CD, 0xD13752ED,
	0xE5C5C9C1, 0xE5000021, 0x45CD053E, 0x0E3EEBDA, 0x01DA3FCD, 0xA17E0880, 0x23E30320, 0x4F0F79E3, 0x0628B37A, 0x23EF101B, 0xCDE1E918, 0xE1EBD910, 0x21D5C9C1, 0x11190010, 0x063E1000, 0xB7DA54CD,
	0x0328237E, 0xB72315B6, 0x151C0128, 0xD1EBEC20, 0x54CD023E, 0x3D3D3DDA, 0xFB1829C8, 0xE67DC5E5, 0xEB112003, 0x45CD073E, 0xDBF3CDDA, 0x1530EB3F, 0xAFD948CD, 0xCD083E47, 0x2011DA3F, 0x01180400,
	0xEBFD1019, 0xC9E1C137, 0xEBCD023E, 0x083EEBDB, 0xCDDA3FCD, 0x0B3ED9E8, 0xEBDA45CD, 0xEBDBF3CD, 0xCD0C3ED0, 0xCD19DA3F, 0xC8BED9C8, 0x2929EBF5, 0xEBD9B8CD, 0xD233DAD1, 0xC5E5C972, 0xEBCD023E,
	0x083EEBDB, 0x0EDA3FCD, 0xD9F3CD01, 0x45CD0B3E, 0xF3CDEBDA, 0x0A30EBDB, 0x3FCD0C3E, 0xC8CD19DA, 0xE1C177D9, 0xD8D9B8CD, 0x13EBE5D5, 0x35CD023E, 0x722373DA, 0xC937D1E1, 0x023EE5D5, 0x5EDA35CD,
	0xCDE15623, 0xC9D1DBF3, 0x8006E5C5, 0x3FCD083E, 0x2386AFDA, 0xC1E1FC10, 0x0921E5C9, 0xE5041800, 0x19000A21, 0xC9E1877E, 0xCDE5D5C5, 0x4CCDDA06, 0xC50B18C5, 0xCDC5E5D5, 0xCDC1DA06, 0xC2B7C52E,
	0xD1E1CDB6, 0x44D5C9C1, 0xC51ACD4D, 0xCD0D3ED1, 0x4D44DA45, 0xDA45CDAF, 0x957B030B, 0x579C7A5F, 0xE519F730, 0xC1C524CD, 0xDA3FCDAF, 0xC55ACDEB, 0x29C3444D, 0x0386FDC5, 0x048EFD6F, 0xCDC96795,
	0xF9C3DA35, 0x0A3EF5DB, 0xF1DA3FCD, 0x958C6F85, 0xDBF9C367, 0xDA45CDE5, 0x11C9E17D, 0x031800E4, 0x0E00F411, 0xDA74CD20, 0x6FCD2B18, 0x0E2618DA, 0x00E411FF, 0xC5DAA0CD, 0x03030B16, 0x283FFE0A,
	0xF7201569, 0x0006C9C1, 0x18DAA6CD, 0x11200E08, 0xA0CD00E4, 0x000D21DA, 0x23FF3609, 0xC9FF3623, 0x28DAB6CD, 0x200EC945, 0xCD00E411, 0x0EC5DAB6, 0xDB8ECC0B, 0xCDE5C9C1, 0xFDD5CA98, 0x1312007E,
	0x12017EFD, 0x0E41C513, 0xDB85CD08, 0xCD030E78, 0x0301DB90, 0xCAAFCD00, 0xD5E1D1C1, 0xD1DAEDCD, 0x4B420830, 0xFE1A1313, 0xAFC3C920, 0x97CD2BCD, 0x4FD83FDB, 0x3AFEC5E5, 0xA5CD0628, 0x37F738DB,
	0x3879E1C1, 0x30FE133E, 0x3AFE1F38, 0x30D61B30, 0xA5CD124F, 0x3830FEDB, 0x303AFE10, 0xC00DB70C, 0x10FEDAC6, 0xA5CD12D0, 0x51FE1BDB, 0x41FE0A30, 0x41D60638, 0xDBA5CD12, 0xEEDB9BCD, 0x97CDC03A,
	0x13D83FDB, 0xC82EFE13, 0x58CD080E, 0x2EEED8DB, 0xDB97CDC0, 0x2D30030E, 0x293820FE, 0x2147C5E5, 0x237EDBB2, 0xB80428B7, 0x7837F820, 0x1538E1C1, 0x2AFEF80D, 0x12DB8ECC, 0xDBA5CD13, 0x20FE0730,
	0x9BCDD620, 0x203EF5DB, 0xF1DB90CD, 0x3F3EC93F, 0x12C80D0C, 0xCDFA1813, 0xFED0DBA5, 0xCDC03720, 0xF738DBA5, 0xC8B778C9, 0xE6E70523, 0xCAA6CD7F, 0x3E3CC937, 0x3A3B2C2E, 0x5F5D5B3D, 0x29287C25,
	0x007F5C2F, 0x0218013E, 0xFDC50B3E, 0x0A18024E, 0x165F0AC5, 0x4F0A0B00, 0xEBCD0C3E, 0xE5C9C1CA, 0x2001C5D5, 0xC1B0ED00, 0xCBC9E1D1, 0x3D1DCB3C, 0xE5C9F920, 0xE152EDB7, 0x235ED5C9, 0xC9D1EB56
};

/* UPD */
static struct {
	struct {
		char debut[ 0x30 ];
		BYTE NbTracks, NbHeads;
		SHORT TrackSize;
		BYTE TrackSizeTable[ 0xCC ];
	} Infos;
	CPCEMUTrack * Tracks[ MAX_TRACKS ][ 2 ];
	BYTE * Data[ MAX_TRACKS ][ 2 ], FlagWrite;
	char NomFic[ 256 ];
	ULONG LongFic;
	BYTE Image;
} Dsk[ MAX_DSK ];

static BYTE DataDsk[ 2 ][ 1024 * 1024 ];

static int CmdInvalid( int, int, int );
static int CmdReadTrack( int, int, int );
static int CmdSpecify( int, int, int );
static int CmdSenseDriveStatus( int, int, int );
static int CmdWriteData( int, int, int );
static int CmdReadData( int, int, int );
static int CmdRecalibrate( int, int, int );
static int CmdSenseInterStatus( int, int, int );
static int CmdReadID( int, int, int );
static int CmdReadDelData( int, int, int );
static int CmdFormatTrack( int, int, int );
static int CmdSeek( int, int, int );
static int CmdWriteDelData( int, int, int );
static int CmdScanLow( int, int, int );
static int CmdScanLowEq( int, int, int );
static int CmdScanHighEq( int, int, int );

static pFctExec FctExec;

static int PosStack;

static pFctExec TabCmdUPD[ 0x20 ] = {
	CmdInvalid, CmdInvalid, CmdReadTrack, CmdSpecify, CmdSenseDriveStatus, CmdWriteData, CmdReadData, CmdRecalibrate, CmdSenseInterStatus,
	CmdWriteDelData, CmdReadID, CmdInvalid, CmdReadDelData, CmdFormatTrack, CmdInvalid, CmdSeek, CmdInvalid, CmdScanLow, CmdInvalid, CmdInvalid,
	CmdInvalid, CmdInvalid, CmdInvalid, CmdInvalid, CmdInvalid, CmdScanLowEq, CmdInvalid, CmdInvalid, CmdInvalid, CmdScanHighEq, CmdInvalid, CmdInvalid
};

static int Status, ST0, ST1, ST2, ST3;
static int UnitSel, HeadSel, SRT, HUT, HLT;
static int TpsReady, TpsNextByte, TpsOverRun, CptOverRun;
static int NewCylinder, NbSteps, IndexSecteur, PosData, TailleSect, Weak, Break, PosWeak;
static CPCEMUTrack * CurrTrackDatasDSK[ MAX_DSK ][ 2 ];

/* GENERAL */
static struct {
	BYTE Port, Ligne;
} TrKey[ 256 ] = {
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0x7F, 0x09 }, { 0xEF, 0x08 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFB, 0x02 }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xDF, 0x02 }, { 0x7F, 0x02 }, { 0xFD, 0x01 }, { 0xFF, 0x0F }, { 0xBF, 0x08 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFB, 0x08 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0x7F, 0x05 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFE, 0x01 }, { 0xFE, 0x00 }, { 0xFD, 0x00 },
	{ 0xFB, 0x00 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFE, 0x02 }, { 0xFF, 0x0F },
	{ 0xFE, 0x04 }, { 0xFE, 0x08 }, { 0xFD, 0x08 }, { 0xFD, 0x07 }, { 0xFE, 0x07 }, { 0xFD, 0x06 }, { 0xFE, 0x06 }, { 0xFD, 0x05 },
	{ 0xFE, 0x05 }, { 0xFD, 0x04 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xF7, 0x08 }, { 0xBF, 0x06 }, { 0xBF, 0x07 }, { 0xDF, 0x07 }, { 0xFB, 0x07 }, { 0xDF, 0x06 }, { 0xEF, 0x06 },
	{ 0xEF, 0x05 }, { 0xF7, 0x04 }, { 0xDF, 0x05 }, { 0xDF, 0x04 }, { 0xEF, 0x04 }, { 0xDF, 0x03 }, { 0xBF, 0x05 }, { 0xFB, 0x04 },
	{ 0xF7, 0x03 }, { 0xDF, 0x08 }, { 0xFB, 0x06 }, { 0xEF, 0x07 }, { 0xF7, 0x06 }, { 0xFB, 0x05 }, { 0x7F, 0x06 }, { 0x7F, 0x08 },
	{ 0x7F, 0x07 }, { 0xF7, 0x05 }, { 0xF7, 0x07 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFD, 0x01 }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0x7F, 0x01 }, { 0xDF, 0x01 }, { 0xBF, 0x01 }, { 0xDF, 0x00 }, { 0xEF, 0x02 }, { 0xEF, 0x01 }, { 0xEF, 0x00 }, { 0xFB, 0x01 },
	{ 0xF7, 0x01 }, { 0xF7, 0x00 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0x7F, 0x00 }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xBF, 0x02 }, { 0xFE, 0x03 }, { 0xBF, 0x04 }, { 0xFF, 0x0F }, { 0x7F, 0x04 }, { 0x7F, 0x03 },
	{ 0xEF, 0x03 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFD, 0x03 }, { 0xFD, 0x02 }, { 0xFB, 0x03 }, { 0xF7, 0x02 }, { 0xBF, 0x03 },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xF7, 0x02 }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F },
	{ 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }, { 0xFF, 0x0F }
};

static int TabPoints[ 4 ][ 256 ][ 8 ];

static struct {
	char Id[ 0x10 ];
	BYTE Version;
	Registre AF, BC, DE, HL, IR;
	BYTE IFF1, IFF2;
	Registre IX, IY, SP, PC;
	BYTE InterruptMode;
	Registre _AF, _BC, _DE, _HL;
	BYTE InkReg, InkData[ 17 ];
	BYTE VGARom, VGARam;
	BYTE CRTCIndex, CRTCReg[ 18 ];
	BYTE NumRom;
	BYTE PPI[ 4 ];
	BYTE PsgIndex, PsgData[ 16 ];
	USHORT MemSize;
	BYTE CpcType;
	BYTE last_interrupt;
	BYTE ScrModes[ 6 ];
	BYTE DrvAFilename[ 13 ], DrvBFilename[ 13 ], CartFilename[ 13 ];
	BYTE FddMotor, FddCurrTrack[ 4 ];
	BYTE PrinterData;
	BYTE EnvStep, EnvDir;
	BYTE CrtcType;
	ULONG Unused3;
	BYTE HCC, Unused4, VCC, VLC, VtAdj, TailleHBL, TailleVBL;
	USHORT CrtcFlag;
	BYTE SyncCount, CntHSync;
	BYTE IRQ;
	BYTE Unused5[ 0x4B ];
} Cpc;

static int finMain, Turbo;
static HDC DeviceContext, DcBitmap;
static int * MemBitmap;
static char NomDisc[ MAX_PATH ];

/* VGA */
static void SetMemCPC( void ) {
	static int AdjRam[ 4 ][ 8 ] = {
		0x00000, 0x00000, 0x10000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
		0x04000, 0x04000, 0x14000, 0x0C000, 0x10000, 0x14000, 0x18000, 0x1C000,
		0x08000, 0x08000, 0x18000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
		0x0C000, 0x1C000, 0x1C000, 0x1C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000
	};

	TabPOKE[ 0 ] = &MemCPC[ AdjRam[ 0 ][ Cpc.VGARam ] ];
	TabPEEK[ 0 ] = Cpc.VGARom & 0x04 ? &MemCPC[ AdjRam[ 0 ][ Cpc.VGARam ] ] : ( BYTE * )ROMINF;
	TabPOKE[ 1 ] = TabPEEK[ 1 ] = &MemCPC[ AdjRam[ 1 ][ Cpc.VGARam ] ];
	TabPOKE[ 2 ] = TabPEEK[ 2 ] = &MemCPC[ AdjRam[ 2 ][ Cpc.VGARam ] ];
	TabPOKE[ 3 ] = &MemCPC[ AdjRam[ 3 ][ Cpc.VGARam ] ];
	TabPEEK[ 3 ] = Cpc.VGARom & 0x08 ? &MemCPC[ AdjRam[ 3 ][ Cpc.VGARam ] ] : Cpc.NumRom == 7 ? ( BYTE * )ROMDISC : ( BYTE * )ROMSUP;
}

static void VGASyncColor( void ) {
	static unsigned int RgbCPCColor[ 256 ] = {
		0x6E7D6B, 0x6E7B6D, 0x00F36B, 0xF3F36D, 0x00026B, 0xF00268, 0x007868, 0xF37D6B,
		0xF30268, 0xF3F36B, 0xF3F30D, 0xFFF3F9, 0xF30506, 0xF302F4, 0xF37D0D, 0xFA80F9,
		0x000268, 0x02F36B, 0x02F001, 0x0FF3F2, 0x000201, 0x0C02F4, 0x027801, 0x0C7BF4,
		0x690268, 0x71F36B, 0x71F504, 0x71F3F4, 0x6C0201, 0x6C02F2, 0x6E7B01, 0x6E7BF6
	};

	if ( MemoColor < 0x20 ) {
		TabCoul[ Cpc.InkReg ] = RgbCPCColor[ MemoColor ];
		Cpc.InkData[ Cpc.InkReg ] = MemoColor;
		MemoColor = 0xFF;
	}
}

static void WriteVGA( int val ) {
	int newVal = val & 0x1F;
	switch( val >> 6 ) {
	case 0 :
		Cpc.InkReg = min( newVal, 16 );
		break;

	case 1 :
		MemoColor = newVal;
		break;

	case 2 :
		MemoMode = val & 3;
		Cpc.VGARom = val;
		SetMemCPC();
		if ( val & 16 )
			Cpc.CntHSync = Cpc.IRQ = 0;
		break;

	case 3 :
		Cpc.VGARam = val & 7;
		SetMemCPC();
		break;
	}
}

/* PSG */
static void Write8912( int Reg, int Val ) {
	Cpc.PsgData[ Reg ] = Val;
	switch( Reg ) {
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
		TabTone[ Reg >> 1 ].Step = max( 1, Cpc.PsgData[ Reg & 6 ] + ( ( Cpc.PsgData[ 1 + ( Reg & 6 ) ] & 0x0F ) << 8 ) );
		break;

	case 6:
		stepN = max( 1, Cpc.PsgData[ 6 ] & 0x1F );
		break;

	case 11:
	case 12:
		stepE = max( 2, ( Cpc.PsgData[ 11 ] + ( Cpc.PsgData[ 12 ] << 8 ) ) << 1 );
		break;

	case 13:
		EnvStep = 0x0F;
		Holding = 0;
		Attack = Val & 0x04 ? 0x0F : 0x00;
		Hold = ( Val & 0x08 >> 3 ) | Val & 0x01;
		Alternate = Val & 0x08 ? Val & 0x02 ? 0x0F : 0x00 : Attack;
		break;
	}
}

static DWORD WINAPI ThreadFunc( void * Param ) {
	static int VolTable[ 16 ] = { 0, 38, 115, 192, 346, 462, 693, 1118, 1349, 2197, 3045, 4048, 5359, 6786, 8791, 10911 };
	MSG Msg;

	while( ! Exit && GetMessage( &Msg, 0, 0, 0 ) && Msg.message != MM_WOM_CLOSE ) {
		if ( Msg.message == MM_WOM_DONE ) {
			int i, c;
			WAVEHDR * h = ( WAVEHDR * )Msg.lParam;
			BYTE * Buf = ( BYTE * )h->lpData;
			for ( i = 0; i < SND_BUFSIZE; i++ ) {
				int vol = 0x8000;
				if ( ++CntNoise >= stepN ) {
					CntNoise = 0;
					OutNoise ^= ( ( Rng + 1 ) & 2 ) >> 1;
					Rng = ( Rng ^ ( Rng & 1 ? 0x24000 : 0 ) ) >> 1;
				}
				if ( ! Holding ) {
					if ( ++CntEnv >= stepE ) {
						CntEnv = 0;
						if ( --EnvStep < 0 ) {
							Attack ^= Alternate;
							EnvStep = Hold ? 0x00 : 0x0F;
							Holding = Hold;
						}
					}
				}
				for ( c = 0; c < 3; c++ ) {
					if ( ++TabTone[ c ].Cnt >= TabTone[ c ].Step ) {
						TabTone[ c ].Cnt = 0;
						TabTone[ c ].Out ^= 1;
					}
					if ( ( TabTone[ c ].Out | ( ( Cpc.PsgData[ 7 ] >> c ) & 1 ) ) & ( OutNoise | ( ( Cpc.PsgData[ 7 ] >> ( c + 3 ) ) & 1 ) ) )
						vol += VolTable[ ( Cpc.PsgData[ 8 + c ] & 16 ) ? ( EnvStep ^ Attack ) : Cpc.PsgData[ 8 + c ] & 0x0F ];
				}
				Buf[ i ] = IsSound ? vol >> 8 : 0x80;
			}
			if ( h->dwFlags & WHDR_PREPARED )
				waveOutUnprepareHeader( ( HWAVEOUT )Msg.wParam, h, sizeof( WAVEHDR ) );

			h->dwFlags = 0;
			waveOutPrepareHeader( ( HWAVEOUT )Msg.wParam, h, sizeof( WAVEHDR ) );
			waveOutWrite( ( HWAVEOUT )Msg.wParam, h, sizeof( WAVEHDR ) );
		}
	}
	return 0;
}

/* PPI */
static void WritePPI( int adr, int val ) {
	switch( ( adr >> 8 ) & 3 ) {
	case 0 :
		Cpc.PPI[ 0 ] = val;
		Cpc.PsgIndex = modePSG == 3 ? val & 0x0F : Cpc.PsgIndex;
		if ( modePSG == 2 )
			Write8912( Cpc.PsgIndex, val );
		break;

	case 2 :
		Cpc.PPI[ 2 ] = val;
		Cpc.PsgData[ 14 ] = clav[ val & 0x0F ];
		modePSG = val >> 6;
		Cpc.PsgIndex = modePSG == 3 ? Cpc.PPI[ 0 ] & 0x0F : Cpc.PsgIndex;
		if ( modePSG == 2 )
			Write8912( Cpc.PsgIndex, Cpc.PPI[ 0 ] );
		break;

	case 3 :
		Cpc.PPI[ 3 ] = val;
		if ( val & 0x80 ) {
			Cpc.PPI[ 0 ] = 0;
			MaskPortC = ( val & 0x09 ) == 0x09 ? 0x00 : val & 0x08 ? 0x0F : val & 0x01 ? 0xF0 : 0xFF;
		}
		else
			Cpc.PPI[ 2 ] ^= ( - ( val & 1 ) ^ Cpc.PPI[ 2 ] ) & ( 1 << ( ( val >> 1 ) & 0x07 ) );
		break;
	}
}

/* CRTC */
static void WriteCRTC( int adr, int val ) {
	adr &= 0x300;
	if ( ! adr )
		Cpc.CRTCIndex = val & 0x1F;
	else
		if ( adr == 0x100 ) {
			switch( Cpc.CRTCIndex ) {
			case 4 :
			case 6 :
			case 7 :
				val &= 0x7F;
				break;

			case 5 :
			case 9 :
				val &= 0x1F;
				break;

			case 12 :
				MaHi = ( ( val << 8 ) & 0x0F00 ) | ( ( val << 9 ) & 0x6000 );
				break;
			}
			Cpc.CRTCReg[ Cpc.CRTCIndex ] = val;
		}
}

/* UPD */
static void WriteUPD( int port, int val ) {
	port &= 0x101;
	if ( port == 0x101 ) {
		if ( ! ( Status & STATUS_DIO ) ) {
			if ( ! FctExec ) {
				FctExec = TabCmdUPD[ val & 0x1F ];
				Status |= STATUS_CB;
				PosStack = 0;
			}
			FctExec( val, SENS_WR, 0 );
		}
	}
	else
		if ( port <= 1 ) {
			TpsReady = Cpc.FddMotor ? TpsReady : TPS_MOTOR_ON;
			Cpc.FddMotor = val;
			if ( ! Cpc.FddMotor ) {
				ST3 &= ~ST3_RY;
				Break = 1;
			}
		}
}

/* GENERAL */
static int ReadPort( int port ) {
	if ( ! ( port & 0x0800 ) ) {
		switch( ( port >> 8 ) & 3 ) {
		case 0 :
			return modePSG == 1 ? Cpc.PsgData[ Cpc.PsgIndex ] : 0xFF;

		case 1 :
			return 0x1E | VSync ;

		case 2 :
			return Cpc.PPI[ 2 ] & MaskPortC;
		}
	}

	if ( ! ( port & 0x4000 ) ) {
		port &= 0x300;
		if ( port == 0x300 )
			return Cpc.CRTCIndex >= 10 ? Cpc.CRTCReg[ Cpc.CRTCIndex ] : 0;

		return port == 0x200 ? VSync << 5 : 0xFF;
	}

	if ( ! ( port & 0x0480 ) && ( port & 0x101 ) ) {
		if ( ! ( port & 1 ) )
			return Status;

		Status &= ~STATUS_RQM;
		TpsNextByte += TPS_DATA_READ;
		if ( Status & STATUS_DIO )
			return FctExec( 0, SENS_RD, 0 );
	}
	return 0xFF;
}

static void WritePort( int port, int val ) {
	if ( ( port & 0xC000 ) == 0x4000 )
		WriteVGA( val );

	if ( ! ( port & 0x4000 ) )
		WriteCRTC( port, val );

	if ( ! ( port & 0x2000 ) ) {
		Cpc.NumRom = val;
		SetMemCPC();
	}
	if ( ! ( port & 0x0800 ) )
		WritePPI( port, val );

	if ( ! ( port & 0x0480 ) )
		WriteUPD( port, val );
}

/* Z80 */
static BYTE Fetch( void ) {
	return TabPEEK[ RegPC >> 14 ][ RegPC++ & 0x3FFF ];
}

static BYTE PEEK8( USHORT adr ) {
	return TabPEEK[ adr >> 14 ][ adr & 0x3FFF ];
}

static void POKE8( USHORT adr, BYTE val ) {
	TabPOKE[ adr >> 14 ][ adr & 0x3FFF ] = val;
}

static int PEEK16( USHORT adr ) {
	int q = adr >> 14;
	adr &= 0x3FFF;
	return TabPEEK[ q ][ adr ] | ( TabPEEK[ q ][ adr + 1 ] << 8 );
}

static void POKE16( USHORT adr, int val ) {
	int q = adr >> 14;
	TabPOKE[ q ][ adr &= 0x3FFF ] = val;
	TabPOKE[ q ][ ++adr ] = val >> 8;
}

/* 8 Bits Load Group */
static int LD_R8( void ) {
	* PtrRegs[ ( LastInstr >> 3 ) & 0x07 ] = LastInstr & 0x40 ? * PtrRegs[ LastInstr & 0x07 ] : Fetch();
	return LastInstr & 0x40 ? 1 : 2;
}

static int LD_R_INDIRECT_HL( void ) {
	* Regs[ ( LastInstr >> 3 ) & 0x07 ] = ( BYTE )PEEK8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ] );
	return PtrRegs == Regs ? 2 : 4;
}

static int LD_INDIRECT_HL_R( void ) {
	POKE8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ], * Regs[ LastInstr & 0x07 ] );
	return PtrRegs == Regs ? 2 : 4;
}

static int LD_INDIRECT_HL_N( void ) {
	POKE8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ], Fetch() );
	return PtrRegs == Regs ? 3 : 5;
}

static int LD_A_INDIRECT_BCDE( void ) {
	RegA = ( BYTE )PEEK8( memPtr = LastInstr & 0x10 ? RegDE : RegBC );
	memPtr++;
	return 2;
}

static int LD_A_INDIRECT_NN( void ) {
	memPtr = PEEK16( RegPC );
	RegA = ( BYTE )PEEK8( memPtr++ );
	RegPC += 2;
	return 4;
}

static int LD_INDIRECT_BCDE_A( void ) {
	POKE8( memPtr = LastInstr & 0x10 ? RegDE : RegBC, RegA );
	memPtr = ( ++memPtr & 0xFF ) + ( RegA << 8 );
	return 2;
}

static int LD_INDIRECT_NN_A( void ) {
	POKE8( memPtr = PEEK16( RegPC ), RegA );
	memPtr = ( ++memPtr & 0xFF ) | ( RegA << 8 );
	RegPC += 2;
	return 4;
}

static int LD_A_I_LD_A_R( void ) {
	int a = ( Cpc.IR.Word >> ( - LastInstr & 0x08 ) ) & 0xFF;
	RegAF = ( a << 8 ) | TabSZYX[ a ] | Cpc.IFF2 | ( FLAGS & FLAG_C );
	SupIrqWaitState = 1;
	return 3;
}

static int LD_I_A_LD_R_A( void ) {
	Cpc.IR.Word = ( RegA << ( - LastInstr & 0x08 ) ) | ( Cpc.IR.Word & ( 0xFF << ( LastInstr & 0x08 ) ) );
	SupIrqWaitState = 1;
	return 3;
}

/* 16 Bits Load Group */
static int LD_RR_NN( void ) {
	*( USHORT * )PtrRegs[ ( LastInstr >> 4 ) + 8 ] = PEEK16( RegPC );
	RegPC += 2;
	return 3;
}

static int LD_RR_INDIRECT_NN( void ) {
	memPtr = PEEK16( RegPC );
	*( USHORT * )PtrRegs[ LastInstr & 0x40 ? ( LastInstr >> 4 ) + 4 : 6 ] = PEEK16( memPtr++ );
	RegPC += 2;
	return LastInstr & 0x40 ? 6 : 5;
}

static int LD_INDIRECT_NN_RR( void ) {
	memPtr = PEEK16( RegPC );
	POKE16( memPtr++, *( USHORT * )PtrRegs[ LastInstr & 0x40 ? ( LastInstr >> 4 ) + 4 : 6 ] );
	RegPC += 2;
	return LastInstr & 0x40 ? 6 : 5;
}

static int LD_SP_HL( void ) {
	RegSP = *( USHORT * )PtrRegs[ 6 ];
	return 2;
}

static int PUSH_SS( void ) {
	RegSP -= 2;
	POKE16( RegSP, *( USHORT * )PtrRegs[ LastInstr >> 4 ] );
	return 4;
}

static int POP_SS( void ) {
	*( USHORT * )PtrRegs[ LastInstr >> 4 ] = PEEK16( RegSP );
	RegSP += 2;
	SupIrqWaitState = 1;
	return 3;
}

/* Exchange, Block Transfer and Search Groups */
static int EX_DE_HL( void ) {
	USHORT t = RegDE;
	RegDE = RegHL;
	RegHL = t;
	return 1;
}

static int EX_AF_AF_PRIME( void ) {
	USHORT t = RegAF;
	RegAF = Reg_AF;
	Reg_AF = t;
	return 1;
}

static int EXX( void ) {
	static int tmp[ 4 ];

	memcpy( tmp, &Cpc.BC, 6 );
	memcpy( &Cpc.BC, &Cpc._BC, 6 );
	memcpy( &Cpc._BC, &tmp, 6 );
	return 1;
}

static int EX_INDIRECT_SP_HL( void ) {
	memPtr = PEEK16( RegSP );
	POKE16( RegSP, *( USHORT * )PtrRegs[ 6 ] );
	*( USHORT * )PtrRegs[ 6 ] = memPtr;
	SupIrqWaitState = 2;
	return 6;
}

static int LDI_LDD( void ) {
	int n;
	POKE8( RegDE, n = PEEK8( RegHL ) );
	n += RegA;
	FLAGS = ( FLAGS & FLAGS_SZC ) | ( --RegBC ? FLAG_V : 0 ) | ( n & FLAG_3 ) | ( ( n << 4 ) & FLAG_5 );
	RegDE += LastInstr & 0x08 ? -1 : 1;
	RegHL += LastInstr & 0x08 ? -1 : 1;
	SupIrqWaitState = 1;
	return 5;
}

static int LDIR_LDDR( void ) {
	LDI_LDD();
	if ( ! RegBC )
		return 5;

	memPtr = --RegPC;
	RegPC--;
	return 6;
}

static int DO_CPI_CPD( void ) {
	int n = PEEK8( RegHL ), z = RegA - n, f = ( RegA ^ n ^ z ) & FLAG_H;
	n = z - ( f >> 4 );
	FLAGS = f | ( ( n << 4 ) & FLAG_5 ) | ( n & FLAG_3 ) | ( TabSZYX[ z & 0xFF ] & FLAGS_SZ ) | ( --RegBC ? FLAG_V : 0 ) | FLAG_N | ( FLAGS & FLAG_C );
	RegHL += LastInstr & 0x08 ? -1 : 1;
	memPtr += LastInstr & 0x08 ? -1 : 1;
	return z;
}

static int CPI_CPD( void ) {
	DO_CPI_CPD();
	return 4;
}

static int CPIR_CPDR( void ) {
	if ( DO_CPI_CPD() && RegBC ) {
		memPtr = --RegPC;
		RegPC--;
		SupIrqWaitState = 1;
		return 6;
	}
	return 4;
}

/* 8 Bit Arithmetic and Logical Group */
static int ADD_R8( void ) {
	int x = LastInstr & 0x40 ? Fetch() : ( LastInstr & 0x07 ) == INDIRECT_HL ? PEEK8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ] ) : * PtrRegs[ LastInstr & 0x07 ];
	int z = RegA + x + ( FLAGS & FLAG_C & ( LastInstr & 0x08 ) == 0x08 ), c = RegA ^ x ^ z;
	FLAGS = ( c & FLAG_H ) | TabSZYX[ RegA = z ] | TabV[ c >> 7 ] | ( z >> 8 );
	return LastInstr & 0x40 ? 2 : ( LastInstr & 0x07 ) == INDIRECT_HL ? PtrRegs == Regs ? 2 : 4 : 1;
}

static int SUB_R8( void ) {
	int x = LastInstr & 0x40 ? Fetch() : ( LastInstr & 0x07 ) == INDIRECT_HL ? PEEK8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ] ) : * PtrRegs[ LastInstr & 0x07 ];
	int z = RegA - x - ( FLAGS & FLAG_C & ( LastInstr & 0x08 ) == 0x08 ), c = ( RegA ^ x ^ z ) & 0x190;
	FLAGS = FLAG_N | ( c & FLAG_H ) | TabSZYX[ RegA = z ] | TabV[ c >> 7 ] | ( c >> 8 );
	return LastInstr & 0x40 ? 2 : ( LastInstr & 0x07 ) == INDIRECT_HL ? PtrRegs == Regs ? 2 : 4 : 1;
}

static int LOG_R8( void ) {
	int x = LastInstr & 0x40 ? Fetch() : ( LastInstr & 0x07 ) == INDIRECT_HL ? PEEK8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ] ) : * PtrRegs[ LastInstr & 0x07 ];
	int k = LastInstr & 0x18;
	FLAGS = k == 0 ? TabSZYXP[ RegA &= x ] | FLAG_H : k == 0x08 ? TabSZYXP[ RegA ^= x ] : TabSZYXP[ RegA |= x ];
	return LastInstr & 0x40 ? 2 : ( LastInstr & 0x07 ) == INDIRECT_HL ? PtrRegs == Regs ? 2 : 4 : 1;
}

static int CP_R8( void ) {
	int x = LastInstr & 0x40 ? Fetch() : ( LastInstr & 0x07 ) == INDIRECT_HL ? PEEK8( PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ] ) : * PtrRegs[ LastInstr & 0x07 ];
	int z = RegA - x, c = ( RegA ^ x ^ z ) & 0x190;
	FLAGS = FLAG_N | ( c & FLAG_H ) | ( TabSZYX[ z & 0xFF ] & FLAGS_SZ ) | ( x & FLAGS_53 ) | TabV[ c >> 7 ] | ( c >> 8 );
	return LastInstr & 0x40 ? 2 : ( LastInstr & 0x07 ) == INDIRECT_HL ? PtrRegs == Regs ? 2 : 4 : 1;
}

static int INC_R( void ) {
	BYTE z = ++* PtrRegs[ LastInstr >> 3 ];
	FLAGS = ( FLAGS & FLAG_C ) | ( z & 0x0F ? 0 : FLAG_H ) | TabSZYX[ z ] | ( z == 0x80 ? FLAG_V : 0 );
	return 1;
}

static int INC_INDIRECT_HL( void ) {
	int z, d = PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ];
	POKE8( d, z = PEEK8( d ) + 1 );
	FLAGS = ( FLAGS & FLAG_C ) | ( z & 0x0F ? 0 : FLAG_H ) | TabSZYX[ z & 0xFF ] | ( z == 0x80 ? FLAG_V : 0 );
	return PtrRegs == Regs ? 3 : 5;
}

static int DEC_R( void ) {
	BYTE z = --* PtrRegs[ LastInstr >> 3 ];
	FLAGS = FLAG_N | ( FLAGS & FLAG_C ) | ( ( z & 0x0F ) == 0x0F ? FLAG_H : 0 ) | TabSZYX[ z ] | ( z == 0x7F ? FLAG_V : 0 );
	return 1;
}

static int DEC_INDIRECT_HL( void ) {
	int z, d = PtrRegs == Regs ? RegHL : ( char )Fetch() + *( USHORT * )PtrRegs[ 6 ];
	POKE8( d, z = PEEK8( d ) - 1 );
	FLAGS = FLAG_N | ( FLAGS & FLAG_C ) | ( ( z & 0x0F ) == 0x0F ? FLAG_H : 0 ) | TabSZYX[ z & 0xFF ] | ( z == 0x7F ? FLAG_V : 0 );
	return PtrRegs == Regs ? 3 : 5;
}

/* 16 Bit Arithmetic Group */
static int ADD_HL_RR( void ) {
	int z, c, y = *( USHORT * )PtrRegs[ ( LastInstr >> 4 ) + 8 ];
	memPtr = *( USHORT * )PtrRegs[ 6 ];
	z = memPtr + y;
	c = memPtr++ ^ y ^ z;
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( ( z >> 8 ) & FLAGS_53 ) | ( ( c >> 8 ) & FLAG_H ) | ( c >> 16 );
	*( USHORT * )PtrRegs[ 6 ] = z;
	return 3;
}

static int ADC_HL_RR( void ) {
	int y = *( USHORT * )PtrRegs[ ( LastInstr >> 4 ) + 4 ], z = RegHL + y + ( FLAGS & FLAG_C ), c = RegHL ^ y ^ z;
	FLAGS = ( z & 0xffff ? ( z >> 8 ) & FLAGS_S53 : FLAG_Z ) | ( ( c >> 8 ) & FLAG_H ) | TabV[ c >> 15 ] | ( z >> 16 );
	memPtr = ++RegHL;
	RegHL = z;
	return 4;
}

static int SBC_HL_RR( void ) {
	int y = *( USHORT * )PtrRegs[ ( LastInstr >> 4 ) + 4 ], z = RegHL - y - ( FLAGS & FLAG_C ), c = ( RegHL ^ y ^ z ) & 0x19000;
	FLAGS = FLAG_N | ( z & 0xffff ? ( z >> 8 ) & FLAGS_S53 : FLAG_Z ) | ( ( c >> 8 ) & FLAG_H ) | TabV[ c >> 15 ] | ( c >> 16 );
	memPtr = ++RegHL;
	RegHL = z;
	return 4;
}

static int INC_RR( void ) {
	( *( USHORT * )PtrRegs[ ( LastInstr >> 4 ) + 8 ] )++;
	SupIrqWaitState = 1;
	return 2;
}

static int DEC_RR( void ) {
	( *( USHORT * )PtrRegs[ ( LastInstr >> 4 ) + 8 ] )--;
	SupIrqWaitState = 1;
	return 2;
}

/* General Purpose Arithmetic and CPU Control Groups */
static int DAA( void ) {
	int a = RegA, f = 0, d = ( RegA & 0x0F ) > 0x09 | ( FLAGS & FLAG_H ) ? 6 : 0;
	if ( a > 0x99 | ( FLAGS & FLAG_C ) ) {
		f = FLAG_C;
		d |= 0x60;
	}
	RegA += FLAGS & FLAG_N ? -d : d;
	FLAGS = TabSZYXP[ RegA ] | ( ( RegA ^ a ) & FLAG_H ) | ( FLAGS & FLAG_N ) | f;
	return 1;
}

static int CPL( void ) {
	RegA ^= 0xFF;
	FLAGS = ( FLAGS & ( FLAGS_SZV | FLAG_C ) ) | ( RegA & FLAGS_53 ) | FLAG_H | FLAG_N;
	return 1;
}

static int NEG( void ) {
	int c = ( RegA ^ -RegA ) & 0x190;
	FLAGS = FLAG_N | ( c & FLAG_H ) | TabSZYX[ RegA = -RegA ] | TabV[ c >> 7 ] | ( c >> 8 );
	return 2;
}

static int CCF( void ) {
	int c = FLAGS & FLAG_C;
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( c << 4 ) | ( RegA & FLAGS_53 ) | ( c ^ FLAG_C );
	return 1;
}

static int SCF( void ) {
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( RegA & FLAGS_53 ) | FLAG_C;
	return 1;
}

static int NOP( void ) {
	return 1;
}

static int HALT( void ) {
	RegPC--;
	return 1;
}

static int DI_EI( void ) {
	Cpc.IFF1 = Cpc.IFF2 = LastInstr & 0x08 ? FLAG_V : 0;
	return 1;
}

static int IM_N( void ) {
	Cpc.InterruptMode = ( LastInstr & 0x18 ) == 0x18 ? 2 : ( LastInstr & 0x18 ) == 0x10;
	SupIrqWaitState = 1;
	return 2;
}

/* Rotate and Shift Group */
static int RLCA( void ) {
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( ( RegA = ( RegA << 1 ) | ( RegA >> 7 ) ) & ( FLAGS_53 | FLAG_C ) );
	return 1;
}

static int RLA( void ) {
	int a = RegA << 1;
	RegA = a | ( FLAGS & FLAG_C );
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( a >> 8 ) | ( RegA & FLAGS_53 );
	return 1;
}

static int RRCA( void ) {
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( RegA & FLAG_C );
	FLAGS |= ( ( RegA = ( RegA >> 1 ) | ( FLAGS << 7 ) ) & FLAGS_53 );
	return 1;
}

static int RRA( void ) {
	int c = RegA & FLAG_C;
	FLAGS = ( FLAGS & FLAGS_SZV ) | ( ( RegA = ( RegA >> 1 ) | ( ( FLAGS & FLAG_C ) << 7 ) ) & FLAGS_53 ) | c;
	return 1;
}

static int RLC_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d ), c = x >> 7;
	FLAGS = TabSZYXP[ x = ( ( x << 1 ) & 0xFF ) | c ] | c;
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int RLC_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ] << 1, c = x >> 8;
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] = x | c ] | c;
		return 2;
	}
	return RLC_INDIRECT_HL();
}

static int RL_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d );
	FLAGS = TabSZYXP[ ( x = ( x << 1 ) | ( FLAGS & FLAG_C ) ) & 0xFF ] | ( x >> 8 );
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int RL_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ] << 1;
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] = x | ( FLAGS & FLAG_C ) ] | ( x >> 8 );
		return 2;
	}
	return RL_INDIRECT_HL();
}

static int RRC_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d ), c = x & 1;
	FLAGS = TabSZYXP[ x = ( x >> 1 ) | ( c << 7 ) ] | c;
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int RRC_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ], c = x & 1;
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] = ( x >> 1 ) | ( c << 7 ) ] | c;
		return 2;
	}
	return RRC_INDIRECT_HL();
}

static int RR_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d ), c = x & 1;
	FLAGS = TabSZYXP[ x = ( x >> 1 ) | ( ( FLAGS & FLAG_C ) << 7 ) ] | c;
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int RR_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ];
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] = ( x >> 1 ) | ( ( FLAGS & FLAG_C ) << 7 ) ] | ( x & 1 );
		return 2;
	}
	return RR_INDIRECT_HL();
}

static int SLA_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d );
	FLAGS = TabSZYXP[ ( x <<= 1 ) & 0xFF ] | ( x >> 8 );
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int SLA_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ] << 1;
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] = x ] | ( x >> 8 );
		return 2;
	}
	return SLA_INDIRECT_HL();
}

static int SLL_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d );
	FLAGS = TabSZYXP[ ( x = ( x << 1 ) | 0x01 ) & 0xFF ] | ( x >> 8 );
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int SLL_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ] << 1;
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] = x | 0x01 ] | ( x >> 8 );
		return 2;
	}
	return SLL_INDIRECT_HL();
}

static int SRA_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d ), c = x & 1;
	FLAGS = c | TabSZYXP[ x = ( ( ( char )x ) >> 1 ) & 0xFF ];
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int SRA_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ];
		FLAGS = ( x & 1 ) | TabSZYXP[ * Regs[ LastInstr & 0x07 ]  = ( ( ( char )x ) >> 1 ) ];
		return 2;
	}
	return SRA_INDIRECT_HL();
}

static int SRL_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], x = PEEK8( d ), c = x & 1;
	FLAGS = TabSZYXP[ x >>= 1 ] | c;
	POKE8( d, x );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int SRL_R( void ) {
	if ( PtrRegs == Regs ) {
		int x = * Regs[ LastInstr & 0x07 ];
		FLAGS = TabSZYXP[ * Regs[ LastInstr & 0x07 ] >>= 1 ] | ( x & 1 );
		return 2;
	}
	return SRL_INDIRECT_HL();
}

static int RLD_RRD( void ) {
	int x = PEEK8( memPtr = RegHL ), y;
	POKE8( memPtr++, y = ( ( RegA & 0xf0 ) << 8 ) | ( LastInstr & 0x08 ? ( x << 4 ) | ( RegA & 0x0F ) : ( ( x & 0x0F ) << 8 ) | ( ( RegA & 0x0F ) << 4 ) | ( x >> 4 ) ) );
	FLAGS = TabSZYXP[ RegA = y >> 8 ] | ( FLAGS & FLAG_C );
	return 5;
}

/* Bit Manipulation Group */
static int BIT_B_INDIRECT_HL( void ) {
	int d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ], y = PtrRegs == Regs ? memPtr >> 8 : d, x = PEEK8( d ) & ( 1 << ( ( LastInstr >> 3 ) & 0x07 ) );
	FLAGS = ( x ? 0 : FLAG_Z | FLAG_V ) | ( x & FLAG_S ) | ( y & FLAGS_53 ) | FLAG_H | ( FLAGS & FLAG_C );
	RegPC += PtrRegs == Regs ? 0 : 2;
	return PtrRegs == Regs ? 3 : 5;
}

static int BIT_B_R( void ) {
	int x = ( PtrRegs == Regs ? * PtrRegs[ LastInstr & 0x07 ] : PEEK8( ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ] ) ) & ( 1 << ( ( LastInstr >> 3 ) & 0x07 ) );
	FLAGS = ( x ? 0 : FLAG_Z | FLAG_V ) | ( x & FLAG_S ) | ( * PtrRegs[ LastInstr & 0x07 ] & FLAGS_53 ) | FLAG_H | ( FLAGS & FLAG_C );
	RegPC += PtrRegs == Regs ? 0 : 2;
	return PtrRegs == Regs ? 2 : 5;
}

static int SET_B_INDIRECT_HL( void ) {
	int x, d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ];
	POKE8( d, x = PEEK8( d ) | 1 << ( ( LastInstr >> 3 ) & 0x07 ) );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int SET_B_R( void ) {
	if ( PtrRegs != Regs ) 
		return SET_B_INDIRECT_HL();

	* Regs[ LastInstr & 0x07 ] |= 1 << ( ( LastInstr >> 3 ) & 0x07 );
	return 2;
}

static int RES_B_INDIRECT_HL( void ) {
	int x, d = PtrRegs == Regs ? RegHL : ( char )PEEK8( RegPC ) + *( USHORT * )PtrRegs[ 6 ];
	POKE8( d, x = PEEK8( d ) & ~( 1 << ( ( LastInstr >> 3 ) & 0x07 ) ) );
	if ( PtrRegs == Regs )
		return 4;

	RegPC += 2;
	if ( ( LastInstr & 0x07 ) != INDIRECT_HL )
		* Regs[ LastInstr & 0x07 ] = x;

	return 6;
}

static int RES_B_R( void ) {
	if ( PtrRegs != Regs )
		return RES_B_INDIRECT_HL();

	* Regs[ LastInstr & 0x07 ] &= ~( 1 << ( ( LastInstr >> 3 ) & 0x07 ) );
	return 2;
}

/* Input and Output Group*/
static int IN_A_N( void ) {
	memPtr = ( RegA << 8 ) | Fetch();
	RegA = ReadPort( memPtr++ );
	return 3;
}

static int IN_R_C( void ) {
	int x = ReadPort( memPtr = RegBC );
	if ( ( LastInstr & 0x38 ) != INDIRECT_HL3 )
		* PtrRegs[ ( LastInstr >> 3 ) & 0x07 ] = x;

	memPtr++;
	FLAGS = TabSZYXP[ x ] | ( FLAGS & FLAG_C );
	return 4;
}

static int INI_IND( void ) {
	int x = ReadPort( memPtr = RegBC );
	POKE8( RegHL, x );
	memPtr += LastInstr & 0x08 ? - 1 : 1;
	RegHL += LastInstr &0x08 ? -1 : 1;
	FLAGS = TabSZYX[ --RegB & 0xFF ] | ( x >> 6 );
	x += memPtr & 0xFF;
	FLAGS |= ( x & 0x0100 ? FLAGS_HC : 0 ) | ( TabSZYXP[ ( x & 0x07 ) ^ RegB ] & FLAG_V );
	return 5;
}

static int INIR_INDR( void ) {
	INI_IND();
	if ( ! RegB )
		return 5;

	RegPC -= 2;
	return 6;
}

static int OUT_N_A( void ) {
	memPtr = ( RegA << 8 ) | Fetch();
	WritePort( memPtr++, RegA );
	return 3;
}

static int OUT_C_R( void ) {
	memPtr = RegBC;
	WritePort( memPtr++, ( LastInstr & 0x38 ) == INDIRECT_HL3 ? 0 : * PtrRegs[ ( LastInstr >> 3 ) & 0x07 ] );
	return 4;
}

static int OUTI_OUTD( void ) {
	FLAGS = FLAGS & ~FLAG_H & ~FLAG_S | ( --RegB ? 0 : FLAG_Z );
	WritePort( memPtr = RegBC, PEEK8( RegHL ) );
	RegHL += LastInstr & 0x08 ? -1 : 1;
	memPtr += LastInstr & 0x08 ? - 1 : 1;
	return 5;
}

static int OTIR_OTDR( void ) {
	OUTI_OUTD();
	if ( ! RegB )
		return 5;

	RegPC -= 2;
	return 6;
}

/* Jump Group */
static int CC( int mask ) {
	static int TabCC[ 16 ] = { FLAG_Z, FLAG_Z, 0, FLAG_Z, FLAG_C, FLAG_C, 0, FLAG_C, FLAG_V, FLAG_V, 0, FLAG_V, FLAG_S, FLAG_S, 0, FLAG_S };

	return ( FLAGS ^ TabCC[ ( LastInstr >> 2 ) & mask ] ) & TabCC[ 1 + ( ( LastInstr >> 2 ) & mask ) ];
}

static int JR_E( void ) {
	RegPC += ( char )PEEK8( RegPC );
	memPtr = ++RegPC;
	return 3;
}

static int JR_DD_E( void ) {
	if ( CC( 0x06 ) )
		return JR_E();

	RegPC++;
	return 2;
}

static int JP_NN( void ) {
	memPtr = RegPC = PEEK16( RegPC );
	return 3;
}

static int JP_CC_NN( void ) {
	if ( CC( 0x0E ) )
		return JP_NN();

	RegPC += 2;
	return 3;
}

static int JP_HL( void ) {
	RegPC = *( USHORT * )PtrRegs[ 6 ];
	return 1;
}

static int DJNZ_E( void ) {
	if ( --RegB )
		return 1 + JR_E();

	RegPC++;
	return 3;
}

/* Call and Return Group */
static int CALL_NN( void ) {
	RegSP -= 2;
	POKE16( RegSP, RegPC + 2 );
	memPtr = RegPC = PEEK16( RegPC );
	return 5;
}

static int CALL_CC_NN( void ) {
	if ( CC( 0x0E ) )
		return CALL_NN();

	RegPC += 2;
	return 3;
}

static int RET( void ) {
	memPtr = RegPC = PEEK16( RegSP );
	RegSP += 2;
	return 3;
}

static int RET_CC( void ) {
	if ( CC( 0x0E ) ) 
		return 1 + RET();

	SupIrqWaitState = 2;
	return 2;
}

static int RETI_RETN( void ) {
	Cpc.IFF1 = Cpc.IFF2;
	return 1 + RET();
}

static int RST_P( void ) {
	RegSP -= 2;
	POKE16( RegSP, RegPC );
	memPtr = RegPC = LastInstr & 0x38;
	return 4;
}

/* Prefixed Group */
static int ED_XX( void ) {
	return 2;
}

static int CB_PREFIX( void ) {
	RegR = ( RegR & 0x80 ) | ( ( RegR + 1 ) & 0x7F );
	return TabInstrCB[ LastInstr = PtrRegs == Regs ? Fetch() : PEEK8( RegPC + 1 ) ]();
}

static int DD_FD_PREFIX( void ) {
	PtrRegs = LastInstr == 0xDD ? RegsDD : RegsFD;
	RegR = ( RegR & 0x80 ) | ( ( RegR + 1 ) & 0x7F );
	return 1 + TabInstr[ LastInstr = Fetch() ]();
}

static int ED_PREFIX( void ) {
	PtrRegs = Regs;
	RegR = ( RegR & 0x80 ) | ( ( RegR + 1 ) & 0x7F );
	return TabInstrED[ LastInstr = Fetch() ]();
}

pfct TabInstr[ 256 ] = {
	NOP, LD_RR_NN, LD_INDIRECT_BCDE_A, INC_RR, INC_R, DEC_R, LD_R8, RLCA,
	EX_AF_AF_PRIME, ADD_HL_RR, LD_A_INDIRECT_BCDE, DEC_RR, INC_R, DEC_R, LD_R8, RRCA,
	DJNZ_E, LD_RR_NN, LD_INDIRECT_BCDE_A, INC_RR, INC_R, DEC_R, LD_R8, RLA,
	JR_E, ADD_HL_RR, LD_A_INDIRECT_BCDE, DEC_RR, INC_R, DEC_R, LD_R8, RRA,
	JR_DD_E, LD_RR_NN, LD_INDIRECT_NN_RR, INC_RR, INC_R, DEC_R, LD_R8, DAA,
	JR_DD_E, ADD_HL_RR, LD_RR_INDIRECT_NN, DEC_RR, INC_R, DEC_R, LD_R8, CPL,
	JR_DD_E, LD_RR_NN, LD_INDIRECT_NN_A, INC_RR, INC_INDIRECT_HL, DEC_INDIRECT_HL, LD_INDIRECT_HL_N, SCF,
	JR_DD_E, ADD_HL_RR, LD_A_INDIRECT_NN, DEC_RR, INC_R, DEC_R, LD_R8, CCF,
	NOP, LD_R8, LD_R8, LD_R8, LD_R8, LD_R8, LD_R_INDIRECT_HL, LD_R8,
	LD_R8, NOP, LD_R8, LD_R8, LD_R8, LD_R8, LD_R_INDIRECT_HL, LD_R8,
	LD_R8, LD_R8, NOP, LD_R8, LD_R8, LD_R8, LD_R_INDIRECT_HL, LD_R8,
	LD_R8, LD_R8, LD_R8, NOP, LD_R8, LD_R8, LD_R_INDIRECT_HL, LD_R8,
	LD_R8, LD_R8, LD_R8, LD_R8, NOP, LD_R8, LD_R_INDIRECT_HL, LD_R8,
	LD_R8, LD_R8, LD_R8, LD_R8, LD_R8, NOP, LD_R_INDIRECT_HL, LD_R8,
	LD_INDIRECT_HL_R, LD_INDIRECT_HL_R, LD_INDIRECT_HL_R, LD_INDIRECT_HL_R, LD_INDIRECT_HL_R, LD_INDIRECT_HL_R, HALT, LD_INDIRECT_HL_R,
	LD_R8, LD_R8, LD_R8, LD_R8, LD_R8, LD_R8, LD_R_INDIRECT_HL, NOP,
	ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8,
	ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8, ADD_R8,
	SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8,
	SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8, SUB_R8,
	LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8,
	LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8,
	LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8, LOG_R8,
	CP_R8, CP_R8, CP_R8, CP_R8, CP_R8, CP_R8, CP_R8, CP_R8,
	RET_CC, POP_SS, JP_CC_NN, JP_NN, CALL_CC_NN, PUSH_SS, ADD_R8, RST_P,
	RET_CC, RET, JP_CC_NN, CB_PREFIX, CALL_CC_NN, CALL_NN, ADD_R8, RST_P,
	RET_CC, POP_SS, JP_CC_NN, OUT_N_A, CALL_CC_NN, PUSH_SS, SUB_R8, RST_P,
	RET_CC, EXX, JP_CC_NN, IN_A_N, CALL_CC_NN, DD_FD_PREFIX, SUB_R8, RST_P,
	RET_CC, POP_SS, JP_CC_NN, EX_INDIRECT_SP_HL, CALL_CC_NN, PUSH_SS, LOG_R8, RST_P,
	RET_CC, JP_HL, JP_CC_NN, EX_DE_HL, CALL_CC_NN, ED_PREFIX, LOG_R8, RST_P,
	RET_CC, POP_SS, JP_CC_NN, DI_EI, CALL_CC_NN, PUSH_SS, LOG_R8, RST_P,
	RET_CC, LD_SP_HL, JP_CC_NN, DI_EI, CALL_CC_NN, DD_FD_PREFIX, CP_R8, RST_P
};

pfct TabInstrCB[ 256 ] = {
	RLC_R, RLC_R, RLC_R, RLC_R, RLC_R, RLC_R, RLC_INDIRECT_HL, RLC_R,
	RRC_R, RRC_R, RRC_R, RRC_R, RRC_R, RRC_R, RRC_INDIRECT_HL, RRC_R,
	RL_R, RL_R, RL_R, RL_R, RL_R, RL_R, RL_INDIRECT_HL, RL_R,
	RR_R, RR_R, RR_R, RR_R, RR_R, RR_R, RR_INDIRECT_HL, RR_R,
	SLA_R, SLA_R, SLA_R, SLA_R, SLA_R, SLA_R, SLA_INDIRECT_HL, SLA_R,
	SRA_R, SRA_R, SRA_R, SRA_R, SRA_R, SRA_R, SRA_INDIRECT_HL, SRA_R,
	SLL_R, SLL_R, SLL_R, SLL_R, SLL_R, SLL_R, SLL_INDIRECT_HL, SLL_R,
	SRL_R, SRL_R, SRL_R, SRL_R, SRL_R, SRL_R, SRL_INDIRECT_HL, SRL_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_R, BIT_B_INDIRECT_HL, BIT_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_R, RES_B_INDIRECT_HL, RES_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R,
	SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_R, SET_B_INDIRECT_HL, SET_B_R
};

pfct TabInstrED[ 256 ] = {
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	IN_R_C, OUT_C_R, SBC_HL_RR, LD_INDIRECT_NN_RR, NEG, RETI_RETN, IM_N, LD_I_A_LD_R_A,
	IN_R_C, OUT_C_R, ADC_HL_RR, LD_RR_INDIRECT_NN, NEG, RETI_RETN, IM_N, LD_I_A_LD_R_A,
	IN_R_C, OUT_C_R, SBC_HL_RR, LD_INDIRECT_NN_RR, NEG, RETI_RETN, IM_N, LD_A_I_LD_A_R,
	IN_R_C, OUT_C_R, ADC_HL_RR, LD_RR_INDIRECT_NN, NEG, RETI_RETN, IM_N, LD_A_I_LD_A_R,
	IN_R_C, OUT_C_R, SBC_HL_RR, LD_INDIRECT_NN_RR, NEG, RETI_RETN, IM_N, RLD_RRD,
	IN_R_C, OUT_C_R, ADC_HL_RR, LD_RR_INDIRECT_NN, NEG, RETI_RETN, IM_N, RLD_RRD,
	IN_R_C, OUT_C_R, SBC_HL_RR, LD_INDIRECT_NN_RR, NEG, RETI_RETN, IM_N, ED_XX,
	IN_R_C, OUT_C_R, ADC_HL_RR, LD_RR_INDIRECT_NN, NEG, RETI_RETN, IM_N, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	LDI_LDD, CPI_CPD, INI_IND, OUTI_OUTD, ED_XX, ED_XX, ED_XX, ED_XX,
	LDI_LDD, CPI_CPD, INI_IND, OUTI_OUTD, ED_XX, ED_XX, ED_XX, ED_XX,
	LDIR_LDDR, CPIR_CPDR, INIR_INDR, OTIR_OTDR, ED_XX, ED_XX, ED_XX, ED_XX,
	LDIR_LDDR, CPIR_CPDR, INIR_INDR, OTIR_OTDR, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX,
	ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX, ED_XX
};

/* GENERAL */
static void SetLedFDC( int Etat ) {
	static RECT Rect = { 660, 550, 674, 556 };

	HBRUSH br = CreateSolidBrush( Etat );
	FillRect( DeviceContext, &Rect, br );
	DeleteObject( br );
}

static void AffichePiste( void ) {
	static char Msg[ 16 ] = "Piste:xx";

	int i = Cpc.FddCurrTrack[ UnitSel ];
	Msg[ 6 ] = ( char )( '0' + ( i / 10 ) );
	Msg[ 7 ] = ( char )( '0' + ( i % 10 ) );
	TextOut( DeviceContext, 690, 544, Msg, 8 );
}

static void AfficheInfos( void ) {
	static RECT Rect = { 20, 556, 768, 570 };

	FillRect( DeviceContext, &Rect, ( HBRUSH )( COLOR_WINDOWFRAME + 1 ) );
	TextOut( DeviceContext, 20, 556, NomDisc, strlen( NomDisc ) );
	AffichePiste();
}

/* UPD */
static void StartExePhase( void ) {
	Status |= STATUS_EXE;
	SetLedFDC( 0xFF );
}

static void EndExePhase( void ) {
	Status &= ~STATUS_EXE;
	SetLedFDC( 0x40 );
}

static void CalcTailleSect( CPCEMUSect * Sect ) {
	TailleSect = Sect->SectSize ? Sect->SectSize : 128 << Sect->N;
	Weak = ( Sect->SectSize / TailleSect ) - 1;
	TailleSect = min( TailleSect, 0x1800 );
}

static int NextSecteur( int dataSect[ 4 ] ) {
	if ( Dsk[ UnitSel ].Image && CurrTrackDatasDSK[ UnitSel ][ HeadSel ] ) {
		CPCEMUSect * Sect = &CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->Sect[ IndexSecteur ];
		ST1 &= ~ST1_ND;
		CalcTailleSect( Sect );
		PosData += TailleSect + PosWeak;
		PosWeak += TailleSect;
		PosWeak = PosWeak < TailleSect * Weak ? PosWeak : 0;
		if ( ++IndexSecteur >= CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->NbSect )
			IndexSecteur = PosData = 0;

		Sect = &CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->Sect[ IndexSecteur ];
		dataSect[ 0 ] = Sect->C;
		dataSect[ 1 ] = Sect->H;
		dataSect[ 2 ] = Sect->R;
		dataSect[ 3 ] = Sect->N;
	}
	else
		memset( dataSect, 0xFF, sizeof( dataSect ) );

	return IndexSecteur;
}

static void SetSecteur( int index ) {
	int HoleIndex = 0, tmpSect[ 4 ];

	if ( Dsk[ UnitSel ].Image && CurrTrackDatasDSK[ UnitSel ][ HeadSel ] ) {
		while( HoleIndex < 2 ) {
			CPCEMUSect * Sect = &CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->Sect[ IndexSecteur ];
			if ( IndexSecteur == index ) {
				ST0 &= ~ST0_IC1;
				ST1 = Sect->ST1 & ( ST1_DE | ST1_ND | ST1_MA );
				ST2 = Sect->ST2 & ( ST2_CM | ST2_CRC | ST2_MD );
				CalcTailleSect( Sect );
				return;
			}
			HoleIndex += NextSecteur( tmpSect ) ? 0 : 1;
		}
	}
	ST0 |= ST0_IC1;
	ST1 |= ST1_ND;
}

static void RechercheSecteur( int findSect[ 5 ] ) {
	int HoleIndex = 0, tmpSect[ 4 ];

	if ( Dsk[ UnitSel ].Image && CurrTrackDatasDSK[ UnitSel ][ HeadSel ] ) {
		while( HoleIndex < 2 ) {
			CPCEMUSect * Sect = &CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->Sect[ IndexSecteur ];
			if ( Sect->R == findSect[ 2 ] ) {
				if ( Sect->C == findSect[ 0 ] ) {
					ST2 &= ~ST2_WC & ~ST2_BC;
					if ( Sect->H == findSect[ 1 ] && Sect->N == findSect[ 3 ] ) {
						ST0 &= ~ST0_IC1;
						ST1 = Sect->ST1 & ( ST1_DE | ST1_ND | ST1_MA );
						ST2 = Sect->ST2 & ( ST2_CM | ST2_CRC | ST2_MD );
						CalcTailleSect( Sect );
						ST1 |= findSect[ 2 ] == findSect[ 4 ] ? ST1_EN : 0;
						return;
					}
				}
				else {
					ST2 |= ST2_WC | ( Sect->C == 0xFF ? ST2_BC : 0 );
					break;
				}
			}
			HoleIndex += NextSecteur( tmpSect ) ? 0 : 1;
		}
	}
	ST0 |= ST0_IC1;
	ST1 |= ST1_ND;
}

static void MoveTrack( int tps, int maxMove ) {
	static int TimeMoveTrack;

	if ( ! tps ) {
		TimeMoveTrack = 0;
		NbSteps = maxMove;
		ST0 &= ~ST0_SE & ~ST0_EC;
		ST3 &= ~ST3_RY;
		Status &= ~STATUS_RQM;
		StartExePhase();
		TpsNextByte += SRT;
	}
	else
		if ( Cpc.FddCurrTrack[ UnitSel ] == NewCylinder | ! NbSteps ) {
			AffichePiste();
			FctExec = 0;
			ST0 = ST0 & ~ST0_IC1 & ~ST0_IC2 | ST0_SE;
			if ( Dsk[ UnitSel ].Image ) {
				CurrTrackDatasDSK[ UnitSel ][ 0 ] = Dsk[ UnitSel ].Tracks[ Cpc.FddCurrTrack[ UnitSel ] ][ 0 ];
				CurrTrackDatasDSK[ UnitSel ][ 1 ] = Dsk[ UnitSel ].Tracks[ Cpc.FddCurrTrack[ UnitSel ] ][ 1 ];
				ST3 |= ST3_RY;
			}
			TpsNextByte = IndexSecteur = PosData = 0;
			ST3 ^= ( - ! Cpc.FddCurrTrack[ UnitSel ] ^ST3 ) & ST3_T0;
			ST0 |= Cpc.FddCurrTrack[ UnitSel ] != NewCylinder ? ST0_EC : 0;
			Status = ( Status & ~STATUS_CB & ~STATUS_DIO ) | STATUS_RQM;
			EndExePhase();
		}
		else {
			TpsNextByte += tps;
			TimeMoveTrack += tps;
			if ( TimeMoveTrack > SRT ) {
				TimeMoveTrack -= SRT;
				Cpc.FddCurrTrack[ UnitSel ] += ( Cpc.FddCurrTrack[ UnitSel ] < NewCylinder ? 1 : -1 );
				NbSteps--;
				AffichePiste();
			}
		}
}

static int CmdInvalid( int val, int sens, int tps ) {
	Status |= sens == SENS_WR ? STATUS_DIO : 0;
	if ( sens == SENS_RD && ( Status & STATUS_DIO ) ) {
		FctExec = 0;
		Status &= ~STATUS_DIO & ~STATUS_CB;
		EndExePhase();
		return ST0 = ST0_IC2;
	}
	return 0xFF;
}

static int CmdSpecify( int val, int sens, int tps ) {
	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) ) {
		switch( PosStack++ ) {
		case 1:
			SRT = ( 16 - ( val >> 4 ) ) * 2000;
			HUT = val & 0x0F;
			break;

		case 2:
			HLT = val & 0xFE;
			FctExec = 0;
			Status &= ~STATUS_DIO & ~STATUS_CB;
			EndExePhase();
			break;
		}
	}
	return 0xFF;
}

static int CmdSenseDriveStatus( int val, int sens, int tps ) {
	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) ) {
		if ( PosStack++ ) {
			UnitSel = val & 3;
			HeadSel = ( val >> 2 ) & 1;
			ST3 = ST3 & 0xF8 | UnitSel | ( HeadSel << 2 );
			Status |= STATUS_DIO;
		}
	}
	if ( sens == SENS_RD && ( Status & STATUS_DIO ) ) {
		FctExec = 0;
		Status &= ~STATUS_DIO & ~STATUS_CB;
		EndExePhase();
		return ST3;
	}
	return 0xFF;
}

static int ReadDataWithCM( int val, int sens, int tps, int mask ) {
	static int cntdata, newPos, posSect, cntExtra, dataSect[ 6 ], sectSize;
	static BYTE BufExtra[ 0x100 ];
	static int posExtra;
	int p, ret, gap, tmpSect[ 4 ];

	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) ) {
		switch( p = PosStack++ ) {
		case 1 :
			UnitSel = val & 3;
			HeadSel = ( val >> 2 ) & 1;
			ST0 &= ~ST0_IC1 & ~ST0_IC2;
			break;

		case 2 :
		case 3 :
		case 4 :
		case 5 :
		case 6 :
		case 7 :
			dataSect[ p - 2 ] = val;
			break;

		case 8 :
			sectSize = val;
			if ( mask == READ_TRACK )
				SetSecteur( posSect = 0 );
			else {
				RechercheSecteur( dataSect );
				ST2 ^= mask;
			}
			cntdata = PosData;
			Status |= STATUS_DIO;
			if ( ST1 & ST1_ND ) {
				PosStack += mask != READ_TRACK + 1;
				Break = 1;
			}
			else {
				StartExePhase();
				TpsOverRun = TPS_OVER_RUN * 8000;
				CptOverRun = 1;
				TailleSect = dataSect[ 3 ] ? TailleSect : sectSize;
			}
			break;
		}
	}
	if ( sens == SENS_RD && ( Status & STATUS_DIO ) ) {
		switch( p = PosStack++ ) {
		case 9 :
			gap = Dsk[ UnitSel ].Image ? CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->Gap3 : 0x4E;
			TpsOverRun = TPS_OVER_RUN;
			posExtra = 0;
			BufExtra[ posExtra++ ] = 0xE5; // ### crc secteur
			BufExtra[ posExtra++ ] = 0xE5; // ### crc secteur
			memset( &BufExtra[ posExtra ], 0x4E, gap );
			posExtra += gap;
			memset( &BufExtra[ posExtra ], 0x00, 12 );
			posExtra += 12;
			BufExtra[ posExtra++ ] = 0xA1;
			BufExtra[ posExtra++ ] = 0xA1;
			BufExtra[ posExtra++ ] = 0xA1;
			BufExtra[ posExtra++ ] = 0xFE;
			BufExtra[ posExtra++ ] = 0x00; // ### C;
			BufExtra[ posExtra++ ] = 0x00; // ### H;
			BufExtra[ posExtra++ ] = 0x00; // ### R;
			BufExtra[ posExtra++ ] = 0x00; // ### N;
			BufExtra[ posExtra++ ] = 0x00; // ### crc id
			BufExtra[ posExtra++ ] = 0x00; // ### crc id
			memset( &BufExtra[ posExtra ], 0x4E, 22 );
			posExtra += 22;
			memset( &BufExtra[ posExtra ], 0x00, 12 );
			posExtra += 12;
			BufExtra[ posExtra++ ] = 0xA1;
			BufExtra[ posExtra++ ] = 0xA1;
			BufExtra[ posExtra++ ] = 0xA1;
			BufExtra[ posExtra++ ] = 0xFB;
			cntExtra = posExtra;
			posExtra = 0;
			ret = Dsk[ UnitSel ].Data[ Cpc.FddCurrTrack[ UnitSel ] ][ HeadSel ][ cntdata++ ];
			if ( Status & STATUS_EXE ) {
				if ( --TailleSect )
					PosStack--;
				else 
					if ( mask != READ_TRACK ) {
						NextSecteur( tmpSect );
						if ( dataSect[ 2 ] < dataSect[ 4 ] ) {
							dataSect[ 2 ]++;
							RechercheSecteur( dataSect );
							cntdata = PosData;
							PosStack--;
						}
						else {
							CptOverRun = 0;
							EndExePhase();
							PosStack++;
						}
					}
			}
			else
				PosStack += mask == READ_TRACK;
			return ret;
			break;

		case 10 :
			TpsOverRun = TPS_OVER_RUN * 6250;
			ret = BufExtra[ posExtra++ ];
			cntExtra--;
			if ( Status & STATUS_EXE ) {
				if ( cntExtra )
					PosStack--;
				else {
					if ( posSect++ < CurrTrackDatasDSK[ UnitSel ][ HeadSel ]->NbSect ) {
						SetSecteur( posSect );
						cntdata = PosData;
						PosStack -= 2;
					}
					else {
						CptOverRun = 0;
						EndExePhase();
					}
				}
			}
			return ret;
			break;

		case 11 :
			CptOverRun = 0;
			return ST0 = ST0 & ~ST0_SE | ST0_IC1;

		case 12 :
			return ST1;

		case 13 :
			return ST2;

		case 14 :
		case 15 :
		case 16 :
		case 17 :
			if ( p == 17 ) {
				FctExec = 0;
				Status &= ~STATUS_DIO & ~STATUS_CB;
			}
			return dataSect[ p - 14 ];
		}
	}
	return 0xFF;
}

static int CmdReadTrack( int val, int sens, int tps ) {
	return ReadDataWithCM( val, sens, tps, READ_TRACK );
}

static int CmdReadData( int val, int sens, int tps ) {
	return ReadDataWithCM( val, sens, tps, 0x00 );
}

static int CmdReadDelData( int val, int sens, int tps ) {
	return ReadDataWithCM( val, sens, tps, ST2_CM );
}

static int WriteDataWithCM( int val, int sens, int tps, int mask ) {
	static int cntdata, newPos, dataSect[ 6 ], sectSize;
	int p, tmpSect[ 4 ];

	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) ) {
		switch( p = PosStack++ ) {
		case 1 :
			UnitSel = val & 3;
			HeadSel = ( val >> 2 ) & 1;
			ST0 &= ~ST0_IC1 & ~ST0_IC2;
			break;

		case 2 :
		case 3 :
		case 4 :
		case 5 :
		case 6 :
		case 7 :
			dataSect[ p - 2 ] = val;
			break;

		case 8 :
			sectSize = val;
			RechercheSecteur( dataSect );
			cntdata = PosData;
			if ( ! ( ST1 & ST1_ND ) ) {
				StartExePhase();
				TailleSect = dataSect[ 3 ] ? TailleSect : sectSize;
			}
			else {
				Status |= STATUS_DIO;
				PosStack++;
			}
			break;

		case 9 :
			Dsk[ UnitSel ].Data[ Cpc.FddCurrTrack[ UnitSel ] ][ HeadSel ][ cntdata++ ] = val;
			Dsk[ UnitSel ].FlagWrite = 1;
			if ( Status & STATUS_EXE ) {
				if ( --TailleSect )
					PosStack--;
				else {
					NextSecteur( tmpSect );
					if ( dataSect[ 2 ]++ < dataSect[ 4 ] ) {
						RechercheSecteur( dataSect );
						cntdata = PosData;
						PosStack--;
					}
					else {
						Status |= STATUS_DIO;
						EndExePhase();
					}
				}
			}
			break;
		}
	}
	if ( sens == SENS_RD && ( Status & STATUS_DIO ) ) {
		switch( p = PosStack++ ) {
		case 10 :
			return ST0 = UnitSel ^ ( - ( ! Cpc.FddMotor | UnitSel > 1 | ! Dsk[ UnitSel ].Image ) ^ UnitSel ) & ( ST0_IC1 | ST0_NR );

		case 11 :
			return ST1;

		case 12 :
			return ST2;

		case 13 :
		case 14 :
		case 15 :
		case 16 :
			if ( p == 16 ) {
				FctExec = 0;
				Status &= ~STATUS_DIO & ~STATUS_CB;
				EndExePhase();
			}
			return dataSect[ p - 13 ];
		}
	}
	return 0xFF;
}

static int CmdWriteData( int val, int sens, int tps ) {
	return WriteDataWithCM( val, sens, tps, 0x00 );
}

static int CmdWriteDelData( int val, int sens, int tps ) {
	return WriteDataWithCM( val, sens, tps, ST2_CM );
}

static int CmdRecalibrate( int val, int sens, int tps ) {
	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) && PosStack++ ) {
		UnitSel = val & 3;
		NewCylinder = 0;
		MoveTrack( 0, 77 );
	}
	if ( sens == SENS_NONE && ( Status & STATUS_EXE ) )
		MoveTrack( tps, 0 );

	return 0xFF;
}

static int CmdSenseInterStatus( int val, int sens, int tps ) {
	Status |= sens == SENS_WR ? STATUS_DIO :0;
	if ( sens == SENS_RD && ( Status & STATUS_DIO ) ) {
		switch( PosStack++ ) {
		case 0:
			if ( Break ) {
				ST0 = ST0 & ~ST0_IC1 | ST0_IC2;
				Break = 0;
			}
			return ST0;

		case 1:
			FctExec = 0;
			Status &= ~STATUS_DIO & ~STATUS_CB;
			EndExePhase();
			return Cpc.FddCurrTrack[ UnitSel ];
		}
	}
	return 0xFF;
}

static int CmdReadID( int val, int sens, int tps ) {
	static int dataSect[ 4 ];
	int p;

	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) && PosStack++ ) {
		UnitSel = val & 3;
		HeadSel = ( val >> 2 ) & 1;
		Status |= STATUS_DIO;
	}
	if ( sens == SENS_RD && ( Status & STATUS_DIO ) ) {
		switch( p = PosStack++ ) {
		case 2 :
			return ST0 = UnitSel ^ ( - ( ! Cpc.FddMotor | UnitSel > 1 | ! Dsk[ UnitSel ].Image ) ^ UnitSel ) & ( ST0_IC1 | ST0_NR );

		case 3 :
			return ST1;

		case 4 :
			return ST2;

		case 5 :
			NextSecteur( dataSect );

		case 6 :
		case 7 :
		case 8 :
			if ( p == 8 ) {
				FctExec = 0;
				Status &= ~STATUS_DIO & ~STATUS_CB;
				EndExePhase();
			}
			return dataSect[ p - 5 ];
		}
	}
	return 0xFF;
}

static int CmdSeek( int val, int sens, int tps ) {
	if ( sens == SENS_WR && ! ( Status & STATUS_DIO ) ) {
		switch( PosStack++ ) {
		case 1:
			UnitSel = val & 3;
			break;

		case 2:
			NewCylinder = val;
			MoveTrack( 0, 255 );
		}
	}
	if ( sens == SENS_NONE && ( Status & STATUS_EXE ) )
		MoveTrack( tps, 0 );

	return 0xFF;
}

static int CmdFormatTrack( int val, int sens, int tps ) {
	return 0;
}

static int CmdScanLow( int val, int sens, int tps ) {
	return 0;
}

static int CmdScanLowEq( int val, int sens, int tps ) {
	return 0;
}

static int CmdScanHighEq( int val, int sens, int tps ) {
	return 0;
}

static void ResetUPD( void ) {
	Status = STATUS_RQM;
	FctExec = 0;
	Cpc.FddMotor = 0;
	ST0 = ST0_SE;
	ST1 = ST2 = 0;
	ST3 = ST3_T0;
	SRT = 12000;
	PosWeak = PosStack = TpsNextByte = IndexSecteur = PosData = Break = HeadSel = NbSteps = 0;
	Weak = 1;
	for ( UnitSel = 4; --UnitSel; ) {
		Cpc.FddCurrTrack[ UnitSel ] = 0;
		if ( Dsk[ UnitSel ].Image ) {
			CurrTrackDatasDSK[ UnitSel ][ 0 ] = Dsk[ UnitSel ].Tracks[ 0 ][ 0 ];
			CurrTrackDatasDSK[ UnitSel ][ 1 ] = Dsk[ UnitSel ].Tracks[ 0 ][ 1 ];
			ST3 |= ST3_RY;
		}
	}
}

/* GENERAL */
static char * OpenNewFile( char * Filtre, char * Ext, char * Dir, char * OldName, BOOL Save ) {
	static char NomFic[ 256 ];
	OPENFILENAME ofn;
	BOOL ret;
	char * p = strchr( Dir + 1, '.' );
	if ( p )
		while( * p && * p != '\\' )
			* p--= 0;

	strcpy( NomFic, OldName );
	memset( &ofn, 0, sizeof( ofn ) );
	ofn.lStructSize = sizeof( OPENFILENAME );
	ofn.lpstrFilter = Filtre;
	ofn.nFilterIndex = 1L;
	ofn.lpstrFile = ( LPTSTR )NomFic;
	ofn.nMaxFile = sizeof( NomFic );
	ofn.lpstrInitialDir = Dir;
	ofn.lpstrDefExt = Ext;
	ofn.Flags = OFN_PATHMUSTEXIST | ( Save ? OFN_OVERWRITEPROMPT : OFN_FILEMUSTEXIST );
	IsSound = 0;
	ret = Save ? GetSaveFileName( &ofn ) : GetOpenFileName( &ofn );
	IsSound = 1;
	if ( ret )
		strcpy( Dir, ofn.lpstrFile );

	return NomFic;
}

static void ResetCPC( void ) {
	int i;
	RegPC = RegSP = 0;
	MemoColor = 0xFF;
	WriteVGA( 0x88 );
	ResetUPD();
	xEcr = yEcr = LigneCRTC = HDelay = VDelay = VswCount = HDT = VDT = MaCRTC = MaHi = HS = MR = VT = OldVSync = VCharCount = VHCount = 0;
	Rng = OutNoise = 1;
	memset( TabTone, 0, sizeof( TabTone ) );
	CntNoise = CntEnv = 0;
	IsSound = 1;
	for ( i = 0; i < 14; i++ )
		Write8912( i, 0 );
}

static void EjectDSK( int DrvNum ) {
	Dsk[ DrvNum ].Image = 0;
	if ( Dsk[ DrvNum ].FlagWrite ) {
		if ( MessageBox( NULL, "Sauvegarde fichier disquette ?", "Emu-CPC", MB_YESNO ) == IDYES ) {
			HANDLE f = CreateFile( Dsk[ DrvNum ].NomFic, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL );
			if ( f != INVALID_HANDLE_VALUE ) {
				DWORD BytesWritten;
				WriteFile( f, &Dsk[ DrvNum ].Infos, sizeof( Dsk[ DrvNum ].Infos ), &BytesWritten, NULL );
				WriteFile( f, DataDsk[ DrvNum ], Dsk[ DrvNum ].LongFic, &BytesWritten, NULL );
				CloseHandle( f );
			}
		}
		Dsk[ DrvNum ].FlagWrite = 0;
	}
}

static void SetDSK( int DrvNum ) {
	HANDLE f = CreateFile( NomDisc, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL );
	if ( f != INVALID_HANDLE_VALUE ) {
		int ImageType;
		DWORD BytesRead;
		//ResetUPD();
		EjectDSK( DrvNum );
		strcpy( Dsk[ DrvNum ].NomFic, NomDisc );
		ReadFile( f, &Dsk[ DrvNum ].Infos, sizeof( Dsk[ DrvNum ].Infos ), &BytesRead, NULL );
		ReadFile( f, &DataDsk[ DrvNum ], sizeof( DataDsk[ DrvNum ] ), &Dsk[ DrvNum ].LongFic, NULL );
		CloseHandle( f );
		ImageType = ! strncmp( Dsk[ DrvNum ].Infos.debut, "MV - CPC", 8 ) ? 1 : ! strncmp( Dsk[ DrvNum ].Infos.debut, "EXTENDED", 8 ) ? 2 : 0;
		if ( ImageType && Dsk[ DrvNum ].LongFic ) {
			int t, h, s, Pos = 0, PosTable = 0;
			memset( Dsk[ DrvNum ].Tracks, 0, sizeof( Dsk[ DrvNum ].Tracks ) );
			for ( h = 0; h < Dsk[ DrvNum ].Infos.NbHeads; h++ )
				for ( t = 0; t < Dsk[ DrvNum ].Infos.NbTracks; t++ ) {
					CPCEMUTrack * CurTrack = ( CPCEMUTrack * )&DataDsk[ DrvNum ][ Pos ];
					if ( ! strncmp( CurTrack->ID, "Track-Info", 10 ) ) {
						int NumTrack = CurTrack->Track;
						int NumHead = CurTrack->Head & 1;
						Dsk[ DrvNum ].Tracks[ NumTrack ][ NumHead ] = CurTrack;
						Dsk[ DrvNum ].Data[ NumTrack ][ NumHead ] = &DataDsk[ DrvNum ][ Pos + sizeof( CPCEMUTrack) ];
						if ( ! NumHead && Dsk[ DrvNum ].Infos.NbHeads == 1 ) {
							Dsk[ DrvNum ].Tracks[ NumTrack ][ 1 ] = CurTrack;
							Dsk[ DrvNum ].Data[ NumTrack ][ 1 ] = &DataDsk[ DrvNum ][ Pos + sizeof( CPCEMUTrack) ];
						}
						if ( ImageType == 2 )
							Pos += Dsk[ DrvNum ].Infos.TrackSizeTable[ PosTable++ ] << 8;
						else
							if ( Dsk[ DrvNum ].Infos.TrackSize )
								Pos += Dsk[ DrvNum ].Infos.TrackSize;
							else {
								for ( s = 0; s < CurTrack->NbSect; s++ ) {
									int n = CurTrack->Sect[ s ].SectSize;
									n = n & 0xFF ? ( n + 0xFF ) & 0x1F00 : n;
									if ( ! n ) {
										n = CurTrack->Sect[ s ].N;
										n = n < 6 ? 128 << n : 0x1800;
									}
									Pos += max( 0x100, n );
								}
								Pos += sizeof( CPCEMUTrack );
							}
					}
				}
				Dsk[ DrvNum ].Image = 1;
				Dsk[ DrvNum ].FlagWrite = 0;
				AfficheInfos();
		}
	}
}

static void TouchesFonctions( int Shift, WPARAM wParam ) {
	static char DirSnap[ 512 ], DirDisc[ 512 ];
	DWORD BytesWritten, BytesRead;
	int i;
	char * Nom;

	switch( wParam ) {
	case 0x71 :
		Turbo ^= 1;
		memset( MemBitmap, 0, X_ECR_MAX * Y_ECR_MAX * 4 );
		AfficheInfos();
		break;

	case 0x73 :
		Nom = OpenNewFile( "Snapshot files (*.SNA)\0*.SNA\0", "*.SNA", DirSnap, "", FALSE );
		if ( * Nom ) {
			HANDLE f = CreateFile( Nom, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL );
			if ( f != INVALID_HANDLE_VALUE ) {
				ResetCPC();
				ReadFile( f, &Cpc, sizeof( Cpc ), &BytesRead, NULL );
				if ( ! strncmp( Cpc.Id, "MV - SNA", 8 ) ) {
					Cpc.VGARam &= 7;
					for ( i = 0; i < 17; i++ ) {
						WriteVGA( i );
						WriteVGA( 0x40 | Cpc.InkData[ i ] );
						VGASyncColor();
					}
					for ( i = 0; i < 4; i++ )
						WritePPI( i + 0xF400 + ( i << 8 ), Cpc.PPI[ i ] );

					for ( i = 0; i < 16; i++ )
						Write8912( i, Cpc.PsgData[ i ] );

					MemoMode = Cpc.VGARom & 3;
					SetMemCPC();
					for ( i = 0; i < 18; i++ ) {
						WriteCRTC( 0xBC00, i );
						WriteCRTC( 0xBD00, Cpc.CRTCReg[ i ] );
					}
					ReadFile( f, MemCPC, 0x20000, &BytesRead, NULL );
				}
				CloseHandle( f );
			}
		}
		break;

	case 0x74 :
		Nom = OpenNewFile( "Snapshot files (*.SNA)\0*.SNA\0", "*.SNA", DirSnap, "", TRUE );
		if ( * Nom ) {
			HANDLE f = CreateFile( Nom, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL );
			if ( f != INVALID_HANDLE_VALUE ) {
				WriteFile( f, &Cpc, sizeof( Cpc ), &BytesWritten, NULL );
				WriteFile( f, MemCPC, 0x20000, &BytesWritten, NULL );
				CloseHandle( f );
			}
		}
		break;

	case 0x76 :
		Nom = OpenNewFile( "DSK files (*.DSK)\0*.DSK\0", "*.DSK", DirDisc, NomDisc, FALSE );
		if ( * Nom ) {
			strcpy( NomDisc, Nom );
			SetDSK( Shift );
		}
		break;

	case 0x77 :
		ResetCPC();
		break;
	}
}

static LRESULT CALLBACK WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam ) {
	static int Shift = 0;

	switch( message ) {
	case WM_KEYUP :
		Shift = wParam == 0x10 ? 0 : Shift;
		clav[ TrKey[ wParam ].Ligne ] |= ~TrKey[ wParam ].Port;
		TouchesFonctions( Shift, wParam );
		break;

	case WM_KEYDOWN :
		Shift = wParam == 0x10 ? 1 : Shift;
		clav[ TrKey[ wParam ].Ligne ] &= TrKey[ wParam ].Port;
		break;

	case WM_PAINT :
		AfficheInfos();
		break;

	case WM_DESTROY :
	case WM_CLOSE :
		finMain = 1;
		break;
	}
	return DefWindowProc( hWnd, message, wParam, lParam );
}

static void Go( void ) {
	MSG msg;
	unsigned long TimeOut = 0, Time, OldTime = 0;
	int i, t, DoResync, CalcMa;

	while( ! finMain ) {
		PtrRegs = Regs;
		RegR = ( RegR & 0x80 ) + ( ( 1 + RegR ) & 0x7F );
		t = TabInstr[ LastInstr = Fetch() ]();
		if ( Cpc.IRQ && Cpc.IFF1 && LastInstr != 0xFB ) {
			RegSP -= 2;
			POKE16( RegSP, RegPC + ( LastInstr == 0x76 ) );
			RegR = ( RegR & 0x80 ) + ( ( 1 + RegR ) & 0x7F );
			memPtr = RegPC = Cpc.InterruptMode < 2 ? 0x38 : PEEK16( Cpc.IR.Word | 0xFF );
			t += 6 + ( Cpc.InterruptMode >> 1 ) - SupIrqWaitState;
			SupIrqWaitState = Cpc.IRQ = Cpc.IFF1 = Cpc.IFF2 = 0;
			Cpc.CntHSync &= 0x1F;
		}
		TimeOut += t;
		if ( CptOverRun ) {
			TpsOverRun -= t;
			if ( TpsOverRun < 0 ) {
				ST1 |= ST1_OR;
				CptOverRun = 0;
				EndExePhase();
			}
		}
		if ( FctExec && ( Status & ( STATUS_EXE | STATUS_DIO ) ) )
			FctExec( 0, SENS_NONE, t );

		if ( TpsNextByte > 0 && ! ( Status & STATUS_RQM ) )
			TpsNextByte -= t;
		else
			Status |= STATUS_RQM;

		if ( Cpc.FddMotor && ! ( ST3 & ST3_RY ) ) {
			TpsReady -=t;
			ST3 |= TpsReady <= 0 && Dsk[ UnitSel ].Image ? ST3_RY : 0;
		}
		DoResync = CalcMa = 0;
		for ( ; t--; ) {
			VCharCount++;
			if ( Cpc.HCC++ >= Cpc.CRTCReg[ 0 ] ) {
				Cpc.HCC = 0;
				Cpc.VLC = ++Cpc.VLC & 0x1F;
				if ( VSync && ++Cpc.TailleVBL >= ( ( Cpc.CRTCReg[ 3 ] & 0xF0 ) ? Cpc.CRTCReg[ 3 ] >> 4 : 16 ) ) {
					OldVSync = 1;
					VSync = 0;
				}
				if ( MR ) {
					MR = 0;
					Cpc.VLC = 0;
					MaCRTC += HDT ? 0 : Cpc.CRTCReg[ 1 ];
					Cpc.VCC = ++Cpc.VCC & 0x7F;
				}
				CalcMa = Cpc.VtAdj ? ! --Cpc.VtAdj : CalcMa;
				if ( VT ) {
					VT = 0;
					if ( Cpc.CRTCReg[ 5 ] )
						Cpc.VtAdj = Cpc.CRTCReg[ 5 ];
					else
						CalcMa = 1;
				}
				if ( CalcMa ) {
					CalcMa = OldVSync = Cpc.VLC = Cpc.VCC = 0;
					VDT = 1;
					MaCRTC = Cpc.CRTCReg[ 13 ] + MaHi;
				}
				if ( Cpc.VLC == Cpc.CRTCReg[ 9 ] ) {
					MR = 1;
					VT = ! Cpc.VtAdj && Cpc.VCC == Cpc.CRTCReg[ 4 ] ? 1 : VT;
				}
				VDT = Cpc.VCC == Cpc.CRTCReg[ 6 ] ? 0 : VDT;
				if ( Cpc.VCC == Cpc.CRTCReg[ 7 ] && ! VSync && ! OldVSync ) {
					Cpc.TailleVBL = 0;
					VSync = 1;
					Cpc.SyncCount = 2;
					VDelay = 2;
					VswCount = 4;
				}
				HDT = 1;
			}
			HDT = Cpc.HCC == Cpc.CRTCReg[ 1 ] ? 0 : HDT;
			if ( HS ) {
				if ( HDelay != 2 )
					HDelay++;
				else
					if ( ! --VHCount ) {
						LigneCRTC++;
						VCharCount = 0;
						HDelay++;
					}
			}
			if ( Cpc.TailleHBL && ! --Cpc.TailleHBL ) {
				HS = 0;
				VDelay -= VDelay != 0;
				if ( ! VDelay && VswCount && ! --VswCount ) {
					LigneCRTC = 0;
					DoResync = 1;
				}
				Cpc.CntHSync++;
				if ( ( Cpc.SyncCount && ! --Cpc.SyncCount ) | Cpc.CntHSync == 52 ) {
					Cpc.IRQ = Cpc.CntHSync & 32;
					Cpc.CntHSync = 0;
				}
				LastMode = MemoMode;
			}
			if ( Cpc.HCC == Cpc.CRTCReg[ 2 ] ) {
				HDelay = 0;
				HS = 1;
				Cpc.TailleHBL = ( Cpc.CRTCReg[ 3 ] & 0x0F ) ? Cpc.CRTCReg[ 3 ] & 0x0F : 16;
				VHCount = max( 0, min( Cpc.TailleHBL - 2, 4 ) );
			}
			if ( yEcr && yEcr < Y_ECR_MAX && xEcr < X_ECR_MAX && ( ! Turbo | TimeOut > TIME_OUT_TURBO ) ) {
				int * p = MemBitmap + xEcr + ( Y_ECR_MAX - yEcr ) * X_ECR_MAX;
				if ( HDT && VDT ) {
					if ( LigneCRTC > TAILLE_VBL ) {
						int Adr = ( ( ( ( Cpc.HCC + MaCRTC ) << 1 ) + ( ( ( Cpc.HCC + MaCRTC ) & 0x1000 ) << 2 ) ) & 0xC7FF ) | ( Cpc.VLC << 11 );
						int * pt = TabPoints[ LastMode ][ MemCPC[ Adr++ ] ];
						for ( i = 0; i < 16; i++ ) {
							p[ i ]= TabCoul[ * pt++ ];
							if ( i == 7 ) {
								VGASyncColor();
								pt = TabPoints[ LastMode ][ MemCPC[ Adr ] ];
							}
						}
					}
					else
						memset( p, 0, 64 );
				}
				else {
					for ( i = 0; i < 16; i++ ) {
						p[ i ] = TabCoul[ 16 ];
						if ( i == 7 )
							VGASyncColor();
					}
				}
				xEcr += 16;
				if ( ! Turbo )
					memcpy( p + X_ECR_MAX, p, 64 );
			}
			else {
				if ( VCharCount == 7 ) {
					xEcr = 0;
					yEcr = yEcr >= Y_ECR_MAX && LigneCRTC == TAILLE_VBL ? 0 : yEcr + 2;
				}
				if ( LigneCRTC == TAILLE_Y_CRTC ) {
					LigneCRTC = 0;
					DoResync = 1;
				}
			}
			VGASyncColor();
		}
		if ( DoResync || TimeOut > TIME_OUT_CRTC ) {
			if ( ! Turbo ) {
				do {
					Time = timeGetTime() * 1000;
					Sleep( Time - OldTime < TimeOut - 1000 );
				}
				while( Time - OldTime < TimeOut );
				OldTime = Time;
			}
			if ( ! Turbo | TimeOut > TIME_OUT_TURBO ) {
				TimeOut = 0;
				BitBlt( DeviceContext, 0, 0, X_ECR_MAX, Y_ECR_MAX - 32, DcBitmap, 0, 0, SRCCOPY );
				if ( PeekMessage( &msg, NULL, 0,0, PM_REMOVE ) )
					DispatchMessage( &msg );
			}
		}
	}
}

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow ) {
	WNDCLASSEX wcex;

	memset( &wcex, 0, sizeof( wcex ) );
	wcex.cbSize = sizeof( WNDCLASSEX ); 
	wcex.lpfnWndProc = ( WNDPROC )WndProc;
	wcex.hbrBackground = CreateSolidBrush( BK_COLOR );
	wcex.hInstance = hInstance;
	wcex.lpszClassName = TITRE;
	if ( RegisterClassEx( &wcex ) ) {
		HWND win = CreateWindowEx( 0, TITRE, TITRE, WS_SYSMENU | WS_MINIMIZEBOX, CW_USEDEFAULT, CW_USEDEFAULT, 774, 596, NULL, NULL, hInstance, NULL );
		if ( win ) {
			BITMAPINFO pbmi;
			HBITMAP bm;
			DeviceContext = GetDC( win );
			memset( &pbmi, 0, sizeof( pbmi ) );
			pbmi.bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
			pbmi.bmiHeader.biWidth = X_ECR_MAX;
			pbmi.bmiHeader.biHeight = Y_ECR_MAX;
			pbmi.bmiHeader.biPlanes = 1;
			pbmi.bmiHeader.biBitCount = 32;
			pbmi.bmiHeader.biCompression = BI_RGB;
			bm = CreateDIBSection( DeviceContext, &pbmi, DIB_RGB_COLORS, ( void ** )&MemBitmap, NULL, 0 );
			if ( bm ) {
				DcBitmap = CreateCompatibleDC( DeviceContext );
				if ( DcBitmap ) {
					HFONT f = CreateFont( 14, 0, 0, 0, FW_BOLD, 0, 0, 0, OEM_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, FIXED_PITCH | FF_DONTCARE,NULL );
					SelectObject( DcBitmap, bm );
					if ( f ) {
						int i;
						DWORD ThreadID = 0;
						HANDLE hThread;
						SetTextColor( DeviceContext, TEXT_COLOR );
						SetBkColor( DeviceContext, BK_COLOR );
						SelectObject( DeviceContext, f );
						ShowWindow( win, SW_SHOW );
						memcpy( Cpc.Id, "MV - SNA", 8 );
						Cpc.MemSize = 128;
						Cpc.Version = 3;

						Regs[ 0 ] = &Cpc.BC.Byte.High;
						Regs[ 1 ] = Regs[ 8 ] = Regs[ 12 ] = &Cpc.BC.Byte.Low;
						Regs[ 2 ] = &Cpc.DE.Byte.High;
						Regs[ 3 ] = Regs[ 9 ] = Regs[ 13 ] = &Cpc.DE.Byte.Low;
						Regs[ 4 ] = &Cpc.HL.Byte.High;
						Regs[ 5 ] = Regs[ 6 ] = Regs[ 10 ] = Regs[ 14 ] = &Cpc.HL.Byte.Low;
						Regs[ 7 ] = &Cpc.AF.Byte.High;
						Regs[ 11 ] = &Cpc.SP.Byte.Low;
						Regs[ 15 ] = &Cpc.AF.Byte.Low;
						memcpy( RegsDD, Regs, sizeof( RegsDD ) );
						memcpy( RegsFD, Regs, sizeof( RegsFD ) );
						RegsDD[ 4 ] = &Cpc.IX.Byte.High;
						RegsDD[ 5 ] = RegsDD[ 6 ] = RegsDD[ 10 ] = RegsDD[ 14 ] = &Cpc.IX.Byte.Low;
						RegsFD[ 4 ] = &Cpc.IY.Byte.High;
						RegsFD[ 5 ] = RegsFD[ 6 ] = RegsFD[ 10 ] = RegsFD[ 14 ] = &Cpc.IY.Byte.Low;

						for ( i = 0; i < 256; i++ ) {
							TabPoints[ 0 ][ i ][ 0 ] = TabPoints[ 0 ][ i ][ 1 ] = TabPoints[ 0 ][ i ][ 2 ] = TabPoints[ 0 ][ i ][ 3 ] = ( i >> 7 ) + ( ( i & 32 ) >> 3 ) + ( ( i & 8 ) >> 2 ) + ( ( i & 2 ) << 2 );
							TabPoints[ 0 ][ i ][ 4 ] = TabPoints[ 0 ][ i ][ 5 ] = TabPoints[ 0 ][ i ][ 6 ] = TabPoints[ 0 ][ i ][ 7 ] = ( ( i & 64 ) >> 6 ) + ( ( i & 16 ) >> 2 ) + ( ( i & 4 ) >> 1 ) + ( ( i & 1 ) << 3 );
							TabPoints[ 1 ][ i ][ 0 ] = TabPoints[ 1 ][ i ][ 1 ] = ( i >> 7 ) + ( ( i & 8 ) >> 2 );
							TabPoints[ 1 ][ i ][ 2 ] = TabPoints[ 1 ][ i ][ 3 ] = ( ( i & 64 ) >> 6 ) + ( ( i & 4 ) >> 1 );
							TabPoints[ 1 ][ i ][ 4 ] = TabPoints[ 1 ][ i ][ 5 ] = ( ( i & 32 ) >> 5 ) + ( i & 2 );
							TabPoints[ 1 ][ i ][ 6 ] = TabPoints[ 1 ][ i ][ 7 ] = ( ( i & 16 ) >> 4 ) + ( ( i & 1 ) << 1 );
							TabPoints[ 2 ][ i ][ 0 ] = i >> 7;
							TabPoints[ 2 ][ i ][ 1 ] = ( i & 64 ) >> 6;
							TabPoints[ 2 ][ i ][ 2 ] = ( i & 32 ) >> 5;
							TabPoints[ 2 ][ i ][ 3 ] = ( i & 16 ) >> 4;
							TabPoints[ 2 ][ i ][ 4 ] = ( i & 8 ) >> 3;
							TabPoints[ 2 ][ i ][ 5 ] = ( i & 4 ) >> 2;
							TabPoints[ 2 ][ i ][ 6 ] = ( i & 2 ) >> 1;
							TabPoints[ 2 ][ i ][ 7 ] = i & 1;
							TabPoints[ 3 ][ i ][ 0 ] = TabPoints[ 3 ][ i ][ 1 ] = TabPoints[ 3 ][ i ][ 2 ] = TabPoints[ 3 ][ i ][ 3 ] = ( i >> 7 ) + ( ( i & 8 ) >> 2 );
							TabPoints[ 3 ][ i ][ 4 ] = TabPoints[ 3 ][ i ][ 5 ] = TabPoints[ 3 ][ i ][ 6 ] = TabPoints[ 3 ][ i ][ 7 ] = ( ( i & 64 ) >> 6 ) + ( ( i & 4 ) >> 1 );
						}
						memset( clav, 0xFF, sizeof( clav ) );
						hThread = CreateThread( 0, 0, ( LPTHREAD_START_ROUTINE )ThreadFunc, 0, 0, &ThreadID );
						if ( hThread ) {
							WAVEHDR Header[ SND_NB_BUFFERS ];
							BYTE Buf[ SND_NB_BUFFERS ][ SND_BUFSIZE ];
							HWAVEOUT hWave = 0;
							WAVEFORMATEX Format;
							SetThreadPriority( hThread, THREAD_PRIORITY_TIME_CRITICAL );
							CloseHandle( hThread );
							Sleep( 100 );
							Format.wFormatTag = WAVE_FORMAT_PCM;
							Format.nChannels = 1;
							Format.nSamplesPerSec = Format.nAvgBytesPerSec = SND_RATE;
							Format.wBitsPerSample = 8;
							Format.nBlockAlign = 1;
							Format.cbSize = 0;
							if ( waveOutOpen( &hWave, WAVE_MAPPER, &Format, ThreadID, 0 , CALLBACK_THREAD ) ) {
								PostThreadMessage( ThreadID, MM_WOM_CLOSE, ( WPARAM )hWave, 0 );
								hWave = 0;
							}
							else {
								for ( i = 0; i < SND_NB_BUFFERS; i++ ) {
									memset( &Header[ i ], 0, sizeof( WAVEHDR ) );
									memset( &Buf[ i ], 128, SND_BUFSIZE );
									Header[ i ].lpData = ( char * )&Buf[ i ];
									Header[ i ].dwBufferLength = SND_BUFSIZE;
									Header[ i ].dwUser = i;
									waveOutPrepareHeader( hWave, &Header[ i ], sizeof( WAVEHDR ) );
									waveOutWrite( hWave, &Header[ i ], sizeof( WAVEHDR ) );
								}
							}
							ResetCPC();
							timeBeginPeriod( 1 );
							Go();
							timeEndPeriod( 1 );
							EjectDSK( 0 );
							EjectDSK( 1 );
							Exit = 1;
							Sleep( 100 );
							if ( hWave ) {
								waveOutReset( hWave );
								for( i = 0; i < SND_NB_BUFFERS; i++ )
									if ( Header[ i ].dwFlags & WHDR_PREPARED )
										waveOutUnprepareHeader( hWave, &Header[ i ], sizeof( WAVEHDR ) );
								do {
									waveOutReset( hWave );
								}
								while( waveOutClose( hWave ) == WAVERR_STILLPLAYING );
							}
						}
						DeleteObject( f );
					}
					DeleteDC( DcBitmap );
				}
				DeleteObject( bm );
			}
			ReleaseDC( win, DeviceContext );
		}
		UnregisterClass( wcex.lpszClassName, hInstance );
	}
	return 0;
}
